@@ -151,9 +151,10 @@ class IJobDefinition(metaclass=ABCMeta):
         The provider this job definition belongs to
 
         .. note::
-            Technically this still can be None (a provider-less job may exist) but
-            it can only happen in testing. This mode is discouraged and will be eventually
-            forbidden. All job definition units must belong to a provider.
+            Technically this still can be None (a provider-less job may exist)
+            but it can only happen in testing. This mode is discouraged and
+            will be eventually forbidden. All job definition units must belong
+            to a provider.
         """
 
     @abstractproperty
@@ -213,17 +214,9 @@ class IJobResult(metaclass=ABCMeta):
     # A temporary state before the user decides on the outcome of a manual
     # job or any other job that requires manual verification
     OUTCOME_UNDECIDED = 'undecided'
-
-    # List of all valid values of OUTCOME_xxx
-    ALL_OUTCOME_LIST = [
-        OUTCOME_NONE,
-        OUTCOME_PASS,
-        OUTCOME_FAIL,
-        OUTCOME_SKIP,
-        OUTCOME_NOT_SUPPORTED,
-        OUTCOME_NOT_IMPLEMENTED,
-        OUTCOME_UNDECIDED,
-    ]
+    # A kind of failed that indicates the underlying test misbehaved. Currently
+    # it is only used when the test program is killed by a signal.
+    OUTCOME_CRASH = 'crash'
 
     @abstractproperty
     def outcome(self):
@@ -583,6 +576,13 @@ class IJobRunnerUI(metaclass=ABCMeta):
             cmd of the selected action or None
         """
 
+    @abstractmethod
+    def noreturn_job(self):
+        """
+        Method called when job that is about to run has 'noreturn' flag
+        (plainbox will suspend operation after running that job).
+        """
+
 
 class IUserInterfaceIO(metaclass=ABCMeta):
     """
@@ -678,7 +678,7 @@ class IProviderBackend1(metaclass=ABCMeta):
         """
 
 
-class IProvider1(metaclass=ABCMeta):
+class IProvider1(IProviderBackend1):
     """
     Provider for the current type of tests
 
@@ -739,59 +739,60 @@ class IProvider1(metaclass=ABCMeta):
         The value is applicable as argument bindtextdomain()
         """
 
-    @abstractmethod
-    def get_builtin_jobs(self):
+    @abstractproperty
+    def unit_list(self):
         """
-        Load and parse all of the job definitions of this provider.
-
-        :returns:
-            A sorted list of JobDefinition objects
-        :raises RFC822SyntaxError:
-            if any of the loaded files was not valid RFC822
-        :raises IOError, OSError:
-            if there were any problems accessing files or directories.
-            Note that OSError is silently ignored when the `jobs_dir`
-            directory is missing.
-
-        ..note::
-            This method should not be used anymore. Consider transitioning your
-            code to :meth:`load_all_jobs()` which is more reliable.
+        List of loaded units.
         """
 
-    @abstractmethod
-    def load_all_jobs(self):
+    @abstractproperty
+    def job_list(self):
         """
-        Load and parse all of the job definitions of this provider.
-
-        Unlike :meth:`get_builtin_jobs()` this method does not stop after the
-        first problem encountered and instead collects all of the problems into
-        a list which is returned alongside the job list.
-
-        :returns:
-            Pair (job_list, problem_list) where each job_list is a sorted list
-            of JobDefinition objects and each item from problem_list is an
-            exception.
+        List of loaded job definition units.
         """
 
-    @abstractmethod
-    def get_builtin_whitelists(self):
+    @abstractproperty
+    def executable_list(self):
         """
-        Load all the built-in whitelists and return them
+        List of all the executables
         """
 
-    def get_all_executables(self):
+    @abstractproperty
+    def whitelist_list(self):
         """
-        Discover and return all executables offered by this provider
+        List of loaded whitelists.
+
+        .. warning::
+            :class:`WhiteList` is currently deprecated. You should never need
+            to access them in any new code.  They are entirely replaced by
+            :class:`TestPlan`. This property is provided for completeness and
+            it will be **removed** once whitelists classes are no longer used.
         """
 
-    def get_units(self):
+    @abstractproperty
+    def problem_list(self):
+        """
+        list of problems encountered by the loading process
         """
-        Get all units.
 
-        :returns:
-            Pair (unit_list, problem_list) where unit_list is a unsorted list
-            of units, as they showed up in subsequent unit definition files and
-            each item from problem_list is an exception.
+    @abstractproperty
+    def id_map(self):
+        """
+        A mapping from unit identifier to list of units with that identifier.
+
+        .. note::
+            Typically the list will be one element long but invalid providers
+            may break that guarantee. Code defensively if you can.
+        """
+
+    @abstractproperty
+    def path_map(self):
+        """
+        A mapping from filename path to a list of units stored in that file.
+
+        .. note::
+            For ``.pxu`` files this will enumerate all units stored there. For
+            other things it will typically be just the FileUnit.
         """
 
 
@@ -869,12 +870,14 @@ class IExecutionController(metaclass=ABCMeta):
     """
 
     @abstractmethod
-    def execute_job(self, job, config, session_dir, extcmd_popen):
+    def execute_job(self, job, job_state, config, session_dir, extcmd_popen):
         """
         Execute the specified job using the specified subprocess-like object
 
         :param job:
             The JobDefinition to execute
+        :param job_state:
+            The JobState associated to the job to execute.
         :param config:
             A PlainBoxConfig instance which can be used to load missing
             environment definitions that apply to all jobs. It is used to
@@ -998,6 +1001,17 @@ class ISessionStateExporter(metaclass=ABCMeta):
     user interface from becoming annoying.
     """
 
+    @abstractmethod
+    def __init__(self, option_list=None, exporter_unit=None):
+        """
+        Initialize the session exporter object
+
+        :param option_list:
+            a list of option to enable
+        :param exporter_unit:
+            an ExporterUnit object
+        """
+
     @abstractmethod
     # @classproperty -- this is a class-level property
     def supported_option_list(cls):
@@ -1006,7 +1020,7 @@ class ISessionStateExporter(metaclass=ABCMeta):
         """
 
     @abstractmethod
-    def get_session_data_subset(self, session):
+    def get_session_data_subset(self, session_manager):
         """
         Compute a subset of session data.
 
@@ -1035,6 +1049,15 @@ class ISessionStateExporter(metaclass=ABCMeta):
         """
         # TODO: Add a way for the stream to be binary as well.
 
+    @abstractmethod
+    def dump_from_session_manager(self, session_manager, stream):
+        """
+        Dump session information pulled from session manager to stream.
+
+        This method takes session manager instance, extracts session
+        information from it, and dumps it to a stream.
+        """
+
 
 class ISessionStateTransport(metaclass=ABCMeta):
     """
