@@ -33,6 +33,8 @@ from plainbox.i18n import gettext as _
 from plainbox.impl.result import MemoryJobResult
 from plainbox.impl.secure import config
 from plainbox.impl.secure.qualifiers import select_jobs
+from plainbox.impl.session import SessionManager
+from plainbox.impl.session.jobs import InhibitionCause
 
 
 # Deprecated, use plainbox.impl.secure.qualifiers.select_jobs() instead
@@ -52,7 +54,7 @@ def get_whitelist_by_name(provider_list, desired_whitelist):
     providers
     """
     for provider in provider_list:
-        for whitelist in provider.get_builtin_whitelists():
+        for whitelist in provider.whitelist_list:
             if whitelist.name == desired_whitelist:
                 return whitelist
     else:
@@ -69,7 +71,7 @@ def run_job_if_possible(session, runner, config, job, update=True):
     """
     job_state = session.job_state_map[job.id]
     if job_state.can_start():
-        job_result = runner.run_job(job, config)
+        job_result = runner.run_job(job, job_state, config)
     else:
         # Set the outcome of jobs that cannot start to
         # OUTCOME_NOT_SUPPORTED _except_ if any of the inhibitors point to
@@ -77,7 +79,7 @@ def run_job_if_possible(session, runner, config, job, update=True):
         # that outcome. This makes 'skip' stronger than 'not-supported'
         outcome = IJobResult.OUTCOME_NOT_SUPPORTED
         for inhibitor in job_state.readiness_inhibitor_list:
-            if inhibitor.cause != inhibitor.FAILED_DEP:
+            if inhibitor.cause != InhibitionCause.FAILED_DEP:
                 continue
             related_job_state = session.job_state_map[
                 inhibitor.related_job.id]
@@ -113,3 +115,19 @@ class PlainBoxConfig(config.Config):
         filename_list = [
             '/etc/xdg/plainbox.conf',
             os.path.expanduser('~/.config/plainbox.conf')]
+
+
+def get_all_exporter_names():
+    """
+    Get the identifiers (names) of all the supported session state exporters.
+
+    :returns:
+        A list of session exporter names (identifiers) available from all the
+        providers.
+
+    This function creates a temporary session associated with the local
+    device and adds all of the available providers to it. Finally, it returns
+    the list of exporter names. The session is transparently destroyed.
+    """
+    with SessionManager.get_throwaway_manager() as manager:
+        return list(manager.exporter_map.keys())
