@@ -1,6 +1,6 @@
 # This file is part of Checkbox.
 #
-# Copyright 2012-2014 Canonical Ltd.
+# Copyright 2012-2015 Canonical Ltd.
 # Written by:
 #   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
 #
@@ -26,6 +26,7 @@
 
 from shutil import copyfileobj
 import collections
+import datetime
 import io
 import itertools
 import logging
@@ -42,7 +43,8 @@ from plainbox.impl.color import Colorizer
 from plainbox.impl.commands.inv_checkbox import CheckBoxInvocationMixIn
 from plainbox.impl.depmgr import DependencyDuplicateError
 from plainbox.impl.exporter import ByteStringStreamTranslator
-from plainbox.impl.exporter import get_all_exporters
+from plainbox.impl.exporter.text import TextSessionStateExporter
+from plainbox.impl.result import JobResultBuilder
 from plainbox.impl.result import MemoryJobResult
 from plainbox.impl.result import tr_outcome
 from plainbox.impl.runner import JobRunner
@@ -148,6 +150,9 @@ class SilentUI(IJobRunnerUI):
     def pick_action_cmd(self, action_list, prompt=None):
         pass
 
+    def noreturn_job(self):
+        pass
+
 
 class NormalUI(IJobRunnerUI):
 
@@ -194,8 +199,9 @@ class NormalUI(IJobRunnerUI):
             'stdout': sys.stdout,
             'stderr': sys.stderr
         }[stream_name]
-        print(self.C.BLACK(line.decode("UTF-8", "ignore").rstrip('\n')),
-              file=stream)
+        print(self.C.BLACK(line.decode("UTF-8", "ignore")),
+              end='', file=stream)
+        stream.flush()
 
     def finished_executing_program(self, returncode):
         if self.show_cmd_output:
@@ -210,16 +216,19 @@ class NormalUI(IJobRunnerUI):
 
     def notify_about_purpose(self, job):
         if job.tr_purpose() is not None:
+            print(self.C.CYAN(_("Purpose:")))
             print(self.C.CYAN(job.tr_purpose()))
         else:
             self.notify_about_description(job)
 
     def notify_about_steps(self, job):
         if job.tr_steps() is not None:
+            print(self.C.CYAN(_("Steps:")))
             print(self.C.CYAN(job.tr_steps()))
 
     def notify_about_verification(self, job):
         if job.tr_verification() is not None:
+            print(self.C.CYAN(_("Verification:")))
             print(self.C.CYAN(job.tr_verification()))
 
     def job_cannot_start(self, job, job_state, result):
@@ -236,6 +245,10 @@ class NormalUI(IJobRunnerUI):
     def pick_action_cmd(self, action_list, prompt=None):
         return ActionUI(action_list, prompt, self._color).run()
 
+    def noreturn_job(self):
+        print(self.C.RED(_("Waiting for the system to shut down or"
+                           " reboot...")))
+
 
 class ReRunJob(Exception):
     """
@@ -336,13 +349,7 @@ class RunInvocation(CheckBoxInvocationMixIn):
 
     def run(self):
         ns = self.ns
-        if ns.output_format == _('?'):
-            self._print_output_format_list(ns)
-            return 0
-        elif ns.output_options == _('?'):
-            self._print_output_option_list(ns)
-            return 0
-        elif ns.transport == _('?'):
+        if ns.transport == _('?'):
             self._print_transport_list(ns)
             return 0
         else:
@@ -352,15 +359,27 @@ class RunInvocation(CheckBoxInvocationMixIn):
         """
         Proceed through normal set of steps that are required to runs jobs
         """
-        # Create exporter and transport early so that we can handle bugs
-        # before starting the session.
-        self.create_exporter()
+        # Create transport early so that we can handle bugs before starting the
+        # session.
         self.create_transport()
         if self.is_interactive:
             resumed = self.maybe_resume_session()
         else:
             self.create_manager(None)
             resumed = False
+        if self.ns.output_options == _('?'):
+            self._print_output_option_list(self.ns)
+            return 0
+        elif self.ns.output_format == _('?'):
+            self._print_output_format_list(self.ns)
+            return 0
+        if self.ns.output_format not in self.manager.exporter_map:
+            print(_("invalid choice: '{}'".format(self.ns.output_format)))
+            self._print_output_format_list(self.ns)
+            return 1
+        # Create exporter after we get a session to query the manager and get
+        # all exporter units
+        self.create_exporter()
         # Create the job runner so that we can do stuff
         self.create_runner()
         # Set the effective category for each job
@@ -453,14 +472,15 @@ class RunInvocation(CheckBoxInvocationMixIn):
         return resumed
 
     def _print_output_format_list(self, ns):
-        print(_("Available output formats: {}").format(
-            ', '.join(get_all_exporters())))
+        print(_("Available output formats:"))
+        for id, exporter in self.manager.exporter_map.items():
+            print("{} - {}".format(id, exporter.summary))
 
     def _print_output_option_list(self, ns):
         print(_("Each format may support a different set of options"))
-        for name, exporter_cls in get_all_exporters().items():
+        for name, exporter in self.manager.exporter_map.items():
             print("{}: {}".format(
-                name, ", ".join(exporter_cls.supported_option_list)))
+                name, ", ".join(exporter.exporter_cls.supported_option_list)))
 
     def _print_transport_list(self, ns):
         print(_("Available transports: {}").format(
@@ -538,19 +558,12 @@ class RunInvocation(CheckBoxInvocationMixIn):
 
         This sets the attr:`_exporter`.
         """
-        exporter_cls = get_all_exporters()[self.ns.output_format]
         if self.ns.output_options:
             option_list = self.ns.output_options.split(',')
         else:
             option_list = None
-        try:
-            if self.ns.output_format == 'text':
-                # Hacky way to pass color flag down to the text exporter
-                self._exporter = exporter_cls(option_list, self.ns.color)
-            else:
-                self._exporter = exporter_cls(option_list)
-        except ValueError as exc:
-            raise SystemExit(str(exc))
+        self._exporter = self.manager.create_exporter(
+            self.ns.output_format, option_list)
 
     def create_transport(self):
         """
@@ -587,8 +600,7 @@ class RunInvocation(CheckBoxInvocationMixIn):
             If the session cannot be resumed for any reason.
         """
         all_units = list(
-            itertools.chain(*[
-                p.get_units()[0] for p in self.provider_list]))
+            itertools.chain(*[p.unit_list for p in self.provider_list]))
         try:
             if storage is not None:
                 self._manager = SessionManager.load_session(all_units, storage)
@@ -654,9 +666,8 @@ class RunInvocation(CheckBoxInvocationMixIn):
         if test_plan_id is None:
             return
         for provider in self.provider_list:
-            for unit in provider.get_units()[0]:
-                if (unit.Meta.name == 'test plan'
-                        and unit.id == test_plan_id):
+            for unit in provider.id_map[test_plan_id]:
+                if unit.Meta.name == 'test plan':
                     return unit
 
     def set_effective_categories(self):
@@ -677,6 +688,14 @@ class RunInvocation(CheckBoxInvocationMixIn):
             self.ns, self.state.job_list)
         print(self.C.header(_("Analyzing Jobs")))
         self._update_desired_job_list(desired_job_list)
+        # Search each provider for the desired test plan
+        if self.ns.test_plan is not None:
+            # TODO: add high-level unit lookup functions
+            for provider in self.provider_list:
+                for unit in provider.id_map.get(self.ns.test_plan, ()):
+                    if unit.Meta.name == 'test plan':
+                        self.manager.test_plans = (unit,)
+                        break
 
     def maybe_warm_up_authentication(self):
         """
@@ -697,19 +716,34 @@ class RunInvocation(CheckBoxInvocationMixIn):
         self._backtrack_and_run_missing = True
         while self._backtrack_and_run_missing:
             self._backtrack_and_run_missing = False
+            jobs_to_run = []
+            estimated_time = 0
+            # gather jobs that we want to run and skip the jobs that already
+            # have result, this is only needed when we run over the list of
+            # jobs again, after discovering new jobs via the local job output
             for job in self.state.run_list:
                 job_state = self.state.job_state_map[job.id]
-                # Skip jobs that already have result, this is only needed when
-                # we run over the list of jobs again, after discovering new
-                # jobs via the local job output
                 if job_state.result.outcome is None:
-                    self.run_single_job(job)
+                    jobs_to_run.append(job)
+                    if (job.estimated_duration is not None
+                            and estimated_time is not None):
+                        estimated_time += job.estimated_duration
+                    else:
+                        estimated_time = None
+            for job_no, job in enumerate(jobs_to_run, start=1):
+                print(self.C.header(
+                    _('Running job {} / {}. Estimated time left: {}').format(
+                        job_no, len(jobs_to_run),
+                        seconds_to_human_duration(max(0, estimated_time))
+                        if estimated_time is not None else _("unknown")),
+                    fill='-'))
+                self.run_single_job(job)
+                if (job.estimated_duration is not None
+                        and estimated_time is not None):
+                    estimated_time -= job.estimated_duration
 
     def run_single_job(self, job):
-        job_start_time = time.time()
         self.run_single_job_with_ui(job, self.get_ui_for_job(job))
-        job_state = self.state.job_state_map[job.id]
-        job_state.result.execution_duration = time.time() - job_start_time
 
     def get_ui_for_job(self, job):
         if self.ns.dont_suppress_output is False and job.plugin in (
@@ -719,6 +753,7 @@ class RunInvocation(CheckBoxInvocationMixIn):
             return NormalUI(self.C.c, show_cmd_output=True)
 
     def run_single_job_with_ui(self, job, ui):
+        job_start_time = time.time()
         job_state = self.state.job_state_map[job.id]
         ui.considering_job(job, job_state)
         if job_state.can_start():
@@ -726,31 +761,43 @@ class RunInvocation(CheckBoxInvocationMixIn):
             self.metadata.running_job_name = job.id
             self.manager.checkpoint()
             ui.started_running(job, job_state)
-            job_result = self._run_single_job_with_ui_loop(job, ui)
+            result_builder = self._run_single_job_with_ui_loop(
+                job, job_state, ui)
+            assert result_builder is not None
+            result_builder.execution_duration = time.time() - job_start_time
+            job_result = result_builder.get_result()
             self.metadata.running_job_name = None
             self.manager.checkpoint()
             ui.finished_running(job, job_state, job_result)
         else:
-            job_result = MemoryJobResult({
-                'outcome': IJobResult.OUTCOME_NOT_SUPPORTED,
-                'comments': job_state.get_readiness_description()
-            })
+            result_builder = JobResultBuilder(
+                outcome=IJobResult.OUTCOME_NOT_SUPPORTED,
+                comments=job_state.get_readiness_description(),
+                execution_duration=time.time() - job_start_time)
+            job_result = result_builder.get_result()
             ui.job_cannot_start(job, job_state, job_result)
-        if job_result is not None:
-            self.state.update_job_result(job, job_result)
+        self.state.update_job_result(job, job_result)
         ui.finished(job, job_state, job_result)
 
-    def _run_single_job_with_ui_loop(self, job, ui):
+    def _run_single_job_with_ui_loop(self, job, job_state, ui):
         comments = ""
         while True:
             if job.plugin in ('user-interact', 'user-interact-verify',
                               'user-verify', 'manual'):
                 ui.notify_about_purpose(job)
-                if self.is_interactive:
-                    cmd = ui.wait_for_interaction_prompt(job)
+                if (self.is_interactive and
+                        job.plugin in ('user-interact',
+                                       'user-interact-verify',
+                                       'manual')):
+                    ui.notify_about_steps(job)
+                    if job.plugin == 'manual':
+                        cmd = 'run'
+                    else :
+                        cmd = ui.wait_for_interaction_prompt(job)
                     if cmd == 'run' or cmd is None:
-                        ui.notify_about_steps(job)
-                        job_result = self.runner.run_job(job, self.config, ui)
+                        result_builder = self.runner.run_job(
+                            job, job_state, self.config, ui
+                        ).get_builder()
                     elif cmd == 'comment':
                         new_comment = input(self.C.BLUE(
                             _('Please enter your comments:') + '\n'))
@@ -758,38 +805,42 @@ class RunInvocation(CheckBoxInvocationMixIn):
                             comments += new_comment + '\n'
                         continue
                     elif cmd == 'skip':
-                        job_result = MemoryJobResult({
-                            'outcome': IJobResult.OUTCOME_SKIP,
-                            'comments': _("Explicitly skipped before"
-                                          " execution")
-                        })
+                        result_builder = JobResultBuilder(
+                            outcome=IJobResult.OUTCOME_SKIP,
+                            comments=_("Explicitly skipped before"
+                                       " execution"))
                         if comments != "":
-                            job_result.comments = comments
+                            result_builder.comments = comments
                         break
                     elif cmd == 'quit':
                         raise SystemExit()
                 else:
-                    job_result = self.runner.run_job(job, self.config, ui)
+                    result_builder = self.runner.run_job(
+                        job, job_state, self.config, ui
+                    ).get_builder()
             else:
-                job_result = self.runner.run_job(job, self.config, ui)
+                if 'noreturn' in job.get_flag_set():
+                    ui.noreturn_job()
+                result_builder = self.runner.run_job(
+                    job, job_state, self.config, ui
+                ).get_builder()
             if (self.is_interactive and
-                    job_result.outcome == IJobResult.OUTCOME_UNDECIDED):
+                    result_builder.outcome == IJobResult.OUTCOME_UNDECIDED):
                 try:
                     if comments != "":
-                        job_result.comments = comments
+                        result_builder.comments = comments
                     ui.notify_about_verification(job)
-                    job_result = self._interaction_callback(
-                        self.runner, job, job_result, self.config)
+                    self._interaction_callback(
+                        self.runner, job, result_builder, self.config)
                 except ReRunJob:
                     continue
             break
-        return job_result
+        return result_builder
 
     def export_and_send_results(self):
         # Get a stream with exported session data.
         exported_stream = io.BytesIO()
-        data_subset = self.exporter.get_session_data_subset(self.state)
-        self.exporter.dump(data_subset, exported_stream)
+        self.exporter.dump_from_session_manager(self.manager, exported_stream)
         exported_stream.seek(0)  # Need to rewind the file, puagh
         # Write the stream to file if requested
         self._save_results(self.ns.output_file, exported_stream)
@@ -819,8 +870,9 @@ class RunInvocation(CheckBoxInvocationMixIn):
     def _pick_action_cmd(self, action_list, prompt=None):
         return ActionUI(action_list, prompt, self._color).run()
 
-    def _interaction_callback(self, runner, job, result, config,
+    def _interaction_callback(self, runner, job, result_builder, config,
                               prompt=None, allowed_outcome=None):
+        result = result_builder.get_result()
         if prompt is None:
             prompt = _("Select an outcome or an action: ")
         if allowed_outcome is None:
@@ -858,7 +910,8 @@ class RunInvocation(CheckBoxInvocationMixIn):
                     tr_outcome(suggested_outcome)), 'set-suggested'))
         while result.outcome not in allowed_outcome:
             print(_("Please decide what to do next:"))
-            print("  " + _("outcome") + ": {0}".format(self.C.result(result)))
+            print("  " + _("outcome") + ": {0}".format(
+                self.C.result(result)))
             if result.comments is None:
                 print("  " + _("comments") + ": {0}".format(
                     C_("none comment", "none")))
@@ -867,23 +920,21 @@ class RunInvocation(CheckBoxInvocationMixIn):
                     self.C.CYAN(result.comments, bright=False)))
             cmd = self._pick_action_cmd(allowed_actions)
             if cmd == 'set-pass':
-                result.outcome = IJobResult.OUTCOME_PASS
+                result_builder.outcome = IJobResult.OUTCOME_PASS
             elif cmd == 'set-fail':
-                result.outcome = IJobResult.OUTCOME_FAIL
+                result_builder.outcome = IJobResult.OUTCOME_FAIL
             elif cmd == 'set-skip' or cmd is None:
-                result.outcome = IJobResult.OUTCOME_SKIP
+                result_builder.outcome = IJobResult.OUTCOME_SKIP
             elif cmd == 'set-suggested':
-                result.outcome = suggested_outcome
+                result_builder.outcome = suggested_outcome
             elif cmd == 'set-comments':
-                if result.comments is None:
-                    result.comments = ""
                 new_comment = input(self.C.BLUE(
                     _('Please enter your comments:') + '\n'))
                 if new_comment:
-                    result.comments += new_comment + '\n'
+                    result_builder.add_comment(new_comment)
             elif cmd == 're-run':
                 raise ReRunJob
-        return result
+            result = result_builder.get_result()
 
     def _update_desired_job_list(self, desired_job_list):
         problem_list = self.state.update_desired_job_list(desired_job_list)
@@ -896,8 +947,8 @@ class RunInvocation(CheckBoxInvocationMixIn):
 
     def print_estimated_duration(self):
         print(self.C.header(_("Session Statistics")))
-        print(_("This session is about {0:.2f}% complete").format(
-            self.get_completion_ratio() * 100))
+        print(_("This session is about {0:.2f}{percent} complete").format(
+            self.get_completion_ratio() * 100, percent='%'))
         (estimated_duration_auto,
          estimated_duration_manual) = self.state.get_estimated_duration()
         if estimated_duration_auto:
@@ -960,3 +1011,9 @@ class RunInvocation(CheckBoxInvocationMixIn):
             job_state.effective_category_id = (
                 self._test_plan.get_effective_category(job))
         self._backtrack_and_run_missing = True
+
+
+def seconds_to_human_duration(seconds: float) -> str:
+    """ Convert ammount of seconds to human readable duration string. """
+    delta = datetime.timedelta(seconds=round(seconds))
+    return str(delta)
