@@ -35,11 +35,13 @@ circumstances.
 
 import abc
 import contextlib
+import errno
 try:
     import grp
 except ImportError:
     grp = None
 import itertools
+import json
 import logging
 import os
 try:
@@ -54,6 +56,7 @@ from plainbox.abc import IExecutionController
 from plainbox.abc import IJobResult
 from plainbox.abc import ISessionStateController
 from plainbox.i18n import gettext as _
+from plainbox.impl import get_plainbox_dir
 from plainbox.impl.depmgr import DependencyDuplicateError
 from plainbox.impl.depmgr import DependencyMissingError
 from plainbox.impl.resource import ExpressionCannotEvaluateError
@@ -65,11 +68,12 @@ from plainbox.impl.secure.origin import JobOutputTextSource
 from plainbox.impl.secure.providers.v1 import Provider1
 from plainbox.impl.secure.rfc822 import RFC822SyntaxError
 from plainbox.impl.secure.rfc822 import gen_rfc822_records
+from plainbox.impl.session.jobs import InhibitionCause
 from plainbox.impl.session.jobs import JobReadinessInhibitor
-from plainbox.impl.signal import Signal
 from plainbox.impl.unit.job import JobDefinition
 from plainbox.impl.unit.template import TemplateUnit
 from plainbox.impl.validation import ValidationError
+from plainbox.vendor import morris
 from plainbox.vendor import extcmd
 
 __all__ = [
@@ -166,7 +170,7 @@ class CheckBoxSessionStateController(ISessionStateController):
                 # or simply be on the run_list but just was not executed
                 # yet).
                 inhibitor = JobReadinessInhibitor(
-                    cause=JobReadinessInhibitor.PENDING_RESOURCE,
+                    cause=InhibitionCause.PENDING_RESOURCE,
                     related_job=related_job,
                     related_expression=exc.expression)
                 inhibitors.append(inhibitor)
@@ -180,7 +184,7 @@ class CheckBoxSessionStateController(ISessionStateController):
                 # non-True value. This typically indicates a missing
                 # software package or necessary hardware.
                 inhibitor = JobReadinessInhibitor(
-                    cause=JobReadinessInhibitor.FAILED_RESOURCE,
+                    cause=InhibitionCause.FAILED_RESOURCE,
                     related_job=related_job,
                     related_expression=exc.expression)
                 inhibitors.append(inhibitor)
@@ -191,7 +195,7 @@ class CheckBoxSessionStateController(ISessionStateController):
             # PENDING_DEP inhibitor.
             if dep_job_state.result.outcome == IJobResult.OUTCOME_NONE:
                 inhibitor = JobReadinessInhibitor(
-                    cause=JobReadinessInhibitor.PENDING_DEP,
+                    cause=InhibitionCause.PENDING_DEP,
                     related_job=dep_job_state.job)
                 inhibitors.append(inhibitor)
             # If the dependency is anything but successful add the
@@ -201,7 +205,7 @@ class CheckBoxSessionStateController(ISessionStateController):
             # cannot run.
             elif dep_job_state.result.outcome != IJobResult.OUTCOME_PASS:
                 inhibitor = JobReadinessInhibitor(
-                    cause=JobReadinessInhibitor.FAILED_DEP,
+                    cause=InhibitionCause.FAILED_DEP,
                     related_job=dep_job_state.job)
                 inhibitors.append(inhibitor)
         return inhibitors
@@ -297,6 +301,10 @@ class CheckBoxSessionStateController(ISessionStateController):
                             new_unit, exc)
                     else:
                         session_state.add_unit(new_unit)
+                        if new_unit.Meta.name == 'job':
+                            job_state = session_state.job_state_map[
+                                new_unit.id]
+                            job_state.via_job = job
 
     def _process_local_result(self, session_state, job, result):
         """
@@ -307,6 +315,10 @@ class CheckBoxSessionStateController(ISessionStateController):
         # name, not a new list of jobs)
         new_job_list = []
         for record in gen_rfc822_records_from_io_log(job, result):
+            # Skip non-job units as the code below is wired to work with jobs
+            # Fixes: https://bugs.launchpad.net/plainbox/+bug/1443228
+            if record.data.get('unit', 'job') != 'job':
+                continue
             new_job = job.create_child_job_from_record(record)
             try:
                 new_job.validate()
@@ -333,12 +345,68 @@ class CheckBoxSessionStateController(ISessionStateController):
                       " discarded"),
                     job.id, exc.duplicate_job.id, exc.job.id, exc.job.origin)
             else:
-                # Patch the origin of the existing job so that it traces
-                # back to the job that "generated" it again. This is
-                # basically required to get __category__ jobs to associate
-                # themselves with their children.
-                if added_job is not new_job:
-                    added_job.update_origin(new_job.origin)
+                # Set the via_job attribute of the newly added job to point to
+                # the generator job. This way it can be traced back to the old
+                # __category__-style local jobs or to their corresponding
+                # generator job in general.
+                #
+                # NOTE: this is the only place where we assign via_job so as
+                # long as that holds true, we can detect and break via cycles.
+                #
+                # Via cycles occur whenever a job can reach itself again
+                # through via associations. Note that the chain may be longer
+                # than one link (A->A) and can include other jobs in the list
+                # (A->B->C->A)
+                #
+                # To detect a cycle we must iterate back the via chain (and we
+                # must do it here because we have access to job_state_map that
+                # allows this iteration to happen) and break the cycle if we
+                # see the job being added.
+                job_state_map = session_state.job_state_map
+                job_state_map[added_job.id].via_job = job
+                via_cycle = get_via_cycle(job_state_map, added_job)
+                if via_cycle:
+                    logger.warning(_("Automatically breaking via-cycle: %s"),
+                                    ' -> '.join(str(cycle_job)
+                                                for cycle_job in via_cycle))
+                    job_state_map[added_job.id].via_job = None
+
+
+def get_via_cycle(job_state_map, job):
+    """
+    Find a possible cycle including via_job.
+
+    :param job_state_map:
+        A dictionary mapping job.id to a JobState object.
+    :param via_job:
+        Any job, start of a hypothetical via job cycle.
+    :raises KeyError:
+        If any of the encountered jobs are not present in job_state_map.
+    :return:
+        A list of jobs that represent the cycle or an empty tuple if no cycle
+        is present. The list has the property that item[0] is item[-1]
+
+    A via cycle occurs if *job* is reachable through the *via_job* by
+    recursively following via_job connection until via_job becomes None.
+    """
+    cycle = []
+    seen = set()
+    while job is not None:
+        cycle.append(job)
+        seen.add(job)
+        next_job = job_state_map[job.id].via_job
+        if next_job in seen:
+            break
+        job = next_job
+    else:
+        return ()
+    # Discard all the jobs leading to the cycle.
+    # cycle = cycle[cycle.index(next_job):]
+    # This is just to hold the promise of the return value so
+    # that processing is easier for the caller.
+    cycle.append(next_job)
+    # assert cycle[0] is cycle[-1]
+    return cycle
 
 
 def gen_rfc822_records_from_io_log(job, result):
@@ -386,7 +454,7 @@ class SymLinkNest:
         :param provider:
             A Provider1 instance
         """
-        for filename in provider.get_all_executables():
+        for filename in provider.executable_list:
             self.add_executable(filename)
 
     def add_executable(self, filename):
@@ -428,12 +496,14 @@ class CheckBoxExecutionController(IExecutionController):
         """
         self._provider_list = provider_list
 
-    def execute_job(self, job, config, session_dir, extcmd_popen):
+    def execute_job(self, job, job_state, config, session_dir, extcmd_popen):
         """
         Execute the specified job using the specified subprocess-like object
 
         :param job:
             The JobDefinition to execute
+        :param job_state:
+            The JobState associated to the job to execute.
         :param config:
             A PlainBoxConfig instance which can be used to load missing
             environment definitions that apply to all jobs. It is used to
@@ -458,14 +528,17 @@ class CheckBoxExecutionController(IExecutionController):
             # Get the command and the environment.
             # of this execution controller
             cmd = self.get_execution_command(
-                job, config, session_dir, nest_dir)
+                job, job_state, config, session_dir, nest_dir)
             env = self.get_execution_environment(
-                job, config, session_dir, nest_dir)
+                job, job_state, config, session_dir, nest_dir)
             with self.temporary_cwd(job, config) as cwd_dir:
                 # run the command
                 logger.debug(_("job[%s] executing %r with env %r in cwd %r"),
                              job.id, cmd, env, cwd_dir)
-                return extcmd_popen.call(cmd, env=env, cwd=cwd_dir)
+                return_code = extcmd_popen.call(cmd, env=env, cwd=cwd_dir)
+                if 'noreturn' in job.get_flag_set():
+                    self._halt()
+                return return_code
 
     @contextlib.contextmanager
     def configured_filesystem(self, job, config):
@@ -544,7 +617,7 @@ class CheckBoxExecutionController(IExecutionController):
                 for filename in filenames)
         return leftovers
 
-    @Signal.define
+    @morris.signal
     def on_leftover_files(self, job, config, cwd_dir, leftovers):
         """
         Handle any files left over by the execution of a job definition.
@@ -594,12 +667,15 @@ class CheckBoxExecutionController(IExecutionController):
         """
 
     @abc.abstractmethod
-    def get_execution_command(self, job, config, session_dir, nest_dir):
+    def get_execution_command(self, job, job_state, config, session_dir,
+                              nest_dir):
         """
         Get the command to execute the specified job
 
         :param job:
             job definition with the command and environment definitions
+        :param job_state:
+            The JobState associated to the job to execute.
         :param config:
             A PlainBoxConfig instance which can be used to load missing
             environment definitions that apply to all jobs. It is used to
@@ -619,12 +695,15 @@ class CheckBoxExecutionController(IExecutionController):
             List of command arguments
         """
 
-    def get_execution_environment(self, job, config, session_dir, nest_dir):
+    def get_execution_environment(self, job, job_state, config, session_dir,
+                                  nest_dir):
         """
         Get the environment required to execute the specified job:
 
         :param job:
             job definition with the command and environment definitions
+        :param job_state:
+            The JobState associated to the job to execute.
         :param config:
             A PlainBoxConfig instance which can be used to load missing
             environment definitions that apply to all jobs. It is used to
@@ -719,18 +798,31 @@ class CheckBoxExecutionController(IExecutionController):
             None
         """
 
+    def _halt(self):
+        """
+        Suspend operation until signal is received
+
+        This function is useful when plainbox should stop execution and wait
+        for external process to kill it.
+        """
+        import signal
+        signal.pause()
+
 
 class UserJobExecutionController(CheckBoxExecutionController):
     """
     An execution controller that works for jobs invoked as the current user.
     """
 
-    def get_execution_command(self, job, config, session_dir, nest_dir):
+    def get_execution_command(self, job, job_state, config, session_dir,
+                              nest_dir):
         """
         Get the command to execute the specified job
 
         :param job:
             job definition with the command and environment definitions
+        :param job_state:
+            The JobState associated to the job to execute.
         :param config:
             A PlainBoxConfig instance which can be used to load missing
             environment definitions that apply to all jobs. Ignored.
@@ -781,6 +873,9 @@ class UserJobExecutionController(CheckBoxExecutionController):
                 return -1
             return 1
         else:
+            # Windows jobs won't run on other platforms
+            if 'win32' in job.get_flag_set():
+                return -1
             if job.user is not None:
                 if os.getuid() == 0:
                     return 4
@@ -789,6 +884,165 @@ class UserJobExecutionController(CheckBoxExecutionController):
             return 1
 
 
+class QmlJobExecutionController(CheckBoxExecutionController):
+    """
+    An execution controller that is able to run jobs in QML shell.
+    """
+
+    QML_SHELL_PATH = os.path.join(get_plainbox_dir(), 'data', 'qml-shell',
+                                  'plainbox_qml_shell.qml')
+
+    def get_execution_command(self, job, job_state, config, session_dir,
+                              nest_dir, shell_out_fd, shell_in_fd):
+        """
+        Get the command to execute the specified job
+
+        :param job:
+            job definition with the command and environment definitions
+        :param job_state:
+            The JobState associated to the job to execute.
+        :param config:
+            A PlainBoxConfig instance which can be used to load missing
+            environment definitions that apply to all jobs. Ignored.
+        :param session_dir:
+            Base directory of the session this job will execute in.
+            This directory is used to co-locate some data that is unique to
+            this execution as well as data that is shared by all executions.
+            Ignored.
+        :param nest_dir:
+            A directory with a nest of symlinks to all executables required to
+            execute the specified job. Ingored.
+        :param shell_out_fd:
+            File descriptor number which is used to pipe through result object
+            from the qml shell to plainbox.
+        :param shell_in_fd:
+            File descriptor number which is used to pipe through test meta
+            information from plainbox to qml shell.
+        :returns:
+            List of command arguments
+
+        """
+        cmd = ['qmlscene', '--job', job.qml_file, '--fd-out', shell_out_fd,
+               '--fd-in', shell_in_fd, self.QML_SHELL_PATH]
+        return cmd
+
+    def get_checkbox_score(self, job):
+        """
+        Compute how applicable this controller is for the specified job.
+
+        :returns:
+            4 if the job is a qml job or -1 otherwise
+        """
+        if job.plugin == 'qml':
+            return 4
+        else:
+            return -1
+
+    def gen_job_repr(self, job):
+        """
+        Generate simplified job representation for use in qml shell
+        :returns:
+            dictionary with simplified job representation
+        """
+        logger.debug(_("Generating job repr for job: %r"), job)
+        return {
+            "id": job.id,
+            "summary": job.tr_summary(),
+            "description": job.tr_description(),
+        }
+
+    def execute_job(self, job, job_state, config, session_dir, extcmd_popen):
+        """
+        Execute the specified job using the specified subprocess-like object,
+        passing fd with opened pipe for qml-shell->plainbox communication.
+
+        :param job:
+            The JobDefinition to execute
+        :param config:
+            A PlainBoxConfig instance which can be used to load missing
+            environment definitions that apply to all jobs. It is used to
+            provide values for missing environment variables that are required
+            by the job (as expressed by the environ key in the job definition
+            file).
+        :param session_dir:
+            Base directory of the session this job will execute in.
+            This directory is used to co-locate some data that is unique to
+            this execution as well as data that is shared by all executions.
+        :param extcmd_popen:
+            A subprocess.Popen like object
+        :returns:
+            The return code of the command, as returned by subprocess.call()
+        """
+
+        class DuplexPipe:
+            """
+            Helper context creating two pipes, ensuring they are closed
+            properly
+            """
+            def __enter__(self):
+                self.a_read, self.b_write = os.pipe()
+                self.b_read, self.a_write = os.pipe()
+                return self.a_read, self.b_write, self.b_read, self.a_write
+
+            def __exit__(self, *args):
+                for pipe in (self.a_read, self.b_write,
+                             self.b_read, self.a_write):
+                    # typically those pipes are already closed; trying to
+                    # re-close them causes OSError (errno == 9) to be raised
+                    try:
+                        os.close(pipe)
+                    except OSError as exc:
+                        if exc.errno != errno.EBADF:
+                            raise
+        # CHECKBOX_DATA is where jobs can share output.
+        # It has to be an directory that scripts can assume exists.
+        if not os.path.isdir(self.get_CHECKBOX_DATA(session_dir)):
+            os.makedirs(self.get_CHECKBOX_DATA(session_dir))
+        # Setup the executable nest directory
+        with self.configured_filesystem(job, config) as nest_dir:
+            with DuplexPipe() as (plainbox_read, shell_write,
+                                  shell_read, plainbox_write):
+                # Get the command and the environment.
+                # of this execution controller
+                cmd = self.get_execution_command(
+                    job, job_state, config, session_dir, nest_dir,
+                    str(shell_write), str(shell_read))
+                env = self.get_execution_environment(
+                    job, job_state, config, session_dir, nest_dir)
+                with self.temporary_cwd(job, config) as cwd_dir:
+                    testing_shell_data = json.dumps({
+                        "job_repr": self.gen_job_repr(job),
+                        "session_dir": self.get_CHECKBOX_DATA(session_dir)
+                    })
+                    pipe_out = os.fdopen(plainbox_write, 'wt')
+                    pipe_out.write(testing_shell_data)
+                    pipe_out.close()
+                    # run the command
+                    logger.debug(_("job[%s] executing %r with"
+                                   "env %r in cwd %r"),
+                                 job.id, cmd, env, cwd_dir)
+                    ret = extcmd_popen.call(cmd, env=env, cwd=cwd_dir,
+                                            pass_fds=[shell_write, shell_read])
+                    os.close(shell_read)
+                    os.close(shell_write)
+                    pipe_in = os.fdopen(plainbox_read)
+                    res_object_json_string = pipe_in.read()
+                    pipe_in.close()
+                    if 'noreturn' in job.get_flag_set():
+                        self._halt()
+                    if ret != 0:
+                        return ret
+                    try:
+                        result = json.loads(res_object_json_string)
+                        if result['outcome'] == "pass":
+                            return 0
+                        else:
+                            return 1
+                    except ValueError:
+                        # qml-job did not print proper json object
+                        return 1
+
+
 class CheckBoxDifferentialExecutionController(CheckBoxExecutionController):
     """
     A CheckBoxExecutionController subclass that uses differential environment.
@@ -800,12 +1054,14 @@ class CheckBoxDifferentialExecutionController(CheckBoxExecutionController):
     """
 
     def get_differential_execution_environment(
-            self, job, config, session_dir, nest_dir):
+            self, job, job_state, config, session_dir, nest_dir):
         """
         Get the environment required to execute the specified job:
 
         :param job:
             job definition with the command and environment definitions
+        :param job_state:
+            The JobState associated to the job to execute.
         :param config:
             A PlainBoxConfig instance which can be used to load missing
             environment definitions that apply to all jobs. It is used to
@@ -832,7 +1088,7 @@ class CheckBoxDifferentialExecutionController(CheckBoxExecutionController):
         """
         base_env = os.environ
         target_env = super().get_execution_environment(
-            job, config, session_dir, nest_dir)
+            job, job_state, config, session_dir, nest_dir)
         return {
             key: value
             for key, value in target_env.items()
@@ -840,13 +1096,16 @@ class CheckBoxDifferentialExecutionController(CheckBoxExecutionController):
             or key in job.get_environ_settings()
         }
 
-    def get_execution_environment(self, job, config, session_dir, nest_dir):
+    def get_execution_environment(self, job, job_state, config, session_dir,
+                                  nest_dir):
         """
         Get the environment required to execute the specified job:
 
         :param job:
             job definition with the command and environment definitions.
             Ignored.
+        :param job_state:
+            The JobState associated to the job to execute. Ignored.
         :param config:
             A PlainBoxConfig instance which can be used to load missing
             environment definitions that apply to all jobs. Ignored.
@@ -895,12 +1154,15 @@ class RootViaPTL1ExecutionController(CheckBoxDifferentialExecutionController):
             result = exc.output
         self.is_supported = True if result.strip() == action_id else False
 
-    def get_execution_command(self, job, config, session_dir, nest_dir):
+    def get_execution_command(self, job, job_state, config, session_dir,
+                              nest_dir):
         """
         Get the command to invoke.
 
         :param job:
             job definition with the command and environment definitions
+        :param job_state:
+            The JobState associated to the job to execute.
         :param config:
             A PlainBoxConfig instance which can be used to load missing
             environment definitions that apply to all jobs. Passed to
@@ -925,16 +1187,18 @@ class RootViaPTL1ExecutionController(CheckBoxDifferentialExecutionController):
         # Run plainbox-trusted-launcher-1 as the required user
         cmd = ['pkexec', '--user', job.user, 'plainbox-trusted-launcher-1']
         # Run the specified generator job in the specified environment
-        if job.via is not None:
-            cmd += ['--generator', job.via]
+        if job_state.via_job is not None:
+            cmd += ['--generator', job_state.via_job.checksum]
             parent_env = self.get_differential_execution_environment(
-                job.origin.source.job, config, session_dir, nest_dir)
+                # FIXME: job_state is from an unrelated job :/
+                job_state.via_job, job_state, config, session_dir,
+                nest_dir)
             for key, value in sorted(parent_env.items()):
                 cmd += ['-G', '{}={}'.format(key, value)]
         # Run the specified target job in the specified environment
         cmd += ['--target', job.checksum]
         env = self.get_differential_execution_environment(
-            job, config, session_dir, nest_dir)
+            job, job_state, config, session_dir, nest_dir)
         for key, value in sorted(env.items()):
             cmd += ['-T', '{}={}'.format(key, value)]
         return cmd
@@ -957,6 +1221,9 @@ class RootViaPTL1ExecutionController(CheckBoxDifferentialExecutionController):
         # Doesn't work when connected over SSH (LP: #1299201)
         if os.environ.get("SSH_CONNECTION"):
             return -1
+        # Doesn't work for windows jobs
+        if 'win32' in job.get_flag_set():
+            return -1
         # Only makes sense with jobs that need to run as another user
         # Promote this controller only if the trusted launcher is authorized to
         # run jobs as another user
@@ -1004,12 +1271,15 @@ class RootViaPkexecExecutionController(
     root from the non-system-wide location.
     """
 
-    def get_execution_command(self, job, config, session_dir, nest_dir):
+    def get_execution_command(self, job, job_state, config, session_dir,
+                              nest_dir):
         """
         Get the command to invoke.
 
         :param job:
             job definition with the command and environment definitions
+        :param job_state:
+            The JobState associated to the job to execute.
         :param config:
             A PlainBoxConfig instance which can be used to load missing
             environment definitions that apply to all jobs. Passed to
@@ -1032,7 +1302,7 @@ class RootViaPkexecExecutionController(
         cmd = ['pkexec', '--user', job.user, 'env']
         # Append all environment data
         env = self.get_differential_execution_environment(
-            job, config, session_dir, nest_dir)
+            job, job_state, config, session_dir, nest_dir)
         cmd += ["{key}={value}".format(key=key, value=value)
                 for key, value in sorted(env.items())]
         # Lastly use job.shell -c, to run our command
@@ -1046,6 +1316,9 @@ class RootViaPkexecExecutionController(
         :returns:
             one for jobs with a user override, zero otherwise
         """
+        # Doesn't work for windows jobs
+        if 'win32' in job.get_flag_set():
+            return -1
         if job.user is not None:
             return 1
         else:
@@ -1090,12 +1363,15 @@ class RootViaSudoExecutionController(
             in_admin_group = False
         self.user_can_sudo = in_sudo_group or in_admin_group
 
-    def get_execution_command(self, job, config, session_dir, nest_dir):
+    def get_execution_command(self, job, job_state, config, session_dir,
+                              nest_dir):
         """
         Get the command to invoke.
 
         :param job:
             job definition with the command and environment definitions
+        :param job_state:
+            The JobState associated to the job to execute.
         :param config:
             A PlainBoxConfig instance which can be used to load missing
             environment definitions that apply to all jobs. Ignored.
@@ -1116,7 +1392,7 @@ class RootViaSudoExecutionController(
         cmd = ['sudo', '-u', job.user, 'env']
         # Append all environment data
         env = self.get_differential_execution_environment(
-            job, config, session_dir, nest_dir)
+            job, job_state, config, session_dir, nest_dir)
         cmd += ["{key}={value}".format(key=key, value=value)
                 for key, value in sorted(env.items())]
         # Lastly use job.shell -c, to run our command
@@ -1131,6 +1407,9 @@ class RootViaSudoExecutionController(
             -1 if the job does not have a user override or the user cannot use
             sudo and 2 otherwise
         """
+        # Doesn't work for windows jobs
+        if 'win32' in job.get_flag_set():
+            return -1
         # Only makes sense with jobs that need to run as another user
         if job.user is not None and self.user_can_sudo:
             return 2
