@@ -35,6 +35,7 @@ import pkg_resources
 
 from plainbox.i18n import gettext as _
 from plainbox.abc import ISessionStateExporter
+from plainbox.impl import deprecated
 
 logger = getLogger("plainbox.exporter")
 
@@ -84,6 +85,7 @@ class SessionStateExporterBase(ISessionStateExporter):
     OPTION_WITH_JOB_VIA = 'with-job-via'
     OPTION_WITH_JOB_HASH = 'with-job-hash'
     OPTION_WITH_CATEGORY_MAP = 'with-category-map'
+    OPTION_WITH_CERTIFICATION_STATUS = 'with-certification-status'
 
     SUPPORTED_OPTION_LIST = (
         OPTION_WITH_IO_LOG,
@@ -98,12 +100,14 @@ class SessionStateExporterBase(ISessionStateExporter):
         OPTION_WITH_JOB_VIA,
         OPTION_WITH_JOB_HASH,
         OPTION_WITH_CATEGORY_MAP,
+        OPTION_WITH_CERTIFICATION_STATUS,
     )
 
-    def __init__(self, option_list=None):
+    def __init__(self, option_list=None, exporter_unit=None):
         if option_list is None:
             option_list = []
         self._option_dict = {}
+        self._unit = exporter_unit
         for option_string in option_list:
             # An option can look like "foo" or like "foo=bar". In the first
             # case we assign "True" to that key in the dict, in the second we
@@ -117,6 +121,30 @@ class SessionStateExporterBase(ISessionStateExporter):
             else:
                 value = True
             self._option_dict[option] = value
+        if exporter_unit:
+            for option_string in exporter_unit.option_list:
+                # An option can look like "foo" or like "foo=bar". In the first
+                # case we assign "True" to that key in the dict, in the second
+                # we assign "bar".
+                key_value = option_string.split("=", 1)
+                option = key_value[0]
+                if option not in self.supported_option_list:
+                    raise ValueError("Unsupported option: {}".format(option))
+                if len(key_value) == 2:
+                    value = key_value[1]
+                else:
+                    value = True
+                self._option_dict[option] = value
+
+    @property
+    def unit(self):
+        """
+        Exporter unit this exporter was created with.
+
+        The exporter unit holds additional information that may be of use to
+        applications, such as typical file name extension.
+        """
+        return self._unit
 
     @property
     def _option_list(self):
@@ -159,7 +187,17 @@ class SessionStateExporterBase(ISessionStateExporter):
         """
         return cls.SUPPORTED_OPTION_LIST
 
-    def get_session_data_subset(self, session):
+    def dump_from_session_manager(self, session_manager, stream):
+        """
+        Dump session information pulled from session manager to stream.
+
+        This method takes session manager instance, extracts session
+        information from it, and dumps it to a stream.
+        """
+        self.dump(self.get_session_data_subset(session_manager), stream)
+
+    @deprecated('0.21', 'use .dump_from_session_manager instead')
+    def get_session_data_subset(self, session_manager):
         """
         Compute a subset of session data.
 
@@ -173,6 +211,7 @@ class SessionStateExporterBase(ISessionStateExporter):
         data = {
             'result_map': {}
         }
+        session = session_manager.state
         if self.OPTION_WITH_JOB_LIST in self._option_list:
             data['job_list'] = [job.id for job in session.job_list]
         if self.OPTION_WITH_RUN_LIST in self._option_list:
@@ -228,7 +267,10 @@ class SessionStateExporterBase(ISessionStateExporter):
 
             # Add Parent hash if requested
             if self.OPTION_WITH_JOB_VIA in self._option_list:
-                data['result_map'][job_id]['via'] = job_state.job.via
+                data['result_map'][job_id]['via'] = (
+                    job_state.via_job.checksum
+                    if job_state.via_job is not None else None
+                )
 
             # Add Job hash if requested
             if self.OPTION_WITH_JOB_HASH in self._option_list:
@@ -266,6 +308,11 @@ class SessionStateExporterBase(ISessionStateExporter):
                 else:
                     io_log_data = self._io_log(job_state.result.get_io_log())
                 data['result_map'][job_id]['io_log'] = io_log_data
+
+            # Add certification status if requested
+            if self.OPTION_WITH_CERTIFICATION_STATUS in self._option_list:
+                data['result_map'][job_id]['certification_status'] = (
+                    job_state.effective_certification_status)
         return data
 
     def _build_attachment_map(self, data, job_id, job_state):
@@ -334,23 +381,3 @@ class ByteStringStreamTranslator(RawIOBase):
             :param data: the chunk of data to write.
         """
         return self.dest_stream.write(data.decode(self.encoding))
-
-
-def get_all_exporters():
-    """
-    Discover and load all exporter classes.
-
-    Returns a map of exporters (mapping from name to exporter class)
-    """
-    exporter_map = OrderedDict()
-    iterator = pkg_resources.iter_entry_points('plainbox.exporter')
-    for entry_point in sorted(iterator, key=lambda ep: ep.name):
-        try:
-            exporter_cls = entry_point.load()
-        except pkg_resources.DistributionNotFound as exc:
-            logger.info(_("Unable to load %s: %s"), entry_point, exc)
-        except ImportError as exc:
-            logger.exception(_("Unable to import %s: %s"), entry_point, exc)
-        else:
-            exporter_map[entry_point.name] = exporter_cls
-    return exporter_map
