@@ -37,6 +37,7 @@ from xlsxwriter.utility import xl_rowcol_to_cell
 from plainbox.abc import IJobResult
 from plainbox.i18n import gettext as _, ngettext
 from plainbox.impl.exporter import SessionStateExporterBase
+from plainbox.impl.result import OUTCOME_METADATA_MAP as OMM
 
 
 class XLSXSessionStateExporter(SessionStateExporterBase):
@@ -72,12 +73,12 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
         OPTION_WITH_UNIT_CATEGORIES,
     )
 
-    def __init__(self, option_list=None):
+    def __init__(self, option_list=None, exporter_unit=None):
         """
         Initialize a new XLSXSessionStateExporter.
         """
         # Super-call with empty option list
-        super(XLSXSessionStateExporter, self).__init__(())
+        super().__init__((), exporter_unit=exporter_unit)
         # All the "options" are simply a required configuration element and are
         # not optional in any way. There is no way to opt-out.
         if option_list is None:
@@ -85,6 +86,11 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
         for option in option_list:
             if option not in self.supported_option_list:
                 raise ValueError(_("Unsupported option: {}").format(option))
+        if exporter_unit:
+            for option in exporter_unit.option_list:
+                if option not in self.supported_option_list:
+                    raise ValueError(
+                        _("Unsupported option: {}").format(option))
         self._option_list = (
             SessionStateExporterBase.OPTION_WITH_IO_LOG,
             SessionStateExporterBase.OPTION_FLATTEN_IO_LOG,
@@ -95,8 +101,11 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
             SessionStateExporterBase.OPTION_WITH_RESOURCE_MAP,
             SessionStateExporterBase.OPTION_WITH_ATTACHMENTS,
             SessionStateExporterBase.OPTION_WITH_CATEGORY_MAP,
+            SessionStateExporterBase.OPTION_WITH_CERTIFICATION_STATUS,
         )
         self._option_list += tuple(option_list)
+        if exporter_unit:
+            self._option_list += tuple(exporter_unit.option_list)
         self.total_pass = 0
         self.total_fail = 0
         self.total_skip = 0
@@ -157,6 +166,18 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
             'align': 'center', 'valign': 'vcenter', 'text_wrap': 1, 'size': 8,
             'bg_color': 'gray', 'border': 1, 'border_color': 'white',
         })
+        # Dictionary with formats for each possible outcome
+        self.outcome_format_map = {
+            outcome_info.value: self.workbook.add_format({
+                'align': 'center',
+                'valign': 'vcenter',
+                'text_wrap': '1',
+                'size': 8,
+                'bg_color': outcome_info.color_hex,
+                'border': 1,
+                'border_color': 'white'
+            }) for outcome_info in OMM.values()
+        }
         # Attachments
         self.format13 = self.workbook.add_format({
             'align': 'left', 'valign': 'vcenter', 'text_wrap': 1, 'size': 8,
@@ -177,30 +198,42 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
             'align': 'left', 'valign': 'vcenter', 'size': 8,
             'bg_color': '#E6E6E6', 'indent': 1,
         })
+        # Table rows without borders (center)
+        self.format18 = self.workbook.add_format({
+            'align': 'center', 'valign': 'vcenter', 'size': 8,
+        })
+        # Table rows without borders, grayed out background (center)
+        self.format19 = self.workbook.add_format({
+            'align': 'center', 'valign': 'vcenter', 'size': 8,
+            'bg_color': '#E6E6E6',
+        })
 
     def _hw_collection(self, data):
         hw_info = defaultdict(lambda: 'NA')
-        if '2013.com.canonical.certification::dmi' in data['resource_map']:
+        resource = '2013.com.canonical.certification::dmi'
+        if resource in data['resource_map']:
             result = [
                 '{} {} ({})'.format(
                     i.get('vendor'), i.get('product'), i.get('version'))
-                for i in data["resource_map"]['2013.com.canonical.certification::dmi']
+                for i in data["resource_map"][resource]
                 if i.get('category') == 'SYSTEM']
             if result:
                 hw_info['platform'] = result.pop()
             result = [
                 '{}'.format(i.get('version'))
-                for i in data["resource_map"]['2013.com.canonical.certification::dmi']
+                for i in data["resource_map"][resource]
                 if i.get('category') == 'BIOS']
             if result:
                 hw_info['bios'] = result.pop()
-        if '2013.com.canonical.certification::cpuinfo' in data['resource_map']:
+        resource = '2013.com.canonical.certification::cpuinfo'
+        if resource in data['resource_map']:
             result = ['{} x {}'.format(i['model'], i['count'])
-                      for i in data["resource_map"]['2013.com.canonical.certification::cpuinfo']]
+                      for i in data["resource_map"][resource]]
             if result:
                 hw_info['processors'] = result.pop()
-        if '2013.com.canonical.certification::lspci_attachment' in data['attachment_map']:
-            lspci = data['attachment_map']['2013.com.canonical.certification::lspci_attachment']
+        resource = '2013.com.canonical.certification::lspci_attachment'
+        if resource in data['attachment_map']:
+            lspci = data['attachment_map'][resource]
             content = standard_b64decode(lspci.encode()).decode("UTF-8")
             match = re.search('ISA bridge.*?:\s(?P<chipset>.*?)\sLPC', content)
             if match:
@@ -230,9 +263,10 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                 vram += int(match.group('vram'))
             if vram:
                 hw_info['vram'] = '{} MiB'.format(vram)
-        if '2013.com.canonical.certification::meminfo' in data['resource_map']:
+        resource = '2013.com.canonical.certification::meminfo'
+        if resource in data['resource_map']:
             result = ['{} GiB'.format(format(int(i['total']) / 1073741824,
-                      '.1f')) for i in data["resource_map"]['2013.com.canonical.certification::meminfo']]
+                      '.1f')) for i in data["resource_map"][resource]]
             if result:
                 hw_info['memory'] = result.pop()
         bluetooth = self._get_bluetooth_product_or_path(data)
@@ -318,7 +352,8 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
         self.worksheet1.write(16, 2, hw_info['wireless'], self.format05)
         self.worksheet1.write(17, 1, _('Bluetooth'), self.format04)
         self.worksheet1.write(17, 2, hw_info['bluetooth'], self.format06)
-        if "2013.com.canonical.certification::package" in data["resource_map"]:
+        resource = '2013.com.canonical.certification::package'
+        if resource in data["resource_map"]:
             self.worksheet1.write(
                 19, 1, _('Packages Installed'), self.format03)
             self.worksheet1.write_row(
@@ -327,7 +362,7 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                 self.worksheet1.set_row(
                     i, None, None, {'level': 1, 'hidden': True}
                 )
-            for i, pkg in enumerate(data["resource_map"]["2013.com.canonical.certification::package"]):
+            for i, pkg in enumerate(data["resource_map"][resource]):
                 self.worksheet1.write_row(
                     22 + i, 1,
                     [pkg['name'], pkg['version']],
@@ -337,7 +372,7 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                     22 + i, None, None, {'level': 1, 'hidden': True}
                 )
             self.worksheet1.set_row(
-                22+len(data["resource_map"]["2013.com.canonical.certification::package"]),
+                22+len(data["resource_map"][resource]),
                 None, None, {'collapsed': True}
             )
 
@@ -354,7 +389,8 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
         self.worksheet2.set_column(1, 1, 2)
         self.worksheet2.set_column(3, 3, 27)
         self.worksheet2.write(3, 1, _('Failures summary'), self.format03)
-        self.worksheet2.write(4, 1, '✔', self.format10)
+        self.worksheet2.write(
+            4, 1, OMM['pass'].unicode_sigil, self.outcome_format_map['pass'])
         self.worksheet2.write(
             4, 2, (
                 ngettext('{} Test passed', '{} Tests passed',
@@ -363,7 +399,8 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                 + _('Success Rate: {} ({}/{})').format(
                     pass_rate, self.total_pass, self.total)
             ), self.format02)
-        self.worksheet2.write(5, 1, '✘', self.format11)
+        self.worksheet2.write(
+            5, 1, OMM['fail'].unicode_sigil, self.outcome_format_map['fail'])
         self.worksheet2.write(
             5, 2, (
                 ngettext('{} Test failed', '{} Tests failed',
@@ -372,7 +409,8 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                 + _('Failure Rate: {} ({}/{})').format(
                     fail_rate, self.total_fail, self.total)
             ), self.format02)
-        self.worksheet2.write(6, 1, '-', self.format12)
+        self.worksheet2.write(
+            6, 1, OMM['skip'].unicode_sigil, self.outcome_format_map['skip'])
         self.worksheet2.write(
             6, 2, (
                 ngettext('{} Test skipped', '{} Tests skipped',
@@ -382,7 +420,9 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                     skip_rate, self.total_skip, self.total)
             ), self.format02)
         self.worksheet2.write_column(
-            'L3', [_('Fail'), _('Skip'), _('Pass')], self.format14)
+            'L3', [OMM['fail'].tr_label,
+                   OMM['skip'].tr_label,
+                   OMM['pass'].tr_label], self.format14)
         self.worksheet2.write_column(
             'M3', [self.total_fail, self.total_skip, self.total_pass],
             self.format14)
@@ -391,9 +431,9 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
         chart.set_legend({'position': 'none'})
         chart.add_series({
             'points': [
-                {'fill': {'color': 'red'}},
-                {'fill': {'color': 'gray'}},
-                {'fill': {'color': 'lime'}},
+                {'fill': {'color': OMM['fail'].color_hex}},
+                {'fill': {'color': OMM['skip'].color_hex}},
+                {'fill': {'color': OMM['pass'].color_hex}},
             ],
             'categories': '=' + _("Summary") + '!$L$3:$L$5',
             'values': '=' + _("Summary") + '!$M$3:$M$5'}
@@ -492,24 +532,10 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                 self.worksheet3.write(
                     self._lineno, level + 1,
                     result_map[job]['summary'], self.format15)
-                if (
-                    result_map[job]['category_status'] ==
-                    IJobResult.OUTCOME_PASS
-                ):
-                    self.worksheet3.write(
-                        self._lineno, max_level + 2, _('PASS'), self.format10)
-                elif (
-                    result_map[job]['category_status'] ==
-                    IJobResult.OUTCOME_FAIL
-                ):
-                    self.worksheet3.write(
-                        self._lineno, max_level + 2, _('FAIL'), self.format11)
-                elif (
-                    result_map[job]['category_status'] ==
-                    IJobResult.OUTCOME_SKIP
-                ):
-                    self.worksheet3.write(
-                        self._lineno, max_level + 2, _('skip'), self.format12)
+                outcome = result_map[job]['category_status']
+                self.worksheet3.write(
+                    self._lineno, max_level + 2,
+                    OMM[outcome].tr_label, self.outcome_format_map[outcome])
                 if self.OPTION_WITH_DESCRIPTION in self._option_list:
                     self.worksheet4.write(
                         self._lineno, level + 1,
@@ -543,38 +569,30 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                         result_map[job]['summary'],
                         self.format08 if self._lineno % 2 else self.format09)
                 self.total += 1
-                if result_map[job]['outcome'] == IJobResult.OUTCOME_PASS:
-                    self.worksheet3.write(
-                        self._lineno, max_level, '✔', self.format10)
-                    self.worksheet3.write(
-                        self._lineno, max_level + 2, _('PASS'), self.format10)
+                outcome = result_map[job]['outcome']
+                self.worksheet3.write(
+                    self._lineno, max_level, OMM[outcome].unicode_sigil,
+                    self.outcome_format_map[outcome])
+                self.worksheet3.write(
+                    self._lineno, max_level + 2, OMM[outcome].tr_label,
+                    self.outcome_format_map[outcome])
+                if outcome == IJobResult.OUTCOME_PASS:
                     self.total_pass += 1
-                elif result_map[job]['outcome'] == IJobResult.OUTCOME_FAIL:
-                    self.worksheet3.write(
-                        self._lineno, max_level, '✘', self.format11)
-                    self.worksheet3.write(
-                        self._lineno, max_level + 2, _('FAIL'), self.format11)
+                elif outcome == IJobResult.OUTCOME_FAIL:
                     self.total_fail += 1
-                elif result_map[job]['outcome'] == IJobResult.OUTCOME_SKIP:
-                    self.worksheet3.write(
-                        self._lineno, max_level, '-', self.format12)
-                    self.worksheet3.write(
-                        self._lineno, max_level + 2, _('skip'), self.format12)
-                    self.total_skip += 1
-                elif result_map[job]['outcome'] == \
-                        IJobResult.OUTCOME_NOT_SUPPORTED:
-                    self.worksheet3.write(
-                        self._lineno, max_level, '-', self.format12)
-                    self.worksheet3.write(
-                        self._lineno, max_level + 2,
-                        _('not supported'), self.format12)
-                    self.total_skip += 1
                 else:
-                    self.worksheet3.write(
-                        self._lineno, max_level, '-',    self.format12)
-                    self.worksheet3.write(
-                        self._lineno, max_level + 2, None, self.format12)
+                    # NOTE: this is inaccurate but that's how the original code
+                    # behaved. This will be fixed with detailed per-outcome
+                    # counters later.
                     self.total_skip += 1
+                cert_status = ''
+                if 'certification_status' in result_map[job]:
+                    cert_status = result_map[job]['certification_status']
+                    if cert_status == 'unspecified':
+                        cert_status = ''
+                self.worksheet3.write(
+                    self._lineno, max_level + 3, cert_status,
+                    self.format18 if self._lineno % 2 else self.format19)
                 io_log = ' '
                 if result_map[job]['io_log']:
                     io_log = standard_b64decode(
@@ -585,13 +603,13 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                                                      "").splitlines())
                 desc_lines -= 1
                 self.worksheet3.write(
-                    self._lineno, max_level + 3, io_log,
+                    self._lineno, max_level + 4, io_log,
                     self.format16 if self._lineno % 2 else self.format17)
                 comments = ' '
                 if result_map[job]['comments']:
                     comments = result_map[job]['comments'].rstrip()
                 self.worksheet3.write(
-                    self._lineno, max_level + 4, comments,
+                    self._lineno, max_level + 5, comments,
                     self.format16 if self._lineno % 2 else self.format17)
                 if self.OPTION_WITH_DESCRIPTION in self._option_list:
                     self.worksheet4.write(
@@ -625,11 +643,13 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
         [self.worksheet3.set_column(i, i, 2) for i in range(1, max_level + 1)]
         self.worksheet3.set_column(max_level + 1, max_level + 1, 48)
         self.worksheet3.set_column(max_level + 2, max_level + 2, 12)
-        self.worksheet3.set_column(max_level + 3, max_level + 3, 65)
+        self.worksheet3.set_column(max_level + 3, max_level + 3, 18)
         self.worksheet3.set_column(max_level + 4, max_level + 4, 65)
+        self.worksheet3.set_column(max_level + 5, max_level + 5, 65)
         self.worksheet3.write_row(
             5, max_level + 1,
-            [_('Name'), _('Result'), _('I/O Log'), _('Comments')],
+            [_('Name'), _('Result'), _('Certification Status'), _('I/O Log'),
+             _('Comments')],
             self.format07)
         if self.OPTION_WITH_DESCRIPTION in self._option_list:
             self.worksheet4.write(3, 1, _('Test Descriptions'), self.format03)
