@@ -29,7 +29,6 @@ import logging
 
 from plainbox import __version__ as plainbox_version
 from plainbox.impl.applogic import run_job_if_possible
-from plainbox.impl.exporter import get_all_exporters
 from plainbox.impl.runner import JobRunner
 from plainbox.impl.session import SessionStorageRepository
 from plainbox.impl.session.legacy import SessionStateLegacyAPI as SessionState
@@ -198,7 +197,7 @@ class Explorer:
                     ('gettext_domain', provider.gettext_domain),
                     ('base_dir', provider.base_dir),
                 )))
-            for unit in provider.get_units()[0]:
+            for unit in provider.unit_list:
                 provider_obj.children.append(self._unit_to_obj(unit))
             service_obj.children.append(provider_obj)
         # Milk each repository for session storage data
@@ -221,6 +220,7 @@ class Explorer:
         return service_obj
 
     def _unit_to_obj(self, unit):
+        # Yes, this should be moved to member methods
         if unit.Meta.name == 'test plan':
             return self._test_plan_to_obj(unit)
         elif unit.Meta.name == 'job':
@@ -231,6 +231,12 @@ class Explorer:
             return self._file_to_obj(unit)
         elif unit.Meta.name == 'template':
             return self._template_to_obj(unit)
+        elif unit.Meta.name == 'manifest entry':
+            return self._manifest_entry_to_obj(unit)
+        elif unit.Meta.name == 'packaging meta-data':
+            return self._packaging_meta_data_to_obj(unit)
+        elif unit.Meta.name == 'exporter':
+            return self._exporter_entry_to_obj(unit)
         else:
             raise NotImplementedError(unit.Meta.name)
 
@@ -309,6 +315,34 @@ class Explorer:
                 ('origin', str(unit.origin)),
             )))
 
+    def _manifest_entry_to_obj(self, unit):
+        return PlainBoxObject(
+            unit, group=unit.Meta.name, name=unit.id, attrs=OrderedDict((
+                ('id', unit.id),
+                ('name', unit.name),
+                ('tr_name', unit.tr_name()),
+                ('value_type', unit.value_type),
+                ('value_unit', unit.value_unit),
+                ('resource_key', unit.resource_key),
+                ('origin', str(unit.origin)),
+            )))
+
+    def _packaging_meta_data_to_obj(self, unit):
+        return PlainBoxObject(
+            unit, group=unit.Meta.name, name=unit.os_id, attrs=OrderedDict((
+                ('os_id', unit.os_id),
+                ('os_version_id', unit.os_version_id),
+                ('origin', str(unit.origin)),
+            )))
+
+    def _exporter_entry_to_obj(self, unit):
+        return PlainBoxObject(
+            unit, group=unit.Meta.name, name=unit.id, attrs=OrderedDict((
+                ('id', unit.id),
+                ('summary', unit.summary),
+                ('tr_summary', unit.tr_summary()),
+            )))
+
 
 class Service:
 
@@ -339,15 +373,18 @@ class Service:
         # TODO: construct state
         # TODO: construct manager, binding storage and state
         # TODO: if something fails destroy storage
-        session = SessionState(job_list)
+        unit_list = job_list
+        # Add exporters to the list of units in order to get them from the
+        # session manager exporter_map property.
+        for provider in self.provider_list:
+            for unit in provider.unit_list:
+                if unit.Meta.name == 'exporter':
+                    unit_list.append(unit)
+        session = SessionState(unit_list)
         session.open()
         self._session_list.append(session)
         return session
 
-    def get_all_exporters(self):
-        return {name: exporter_cls.supported_option_list for
-                name, exporter_cls in get_all_exporters().items()}
-
     def export_session(self, session, output_format, option_list):
         temp_stream = BytesIO()
         self._export_session_to_stream(session, output_format,
@@ -355,18 +392,17 @@ class Service:
         return temp_stream.getvalue()
 
     def export_session_to_file(self, session, output_format, option_list,
-                               output_file):
+                               exporter_unit, output_file):
         with open(output_file, 'wb') as stream:
             self._export_session_to_stream(
-                session, output_format, option_list, stream)
+                session, output_format, option_list, exporter_unit, stream)
         return output_file
 
     def _export_session_to_stream(self, session, output_format, option_list,
-                                  stream):
-        exporter_cls = get_all_exporters()[output_format]
-        exporter = exporter_cls(option_list)
-        data_subset = exporter.get_session_data_subset(session)
-        exporter.dump(data_subset, stream)
+                                  exporter_unit, stream):
+        unit = session.manager.exporter_map[exporter_unit]
+        exporter = unit.exporter_cls(option_list, exporter_unit=unit)
+        exporter.dump_from_session_manager(session.manager, stream)
 
     def get_all_transports(self):
         return [transport for transport in get_all_transports()]
