@@ -1,13 +1,13 @@
+# encoding: utf-8
 # This file is part of Checkbox.
 #
-# Copyright 2012, 2013 Canonical Ltd.
+# Copyright 2012-2015 Canonical Ltd.
 # Written by:
 #   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
 #
 # Checkbox is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3,
 # as published by the Free Software Foundation.
-
 #
 # Checkbox is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,6 +18,8 @@
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
 
 """
+Definition of JobRunner class.
+
 :mod:`plainbox.impl.runner` -- job runner
 =========================================
 
@@ -36,38 +38,32 @@ import string
 import sys
 import time
 
-from plainbox.vendor import extcmd
-
-from plainbox.abc import IJobRunner, IJobResult
+from plainbox.abc import IJobResult, IJobRunner
 from plainbox.i18n import gettext as _
-from plainbox.impl.result import DiskJobResult
 from plainbox.impl.result import IOLogRecord
 from plainbox.impl.result import IOLogRecordWriter
-from plainbox.impl.result import MemoryJobResult
-from plainbox.impl.signal import Signal
+from plainbox.impl.result import JobResultBuilder
+from plainbox.vendor import extcmd
+from plainbox.vendor import morris
 
 
 logger = logging.getLogger("plainbox.runner")
 
 
 def slugify(_string):
-    """
-    Slugify - like Django does for URL - transform a random string to a valid
-    slug that can be later used in filenames
-    """
+    """Transform any string to onet that can be used in filenames."""
     valid_chars = frozenset(
         "-_.{}{}".format(string.ascii_letters, string.digits))
     return ''.join(c if c in valid_chars else '_' for c in _string)
 
 
 class IOLogRecordGenerator(extcmd.DelegateBase):
-    """
-    Delegate for extcmd that generates io_log entries.
-    """
+
+    """Delegate for extcmd that generates io_log entries."""
 
     def on_begin(self, args, kwargs):
         """
-        Internal method of extcmd.DelegateBase
+        Internal method of extcmd.DelegateBase.
 
         Called when a command is being invoked.
 
@@ -77,7 +73,7 @@ class IOLogRecordGenerator(extcmd.DelegateBase):
 
     def on_line(self, stream_name, line):
         """
-        Internal method of extcmd.DelegateBase
+        Internal method of extcmd.DelegateBase.
 
         Creates a new IOLogRecord and passes it to :meth:`on_new_record()`.
         Maintains a timestamp of the last message so that approximate delay
@@ -89,10 +85,10 @@ class IOLogRecordGenerator(extcmd.DelegateBase):
         record = IOLogRecord(delay.total_seconds(), stream_name, line)
         self.on_new_record(record)
 
-    @Signal.define
+    @morris.signal
     def on_new_record(self, record):
         """
-        Internal signal method of :class:`IOLogRecordGenerator`
+        Internal signal method of :class:`IOLogRecordGenerator`.
 
         Called when a new record is generated and needs to be processed.
         """
@@ -101,6 +97,7 @@ class IOLogRecordGenerator(extcmd.DelegateBase):
 
 
 class CommandOutputWriter(extcmd.DelegateBase):
+
     """
     Delegate for extcmd that writes output to a file on disk.
 
@@ -120,7 +117,7 @@ class CommandOutputWriter(extcmd.DelegateBase):
 
     def on_begin(self, args, kwargs):
         """
-        Internal method of extcmd.DelegateBase
+        Internal method of extcmd.DelegateBase.
 
         Called when a command is being invoked
         """
@@ -129,7 +126,7 @@ class CommandOutputWriter(extcmd.DelegateBase):
 
     def on_end(self, returncode):
         """
-        Internal method of extcmd.DelegateBase
+        Internal method of extcmd.DelegateBase.
 
         Called when a command finishes running
         """
@@ -138,7 +135,7 @@ class CommandOutputWriter(extcmd.DelegateBase):
 
     def on_abnormal_end(self, signal_num):
         """
-        Internal method of extcmd.DelegateBase
+        Internal method of extcmd.DelegateBase.
 
         Called when a command abnormally finishes running
         """
@@ -147,7 +144,7 @@ class CommandOutputWriter(extcmd.DelegateBase):
 
     def on_line(self, stream_name, line):
         """
-        Internal method of extcmd.DelegateBase
+        Internal method of extcmd.DelegateBase.
 
         Called for each line of output.
         """
@@ -158,6 +155,7 @@ class CommandOutputWriter(extcmd.DelegateBase):
 
 
 class FallbackCommandOutputPrinter(extcmd.DelegateBase):
+
     """
     Delegate for extcmd that prints all output to stdout.
 
@@ -166,11 +164,19 @@ class FallbackCommandOutputPrinter(extcmd.DelegateBase):
     """
 
     def __init__(self, prompt):
+        """Initialize a new fallback command output printer."""
         self._prompt = prompt
         self._lineno = collections.defaultdict(int)
         self._abort = False
 
     def on_line(self, stream_name, line):
+        """
+        Internal method of extcmd.DelegateBase.
+
+        Called for each line of output. Normally each line is just printed
+        (assuming UTF-8 encoding) If decoding fails for any reason that and all
+        subsequent lines are ignored.
+        """
         if self._abort:
             return
         self._lineno[stream_name] += 1
@@ -183,8 +189,9 @@ class FallbackCommandOutputPrinter(extcmd.DelegateBase):
 
 
 class JobRunnerUIDelegate(extcmd.DelegateBase):
+
     """
-    Delegate for extcmd that delegates extcmd events to IJobRunnerUI
+    Delegate for extcmd that delegates extcmd events to IJobRunnerUI.
 
     The file itself is only opened once on_begin() gets called by extcmd. This
     makes it safe to instantiate this without worrying about dangling
@@ -197,7 +204,7 @@ class JobRunnerUIDelegate(extcmd.DelegateBase):
 
     def __init__(self, ui=None):
         """
-        Initialize the JobRunnerUIDelegate
+        Initialize the JobRunnerUIDelegate.
 
         :param ui:
             (optional) an instnace of IJobRunnerUI to delegate events to
@@ -206,7 +213,7 @@ class JobRunnerUIDelegate(extcmd.DelegateBase):
 
     def on_begin(self, args, kwargs):
         """
-        Internal method of extcmd.DelegateBase
+        Internal method of extcmd.DelegateBase.
 
         Called when a command is being invoked
         """
@@ -215,7 +222,7 @@ class JobRunnerUIDelegate(extcmd.DelegateBase):
 
     def on_end(self, returncode):
         """
-        Internal method of extcmd.DelegateBase
+        Internal method of extcmd.DelegateBase.
 
         Called when a command finishes running
         """
@@ -224,7 +231,7 @@ class JobRunnerUIDelegate(extcmd.DelegateBase):
 
     def on_abnormal_end(self, signal_num):
         """
-        Internal method of extcmd.DelegateBase
+        Internal method of extcmd.DelegateBase.
 
         Called when a command abnormally finishes running
 
@@ -236,17 +243,27 @@ class JobRunnerUIDelegate(extcmd.DelegateBase):
 
     def on_line(self, stream_name, line):
         """
-        Internal method of extcmd.DelegateBase
+        Internal method of extcmd.DelegateBase.
 
         Called for each line of output.
         """
         if self.ui is not None:
             self.ui.got_program_output(stream_name, line)
 
+    def on_chunk(self, stream_name, chunk):
+        """
+        Internal method of extcmd.DelegateBase.
+
+        Called for each chunk of output.
+        """
+        if self.ui is not None:
+            self.ui.got_program_output(stream_name, chunk)
+
 
 class JobRunner(IJobRunner):
+
     """
-    Runner for jobs - executes jobs and produces results
+    Runner for jobs - executes jobs and produces results.
 
     The runner is somewhat de-coupled from jobs and session. It still carries
     all checkbox-specific logic about the various types of plugins.
@@ -290,18 +307,20 @@ class JobRunner(IJobRunner):
         """
         self._session_dir = session_dir
         if execution_ctrl_list is None:
-            logger.warning("execution_ctrl_list not passed to JobRunner")
+            logger.debug("execution_ctrl_list not passed to JobRunner")
             if sys.platform == 'linux' or sys.platform == 'linux2':
                 from plainbox.impl.ctrl import RootViaPkexecExecutionController
                 from plainbox.impl.ctrl import RootViaPTL1ExecutionController
                 from plainbox.impl.ctrl import RootViaSudoExecutionController
                 from plainbox.impl.ctrl import UserJobExecutionController
+                from plainbox.impl.ctrl import QmlJobExecutionController
                 execution_ctrl_list = [
                     RootViaPTL1ExecutionController(provider_list),
                     RootViaPkexecExecutionController(provider_list),
                     # XXX: maybe this one should be only used on command line
                     RootViaSudoExecutionController(provider_list),
                     UserJobExecutionController(provider_list),
+                    QmlJobExecutionController(provider_list),
                 ]
             elif sys.platform == 'win32':
                 from plainbox.impl.ctrl import UserJobExecutionController
@@ -322,7 +341,7 @@ class JobRunner(IJobRunner):
     @property
     def log_leftovers(self):
         """
-        flag controlling if leftover files should be logged
+        flag controlling if leftover files should be logged.
 
         If you wish to connect a custom handler to :meth:`on_leftover_files()`
         then it is advisable to set this property to False so that leftover
@@ -334,6 +353,7 @@ class JobRunner(IJobRunner):
 
     @log_leftovers.setter
     def log_leftovers(self, value):
+        """setter for log_leftovers property."""
         self._log_leftovers = value
 
     def get_warm_up_sequence(self, job_list):
@@ -362,12 +382,14 @@ class JobRunner(IJobRunner):
                 warm_up_list.append(warm_up_func)
         return warm_up_list
 
-    def run_job(self, job, config=None, ui=None):
+    def run_job(self, job, job_state, config=None, ui=None):
         """
-        Run the specified job an return the result
+        Run the specified job an return the result.
 
         :param job:
             A JobDefinition to run
+        :param job_state:
+            The JobState associated to the job to execute.
         :param config:
             A PlainBoxConfig that may influence how this job is executed. This
             is only used for the environment variables (that should be
@@ -399,23 +421,23 @@ class JobRunner(IJobRunner):
         try:
             runner = getattr(self, func_name)
         except AttributeError:
-            return MemoryJobResult({
-                'outcome': IJobResult.OUTCOME_NOT_IMPLEMENTED,
-                'comment': _('This type of job is not supported'),
-            })
+            return JobResultBuilder(
+                outcome=IJobResult.OUTCOME_NOT_IMPLEMENTED,
+                comments=_('This type of job is not supported')
+            ).get_result()
         else:
             if self._dry_run and job.plugin not in self._DRY_RUN_PLUGINS:
                 return self._get_dry_run_result(job)
             else:
                 self._job_runner_ui_delegate.ui = ui
                 try:
-                    return runner(job, config)
+                    return runner(job, job_state, config)
                 finally:
                     self._job_runner_ui_delegate.ui = None
 
-    def run_shell_job(self, job, config):
+    def run_shell_job(self, job, job_state, config):
         """
-        Method called to run a job with plugin field equal to 'shell'
+        Method called to run a job with plugin field equal to 'shell'.
 
         The 'shell' job implements the following scenario:
 
@@ -435,11 +457,11 @@ class JobRunner(IJobRunner):
         if job.plugin != "shell":
             # TRANSLATORS: please keep 'plugin' untranslated
             raise ValueError(_("bad job plugin value"))
-        return self._just_run_command(job, config)
+        return self._just_run_command(job, job_state, config).get_result()
 
-    def run_attachment_job(self, job, config):
+    def run_attachment_job(self, job, job_state, config):
         """
-        Method called to run a job with plugin field equal to 'attachment'
+        Method called to run a job with plugin field equal to 'attachment'.
 
         The 'attachment' job implements the following scenario:
 
@@ -460,11 +482,11 @@ class JobRunner(IJobRunner):
         if job.plugin != "attachment":
             # TRANSLATORS: please keep 'plugin' untranslated
             raise ValueError(_("bad job plugin value"))
-        return self._just_run_command(job, config)
+        return self._just_run_command(job, job_state, config).get_result()
 
-    def run_resource_job(self, job, config):
+    def run_resource_job(self, job, job_state, config):
         """
-        Method called to run a job with plugin field equal to 'resource'
+        Method called to run a job with plugin field equal to 'resource'.
 
         The 'resource' job implements the following scenario:
 
@@ -486,11 +508,11 @@ class JobRunner(IJobRunner):
         if job.plugin != "resource":
             # TRANSLATORS: please keep 'plugin' untranslated
             raise ValueError(_("bad job plugin value"))
-        return self._just_run_command(job, config)
+        return self._just_run_command(job, job_state, config).get_result()
 
-    def run_local_job(self, job, config):
+    def run_local_job(self, job, job_state, config):
         """
-        Method called to run a job with plugin field equal to 'local'
+        Method called to run a job with plugin field equal to 'local'.
 
         The 'local' job implements the following scenario:
 
@@ -512,11 +534,11 @@ class JobRunner(IJobRunner):
         if job.plugin != "local":
             # TRANSLATORS: please keep 'plugin' untranslated
             raise ValueError(_("bad job plugin value"))
-        return self._just_run_command(job, config)
+        return self._just_run_command(job, job_state, config).get_result()
 
-    def run_manual_job(self, job, config):
+    def run_manual_job(self, job, job_state, config):
         """
-        Method called to run a job with plugin field equal to 'manual'
+        Method called to run a job with plugin field equal to 'manual'.
 
         The 'manual' job implements the following scenario:
 
@@ -538,11 +560,11 @@ class JobRunner(IJobRunner):
         if job.plugin != "manual":
             # TRANSLATORS: please keep 'plugin' untranslated
             raise ValueError(_("bad job plugin value"))
-        return MemoryJobResult({'outcome': IJobResult.OUTCOME_UNDECIDED})
+        return JobResultBuilder(outcome=IJobResult.OUTCOME_UNDECIDED).get_result()
 
-    def run_user_interact_job(self, job, config):
+    def run_user_interact_job(self, job, job_state, config):
         """
-        Method called to run a job with plugin field equal to 'user-interact'
+        Method called to run a job with plugin field equal to 'user-interact'.
 
         The 'user-interact' job implements the following scenario:
 
@@ -579,11 +601,11 @@ class JobRunner(IJobRunner):
         if job.plugin != "user-interact":
             # TRANSLATORS: please keep 'plugin' untranslated
             raise ValueError(_("bad job plugin value"))
-        return self._just_run_command(job, config)
+        return self._just_run_command(job, job_state, config).get_result()
 
-    def run_user_verify_job(self, job, config):
+    def run_user_verify_job(self, job, job_state, config):
         """
-        Method called to run a job with plugin field equal to 'user-verify'
+        Method called to run a job with plugin field equal to 'user-verify'.
 
         The 'user-verify' job implements the following scenario:
 
@@ -624,15 +646,14 @@ class JobRunner(IJobRunner):
             # TRANSLATORS: please keep 'plugin' untranslated
             raise ValueError(_("bad job plugin value"))
         # Run the command
-        result_cmd = self._just_run_command(job, config)
+        result_builder = self._just_run_command(job, job_state, config)
         # Maybe ask the user
-        result_cmd.outcome = IJobResult.OUTCOME_UNDECIDED
-        return result_cmd
+        result_builder.outcome = IJobResult.OUTCOME_UNDECIDED
+        return result_builder.get_result()
 
-    def run_user_interact_verify_job(self, job, config):
+    def run_user_interact_verify_job(self, job, job_state, config):
         """
-        Method called to run a job with plugin field equal to
-        'user-interact-verify'
+        Method for running jobs with plugin equal to 'user-interact-verify'.
 
         The 'user-interact-verify' job implements the following scenario:
 
@@ -673,10 +694,81 @@ class JobRunner(IJobRunner):
             # TRANSLATORS: please keep 'plugin' untranslated
             raise ValueError(_("bad job plugin value"))
         # Run the command
-        result_cmd = self._just_run_command(job, config)
+        result_builder = self._just_run_command(job, job_state, config)
         # Maybe ask the user
-        result_cmd.outcome = IJobResult.OUTCOME_UNDECIDED
-        return result_cmd
+        result_builder.outcome = IJobResult.OUTCOME_UNDECIDED
+        return result_builder.get_result()
+
+    def run_qml_job(self, job, job_state, config):
+        """
+        Method called to run a job with plugin field equal to 'qml'.
+
+        The 'qml' job implements the following scenario:
+
+        * Maybe display the description to the user
+        * Run qmlscene with provided test and wait for it to finish
+        * Decide on the outcome based on the result object returned by qml
+          shell
+        * The method ends here
+
+        .. note::
+            QML jobs are fully manual jobs with graphical user interface
+            implemented in QML. They implement proposal described in CEP-5.
+        """
+        if job.plugin != "qml":
+            # TRANSLATORS: please keep 'plugin' untranslated
+            raise ValueError(_("bad job plugin value"))
+        try:
+            ctrl = self._get_ctrl_for_job(job)
+        except LookupError:
+            return JobResultBuilder(
+                outcome=IJobResult.OUTCOME_NOT_SUPPORTED,
+                comments=_('No suitable execution controller is available)')
+            ).get_result()
+        # Run the embedded command
+        start_time = time.time()
+        delegate, io_log_gen = self._prepare_io_handling(job, config)
+        # Create a subprocess.Popen() like object that uses the delegate
+        # system to observe all IO as it occurs in real time.
+        delegate_cls = self._get_delegate_cls(config)
+        extcmd_popen = delegate_cls(delegate)
+        # Stream all IOLogRecord entries to disk
+        record_path = os.path.join(
+            self._jobs_io_log_dir, "{}.record.gz".format(
+                slugify(job.id)))
+        with gzip.open(record_path, mode='wb') as gzip_stream, \
+                io.TextIOWrapper(
+                    gzip_stream, encoding='UTF-8') as record_stream:
+            writer = IOLogRecordWriter(record_stream)
+            io_log_gen.on_new_record.connect(writer.write_record)
+            try:
+                # Start the process and wait for it to finish getting the
+                # result code. This will actually call a number of callbacks
+                # while the process is running. It will also spawn a few
+                # threads although all callbacks will be fired from a single
+                # thread (which is _not_ the main thread)
+                logger.debug(
+                    _("job[%s] starting qml shell: %s"), job.id, job.qml_file)
+                # Run the job command using extcmd
+                return_code = self._run_extcmd(job, job_state, config,
+                                               extcmd_popen, ctrl)
+                logger.debug(
+                    _("job[%s] shell return code: %r"), job.id, return_code)
+            finally:
+                io_log_gen.on_new_record.disconnect(writer.write_record)
+        execution_duration = time.time() - start_time
+        # Convert the return of the command to the outcome of the job
+        if return_code == 0:
+            outcome = IJobResult.OUTCOME_PASS
+        else:
+            outcome = IJobResult.OUTCOME_FAIL
+        # Create a result object and return it
+        return JobResultBuilder(
+            outcome=outcome,
+            return_code=return_code,
+            io_log_filename=record_path,
+            execution_duration=execution_duration
+        ).get_result()
 
     def _get_dry_run_result(self, job):
         """
@@ -685,42 +777,42 @@ class JobRunner(IJobRunner):
         Returns a result that is used when running in dry-run mode (where we
         don't really test anything)
         """
-        return MemoryJobResult({
-            'outcome': IJobResult.OUTCOME_SKIP,
-            'comments': _("Job skipped in dry-run mode")
-        })
+        return JobResultBuilder(
+            outcome=IJobResult.OUTCOME_SKIP,
+            comments=_("Job skipped in dry-run mode")
+        ).get_result()
 
-    def _just_run_command(self, job, config):
+    def _just_run_command(self, job, job_state, config):
         """
         Internal method of JobRunner.
 
-        Runs the command embedded in the job and returns the DiskJobResult that
-        describes the result. If the command cannot be executed it returns
-        a MemoryJobResult instead.
+        Runs the command embedded in the job and returns a JobResultBuilder
+        that describes the result.
         """
         try:
             ctrl = self._get_ctrl_for_job(job)
         except LookupError:
-            return MemoryJobResult({
-                'outcome': IJobResult.OUTCOME_NOT_SUPPORTED,
-                'comment': _('No suitable execution controller is available)'),
-            })
+            return JobResultBuilder(
+                outcome=IJobResult.OUTCOME_NOT_SUPPORTED,
+                comments=_('No suitable execution controller is available)'))
         # Run the embedded command
         start_time = time.time()
-        return_code, record_path = self._run_command(job, config, ctrl)
+        return_code, record_path = self._run_command(
+            job, job_state, config, ctrl)
         execution_duration = time.time() - start_time
         # Convert the return of the command to the outcome of the job
         if return_code == 0:
             outcome = IJobResult.OUTCOME_PASS
+        elif return_code < 0:
+            outcome = IJobResult.OUTCOME_CRASH
         else:
             outcome = IJobResult.OUTCOME_FAIL
         # Create a result object and return it
-        return DiskJobResult({
-            'outcome': outcome,
-            'return_code': return_code,
-            'io_log_filename': record_path,
-            'execution_duration': execution_duration
-        })
+        return JobResultBuilder(
+            outcome=outcome,
+            return_code=return_code,
+            io_log_filename=record_path,
+            execution_duration=execution_duration)
 
     def _prepare_io_handling(self, job, config):
         ui_io_delegate = self._command_io_delegate
@@ -768,7 +860,7 @@ class JobRunner(IJobRunner):
         # One listener appends each record to an array
         return delegate, io_log_gen
 
-    def _run_command(self, job, config, ctrl):
+    def _run_command(self, job, job_state, config, ctrl):
         """
         Run the shell command associated with the specified job.
 
@@ -784,7 +876,11 @@ class JobRunner(IJobRunner):
         # Create a subprocess.Popen() like object that uses the delegate
         # system to observe all IO as it occurs in real time.
         delegate_cls = self._get_delegate_cls(config)
-        extcmd_popen = delegate_cls(delegate)
+        flags = 0
+        # Use chunked IO for jobs that explicitly request this
+        if 'use-chunked-io' in job.get_flag_set():
+            flags |= extcmd.CHUNKED_IO
+        extcmd_popen = delegate_cls(delegate, flags=flags)
         # Stream all IOLogRecord entries to disk
         record_path = os.path.join(
             self._jobs_io_log_dir, "{}.record.gz".format(
@@ -803,24 +899,25 @@ class JobRunner(IJobRunner):
                 logger.debug(
                     _("job[%s] starting command: %s"), job.id, job.command)
                 # Run the job command using extcmd
-                return_code = self._run_extcmd(job, config, extcmd_popen, ctrl)
+                return_code = self._run_extcmd(
+                    job, job_state, config, extcmd_popen, ctrl)
                 logger.debug(
                     _("job[%s] command return code: %r"), job.id, return_code)
             finally:
                 io_log_gen.on_new_record.disconnect(writer.write_record)
         return return_code, record_path
 
-    def _run_extcmd(self, job, config, extcmd_popen, ctrl):
+    def _run_extcmd(self, job, job_state, config, extcmd_popen, ctrl):
         ctrl.on_leftover_files.connect(self.on_leftover_files)
         try:
-            return ctrl.execute_job(
-                job, config, self._session_dir, extcmd_popen)
+            return ctrl.execute_job(job, job_state, config, self._session_dir,
+                                    extcmd_popen)
         finally:
             ctrl.on_leftover_files.disconnect(self.on_leftover_files)
 
     def _get_ctrl_for_job(self, job):
         """
-        Get the execution controller most applicable to run this job
+        Get the execution controller most applicable to run this job.
 
         :param job:
             A job definition to run
@@ -847,7 +944,7 @@ class JobRunner(IJobRunner):
             ctrl.__class__.__name__, score, job.id)
         return ctrl
 
-    @Signal.define
+    @morris.signal
     def on_leftover_files(self, job, config, cwd_dir, leftovers):
         """
         Handle any files left over by the execution of a job definition.
