@@ -35,6 +35,7 @@ import re
 
 from plainbox.abc import IJobQualifier
 from plainbox.i18n import gettext as _
+from plainbox.impl import pod
 from plainbox.impl.secure.origin import FileTextSource
 from plainbox.impl.secure.origin import Origin
 from plainbox.impl.secure.origin import UnknownTextSource
@@ -369,21 +370,20 @@ class FieldQualifier(SimpleQualifier):
             self._inclusive)
 
 
-class CompositeQualifier(IJobQualifier):
+class CompositeQualifier(pod.POD):
     """
     A JobQualifier that has qualifies jobs matching any inclusive qualifiers
     while not matching all of the exclusive qualifiers
     """
 
-    def __init__(self, qualifier_list):
-        self.qualifier_list = qualifier_list
+    qualifier_list = pod.Field("qualifier_list", list, pod.MANDATORY)
 
     @property
     def is_primitive(self):
         return False
 
     def designates(self, job):
-        return self.get_vote(job) == self.VOTE_INCLUDE
+        return self.get_vote(job) == IJobQualifier.VOTE_INCLUDE
 
     def get_vote(self, job):
         """
@@ -405,7 +405,7 @@ class CompositeQualifier(IJobQualifier):
                 qualifier.get_vote(job)
                 for qualifier in self.qualifier_list])
         else:
-            return self.VOTE_IGNORE
+            return IJobQualifier.VOTE_IGNORE
 
     def get_primitive_qualifiers(self):
         return get_flat_primitive_qualifier_list(self.qualifier_list)
@@ -415,6 +415,9 @@ class CompositeQualifier(IJobQualifier):
         raise NonPrimitiveQualifierOrigin
 
 
+IJobQualifier.register(CompositeQualifier)
+
+
 class NonPrimitiveQualifierOrigin(Exception):
     """
     Exception raised when IJobQualifier.origin is meaningless as it is being
@@ -583,28 +586,28 @@ class WhiteList(CompositeQualifier):
             (1-based) and pattern_list is a list of regular expression strings
             parsed from the whitelist.
         """
-        pattern_list = []
-        lineno = 0
-        # Load the file
-        for lineno, line in enumerate(text.splitlines(), 1):
-            # Strip shell-style comments if there are any
-            try:
-                index = line.index("#")
-            except ValueError:
-                pass
-            else:
-                line = line[:index]
-            # Strip whitespace
-            line = line.strip()
-            # Skip empty lines (especially after stripping comments)
-            if line == "":
-                continue
-            # Surround the pattern with ^ and $
-            # so that it wont just match a part of the job name.
-            regexp_pattern = r"^{pattern}$".format(pattern=line)
-            # Accumulate patterns into the list
-            pattern_list.append(regexp_pattern)
-        return pattern_list, lineno
+        from plainbox.impl.xparsers import Re
+        from plainbox.impl.xparsers import Visitor
+        from plainbox.impl.xparsers import WhiteList
+
+        class WhiteListVisitor(Visitor):
+
+            def __init__(self):
+                self.pattern_list = []
+                self.lineno = 0
+
+            def visit_Re_node(self, node: Re):
+                self.pattern_list.append(r"^{}$".format(node.text.strip()))
+                self.lineno = max(node.lineno, self.lineno)
+                return super().generic_visit(node)
+
+            visit_ReFixed_node = visit_Re_node
+            visit_RePattern_node = visit_Re_node
+            visit_ReErr_node = visit_Re_node
+
+        visitor = WhiteListVisitor()
+        visitor.visit(WhiteList.parse(text))
+        return visitor.pattern_list, visitor.lineno
 
     @classmethod
     def _load_patterns(cls, pathname):
@@ -706,7 +709,13 @@ def select_jobs(job_list, qualifier_list):
             # optimize the super-common case where a qualifier refers to
             # a specific job by using the id_to_index_map to instantly
             # perform the requested operation on a single job
-            j_index = id_to_index_map[qualifier.matcher.value]
+            try:
+                j_index = id_to_index_map[qualifier.matcher.value]
+            except KeyError:
+                # The lookup can fail if the pattern is a constant reference to
+                # a generated job that doens't exist yet. To maintain correctness
+                # we should just ignore it, as it would not match anything yet.
+                continue
             job = job_list[j_index]
             vote = qualifier.get_vote(job)
             if vote == IJobQualifier.VOTE_INCLUDE:
