@@ -86,8 +86,10 @@ __all__ = [
     'SessionStorage',
     'SessionStorageRepository',
     'UndesiredJobReadinessInhibitor',
+    'InhibitionCause',
 ]
 
+from plainbox.impl.session.jobs import InhibitionCause
 from plainbox.impl.session.jobs import JobReadinessInhibitor
 from plainbox.impl.session.jobs import JobState
 from plainbox.impl.session.jobs import UndesiredJobReadinessInhibitor
@@ -95,7 +97,6 @@ from plainbox.impl.session.legacy import SessionStateLegacyAPI
 from plainbox.impl.session.manager import SessionManager
 from plainbox.impl.session.resume import SessionPeekHelper
 from plainbox.impl.session.resume import SessionResumeError
-from plainbox.impl.session.state import SessionDeviceContext
 from plainbox.impl.session.state import SessionMetaData
 from plainbox.impl.session.state import SessionState
 from plainbox.impl.session.storage import SessionStorage
