@@ -30,11 +30,14 @@ from :class:`~plainbox.impl.session.storage.SessionStorageRepository`,
 and :class:`~plainbox.impl.session.suspend.SessionResumeHelper`.
 """
 
+from collections import OrderedDict
+import contextlib
 import errno
 import logging
 import os
 
 from plainbox.i18n import gettext as _, ngettext
+from plainbox.impl import pod
 from plainbox.impl.session.resume import SessionResumeHelper
 from plainbox.impl.session.state import SessionDeviceContext
 from plainbox.impl.session.state import SessionState
@@ -42,12 +45,14 @@ from plainbox.impl.session.storage import LockedStorageError
 from plainbox.impl.session.storage import SessionStorage
 from plainbox.impl.session.storage import SessionStorageRepository
 from plainbox.impl.session.suspend import SessionSuspendHelper
-from plainbox.impl.signal import Signal
+from plainbox.impl.unit.testplan import TestPlanUnit
+from plainbox.public import get_providers
+from plainbox.vendor import morris
 
 logger = logging.getLogger("plainbox.session.manager")
 
 
-class WellKnownDirsHelper:
+class WellKnownDirsHelper(pod.POD):
     """
     Helper class that knows about well known directories for SessionStorage.
 
@@ -56,17 +61,11 @@ class WellKnownDirsHelper:
     method :meth:`populate()` to create all of those directories, if needed.
     """
 
-    def __init__(self, storage):
-        # assert isinstance(storage, SessionStorage)
-        self._storage = storage
-
-    @property
-    def storage(self):
-        """
-        :class:`~plainbox.impl.session.storage.SessionStorage` associated with
-        this helper
-        """
-        return self._storage
+    storage = pod.Field(
+        doc="SessionStorage associated with this helper",
+        type=SessionStorage,
+        initial=pod.MANDATORY,
+        assign_filter_list=[pod.const, pod.typed])
 
     def populate(self):
         """
@@ -92,7 +91,18 @@ class WellKnownDirsHelper:
         return os.path.join(self.storage.location, "io-logs")
 
 
-class SessionManager:
+def at_most_one_context_filter(
+    instance: pod.POD, field: pod.Field, old: "Any", new: "Any"
+):
+    if len(new) > 1:
+        raise ValueError(_(
+            "session manager currently doesn't support sessions"
+            " involving multiple devices (a.k.a multi-node testing)"
+        ))
+    return new
+
+
+class SessionManager(pod.POD):
     """
     Manager class for coupling SessionStorage with SessionState.
 
@@ -102,43 +112,52 @@ class SessionManager:
     associated with each :class:`SessionManager`.
     """
 
-    def __init__(self, device_context_list, storage):
-        """
-        Initialize a manager with a list of session device context objects and
-        a storage object that will be used for managing storage for all testing
-        related to those objects.
-
-        :param device_context_list:
-            A list of SessionDeviceContext instances. Currently at most one
-            object may exist in that list but this restriction will be lifted
-            later on without changing the interface.
-        :param storage:
-            A SessionStorage instance.
-        """
-        if len(device_context_list) > 1:
-            self._too_many_device_context_objects()
-        self._device_context_list = device_context_list
-        self._storage = storage
-        logger.debug(
-            # TRANSLATORS: please don't translate 'SessionManager'
-            # and 'device_context_list'
-            _("Created SessionManager with device_context_list:%r"
-              " and storage:%r"), device_context_list, storage)
-
-    @property
-    def device_context_list(self):
-        """
+    device_context_list = pod.Field(
+        doc="""
         A list of session device context objects
 
         .. note::
-            You must not modify the return value.
+            You must not modify this field directly.
 
             This is not enforced but please use the
             :meth:`add_device_context()` or :meth:`remove_device_context()` if
             you want to manipulate the list.  Currently you cannot reorder the
             list of context objects.
-        """
-        return self._device_context_list
+        """,
+        type=list,
+        initial=pod.MANDATORY,
+        assign_filter_list=[
+            pod.typed, pod.typed.sequence(SessionDeviceContext),
+            pod.const, at_most_one_context_filter])
+
+    storage = pod.Field(
+        doc="A SesssionStorage instance",
+        type=SessionStorage,
+        initial=pod.MANDATORY,
+        assign_filter_list=[pod.typed, pod.const])
+
+    def _on_test_plans_changed(self, old: "Any", new: "Any") -> None:
+        self._propagate_test_plans()
+
+    test_plans = pod.Field(
+        doc="""
+        Test plans that this session is processing.
+
+        This field contains a tuple of test plans that are active in the
+        session. Any changes here are propagated to each device context
+        participating in the session. This in turn makes all of the overrides
+        defined by those test plans effective.
+
+        .. note::
+            Currently there is no facitly that would allow to use this field to
+            drive test execution. Such facility is likely to be added later.
+        """,
+        type=tuple,
+        initial=(),
+        notify=True,
+        notify_fn=_on_test_plans_changed,
+        assign_filter_list=[
+            pod.typed, pod.typed.sequence(TestPlanUnit), pod.unique])
 
     @property
     def default_device_context(self):
@@ -154,8 +173,8 @@ class SessionManager:
             present in the session. This is never the case for applications
             using the single-device APIs.
         """
-        return (self._device_context_list[0]
-                if len(self._device_context_list) > 0 else None)
+        return (self.device_context_list[0]
+                if len(self.device_context_list) > 0 else None)
 
     @property
     def state(self):
@@ -166,14 +185,6 @@ class SessionManager:
         if self.default_device_context is not None:
             return self.default_device_context.state
 
-    @property
-    def storage(self):
-        """
-        :class:`~plainbox.impl.session.storage.SessionStorage` associated with
-        this manager
-        """
-        return self._storage
-
     @classmethod
     def create(cls, repo=None, legacy_mode=False):
         """
@@ -347,7 +358,8 @@ class SessionManager:
         :meth:`SessionManager.load_session()`.
         """
         logger.debug("SessionManager.checkpoint()")
-        data = SessionSuspendHelper().suspend(self.state)
+        data = SessionSuspendHelper().suspend(
+            self.state, self.storage.location)
         logger.debug(
             ngettext(
                 "Saving %d byte of checkpoint data to %r",
@@ -383,14 +395,14 @@ class SessionManager:
         This method fires the :meth:`on_device_context_added()` signal
         """
         if any(other_context.device == context.device
-               for other_context in self._device_context_list):
+               for other_context in self.device_context_list):
             raise ValueError(
                 _("attmpting to add a context for device {} which is"
                   " already represented in this session"
                   " manager").format(context.device))
-        if len(self._device_context_list) > 0:
+        if len(self.device_context_list) > 0:
             self._too_many_device_context_objects()
-        self._device_context_list.append(context)
+        self.device_context_list.append(context)
         self.on_device_context_added(context)
 
     def add_local_device_context(self):
@@ -412,14 +424,14 @@ class SessionManager:
 
         This method fires the :meth:`on_device_context_removed()` signal
         """
-        if context not in self._device_context_list:
+        if context not in self.device_context_list:
             raise ValueError(_(
                 "attempting to remove a device context not present in this"
                 " session manager"))
-        self._device_context_list.remove(context)
+        self.device_context_list.remove(context)
         self.on_device_context_removed(context)
 
-    @Signal.define
+    @morris.signal
     def on_device_context_added(self, context):
         """
         Signal fired when a session device context object is added
@@ -427,8 +439,9 @@ class SessionManager:
         logger.debug(
             _("Device context %s added to session manager %s"),
             context, self)
+        self._propagate_test_plans()
 
-    @Signal.define
+    @morris.signal
     def on_device_context_removed(self, context):
         """
         Signal fired when a session device context object is removed
@@ -436,9 +449,108 @@ class SessionManager:
         logger.debug(
             _("Device context %s removed from session manager %s"),
             context, self)
+        self._propagate_test_plans()
 
     def _too_many_device_context_objects(self):
         raise ValueError(_(
             "session manager currently doesn't support sessions"
             " involving multiple devices (a.k.a multi-node testing)"
         ))
+
+    def _propagate_test_plans(self):
+        logger.debug(_("Propagating test plans to all devices"))
+        test_plans = self.test_plans
+        for context in self.device_context_list:
+            context.set_test_plan_list(test_plans)
+
+    @property
+    def exporter_map(self):
+        """ Map from exporter id to the corresponding exporter unit. """
+        exporter_map = OrderedDict()
+        for unit in self.state.unit_list:
+            if unit.Meta.name == 'exporter':
+                support = unit.support
+                if support:
+                    exporter_map[unit.id] = support
+        # Patch exporter map to expose short names
+        legacy_mapping = {
+            '2013.com.canonical.plainbox::hexr': 'xml',
+            '2013.com.canonical.plainbox::html': 'html',
+            '2013.com.canonical.plainbox::json': 'json',
+            '2013.com.canonical.plainbox::rfc822': 'rfc822',
+            '2013.com.canonical.plainbox::text': 'text',
+            '2013.com.canonical.plainbox::xlsx': 'xlsx'
+        }
+        for new_id, legacy_id in legacy_mapping.items():
+            if new_id in exporter_map:
+                exporter_map[legacy_id] = exporter_map[new_id]
+        return exporter_map
+
+    def create_exporter(self, exporter_id, option_list=(), strict=True):
+        """
+        Create an exporter object with the specified name and options.
+
+        :param exporter_id:
+            Identifier of the exporter unit (which must have been loaded
+            into the session device context of the first device). For
+            backwards compatibility this can also be any of the legacy
+            identifiers ``xml``, ``html``, ``json``, ``rfc822``, ``text`` or
+            ``xlsx``.
+        :param option_list:
+            (optional) A list of options to pass to the exporter. Each option
+            is a string. Some strings may be of form 'key=value' but those are
+            handled by each exporter separately. By default an empty tuple is
+            used so no special options are enabled.
+        :param strict:
+            (optional) Strict mode, in this mode ``option_list`` must not
+            contain any options that are unrecognized by the exporter. Since
+            many options (but not all) are shared among various exporters,
+            using non-strict mode might make it easier to use a single superset
+            of options to all exporters and let them silently ignore those that
+            they don't understand.
+        :raises LookupError:
+            If the exporter identifier cannot be found. Note that this might
+            indicate that appropriate provider has not been loaded yet.
+        :returns:
+            A ISessionStateExporter instance with appropriate configuration.
+        """
+        exporter_support = self.exporter_map[exporter_id]
+        if not strict:
+            # In non-strict mode silently discard unsupported options.
+            supported_options = frozenset(
+                exporter_support.exporter_cls.supported_option_list)
+            option_list = [
+                item for item in option_list if item in supported_options
+            ]
+        return exporter_support.exporter_cls(
+            option_list, exporter_unit=exporter_support)
+
+    @classmethod
+    @contextlib.contextmanager
+    def get_throwaway_manager(cls, provider_list=None):
+        """
+        Create a temporary session manager.
+
+        :param provider_list:
+            (optional) A list of providers to put into the session manager. By
+            default all known providers are added. You can use this argument to
+            customize the behaviour beyond defaults.
+        :returns:
+            A new SessionManager object that will be destroyed when the context
+            manager is left.
+
+        This method can be used to create a throw-away session manager which is
+        not really meant for running jobs but can be useful to access exporters
+        and other objects stored in providers.
+        """
+        if provider_list is None:
+            provider_list = get_providers()
+            manager = cls.create()
+        try:
+            manager.add_local_device_context()
+            device_context = manager.default_device_context
+            for provider in provider_list:
+                device_context.add_provider(provider)
+            yield manager
+        finally:
+            manager.destroy()
