@@ -1,13 +1,12 @@
 # This file is part of Checkbox.
 #
-# Copyright 2012, 2013 Canonical Ltd.
+# Copyright 2012-2015 Canonical Ltd.
 # Written by:
 #   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
 #
 # Checkbox is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3,
 # as published by the Free Software Foundation.
-
 #
 # Checkbox is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,13 +15,17 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
-
 """
+Session State Handling.
+
 :mod:`plainbox.impl.session.state` -- session state handling
 ============================================================
 """
+import collections
 import logging
+import re
 
+from plainbox.abc import IJobResult
 from plainbox.i18n import gettext as _
 from plainbox.impl import deprecated
 from plainbox.impl.depmgr import DependencyDuplicateError
@@ -30,14 +33,16 @@ from plainbox.impl.depmgr import DependencyError
 from plainbox.impl.depmgr import DependencySolver
 from plainbox.impl.session.jobs import JobState
 from plainbox.impl.session.jobs import UndesiredJobReadinessInhibitor
-from plainbox.impl.signal import Signal
 from plainbox.impl.unit.job import JobDefinition
+from plainbox.impl.unit.testplan import TestPlanUnitSupport
+from plainbox.vendor import morris
 
 
 logger = logging.getLogger("plainbox.session.state")
 
 
 class SessionMetaData:
+
     """
     Class representing non-critical state of the session.
 
@@ -64,6 +69,7 @@ class SessionMetaData:
 
     def __init__(self, title=None, flags=None, running_job_name=None,
                  app_blob=None, app_id=None):
+        """ Initialize a new session state meta-data object. """
         if flags is None:
             flags = []
         self._title = title
@@ -73,6 +79,7 @@ class SessionMetaData:
         self._app_id = app_id
 
     def __repr__(self):
+        """ Get the representation of the session state meta-data. """
         return "<{} title:{!r} flags:{!r} running_job_name:{!r}>".format(
             self.__class__.__name__, self.title, self.flags,
             self.running_job_name)
@@ -91,6 +98,7 @@ class SessionMetaData:
 
     @title.setter
     def title(self, title):
+        """ set the session title to the given value. """
         self._title = title
 
     @property
@@ -109,12 +117,13 @@ class SessionMetaData:
 
     @flags.setter
     def flags(self, flags):
+        """ set the session flags to the given set. """
         self._flags = flags
 
     @property
     def running_job_name(self):
         """
-        id of the running job
+        id of the running job.
 
         .. note::
             This property has a confusing name. It actually refers to job ID,
@@ -132,6 +141,7 @@ class SessionMetaData:
 
     @running_job_name.setter
     def running_job_name(self, running_job_name):
+        """ set the id of the running job. """
         self._running_job_name = running_job_name
 
     @property
@@ -149,6 +159,7 @@ class SessionMetaData:
 
     @app_blob.setter
     def app_blob(self, value):
+        """ set the application specific binary blob to the given value. """
         if value is not None and not isinstance(value, bytes):
             # TRANSLATORS: please don't translate app_blob, None and bytes
             raise TypeError(_("app_blob must be either None or bytes"))
@@ -157,7 +168,7 @@ class SessionMetaData:
     @property
     def app_id(self):
         """
-        Application identifier
+        Application identifier.
 
         A string identifying the application that stored app_blob. It is
         recommended to use reverse domain names or UUIDs.
@@ -166,6 +177,7 @@ class SessionMetaData:
 
     @app_id.setter
     def app_id(self, value):
+        """ Set the application identifier to the given value. """
         if value is not None and not isinstance(value, str):
             # TRANSLATORS: please don't translate app_blob, None and bytes
             raise TypeError(_("app_id must be either None or str"))
@@ -173,6 +185,7 @@ class SessionMetaData:
 
 
 class SessionDeviceContext:
+
     """
     Session context specific to a given device.
 
@@ -184,11 +197,16 @@ class SessionDeviceContext:
         are compatible with the device.
 
     :attr _unit_list:
-        A list of all the units known by this device. Initially it is idenitcal
+        A list of all the units known by this device. Initially it is identical
         to the union of all the units from ``_provider_list`` but it is in fact
         mutable and can be grown (or shrunk in some cases) when jobs are
         created at runtime.
 
+    :attr _test_plan_list:
+        A list of test plans that this device will be executing. This is stored
+        so that all job changes can automatically apply field overrides to job
+        state.
+
     :attr _device:
         Always None, this is a future extension point
 
@@ -200,13 +218,15 @@ class SessionDeviceContext:
 
     # Cache key that stores the list of execution controllers
     _CACHE_EXECUTION_CTRL_LIST = 'execution_controller_list'
+    # Cache key that stores the map of field overrides
+    _CACHE_OVERRIDE_MAP = 'override_map'
 
     def __init__(self, state=None):
         """
-        Initialize a new SessionDeviceContext
+        Initialize a new SessionDeviceContext.
 
         :param state:
-            An (optinal) state to use
+            An (optional) state to use
 
         Note that using an initial state will not cause any of the signals to
         fire for the initial list of units nor the list of providers (derived
@@ -232,6 +252,7 @@ class SessionDeviceContext:
                 unit.provider for unit in self._unit_list
             })
             self._state = state
+        self._test_plan_list = []
         # Connect SessionState's signals to fire our signals. This
         # way all manipulation done through the SessionState object
         # can be observed through the SessionDeviceContext object
@@ -306,9 +327,44 @@ class SessionDeviceContext:
         return self.compute_shared(
             self._CACHE_EXECUTION_CTRL_LIST, self._compute_execution_ctrl_list)
 
+    @property
+    def override_map(self):
+        """
+        A list of execution controllers applicable in this context.
+
+        :returns:
+            A list of IExecutionController objects
+
+        .. note::
+            The return value is different whenever a provider is added to the
+            context. If you have obtained this value in the past it may be
+            no longer accurate.
+        """
+        return self.compute_shared(
+            self._CACHE_OVERRIDE_MAP, self._compute_override_map)
+
+    def set_test_plan_list(self, test_plan_list: "List[TestPlanUnit]"):
+        """
+        Compute all of the effective job state values.
+
+        :param test_plan_list:
+            The list of test plans to consider
+
+        This method is intended to be called exactly once per session, after
+        the application determines the set of test plans it intends to execute.
+
+        The method will collect all of the override values exposed by all of
+        the test plans and apply them in sequence. Note that correct
+        applications must also perform micro-updates whenever a new test job is
+        added to the system.
+        """
+        self._test_plan_list = test_plan_list
+        self._invalidate_override_map()
+        self._bulk_override_update()
+
     def add_provider(self, provider, add_units=True):
         """
-        Add a provider to the context
+        Add a provider to the context.
 
         :param provider:
             The :class:`Provider1` to add
@@ -330,12 +386,12 @@ class SessionDeviceContext:
         self._provider_list.append(provider)
         self.on_provider_added(provider)
         if add_units:
-            for unit in provider.get_units()[0]:
+            for unit in provider.unit_list:
                 self.add_unit(unit)
 
     def add_unit(self, unit):
         """
-        Add a unit to the context
+        Add a unit to the context.
 
         :param unit:
             The :class:`Unit` to add.
@@ -350,12 +406,12 @@ class SessionDeviceContext:
         if unit in self._unit_list:
             raise ValueError(_("attempting to add the same unit twice"))
         self.state.add_unit(unit)
-        # NOTE: no need to fire the on_unit_added() signal becuse the state
+        # NOTE: no need to fire the on_unit_added() signal because the state
         # object and we've connected it to will fire our version.
 
     def remove_unit(self, unit):
         """
-        Remove an unit from the context
+        Remove an unit from the context.
 
         :param unit:
             The :class:`Unit` to remove.
@@ -366,12 +422,12 @@ class SessionDeviceContext:
             raise ValueError(
                 _("attempting to remove unit not in this context"))
         self.state.remove_unit(unit)
-        # NOTE: no need to fire the on_unit_removed() signal becuse the state
+        # NOTE: no need to fire the on_unit_removed() signal because the state
         # object and we've connected it to will fire our version.
 
     def get_ctrl_for_job(self, job):
         """
-        Get the execution controller most applicable to run this job
+        Get the execution controller most applicable to run this job.
 
         :param job:
             A job definition to run
@@ -402,28 +458,29 @@ class SessionDeviceContext:
             ctrl.__class__.__name__, score, job.id)
         return ctrl
 
-    @Signal.define
+    @morris.signal
     def on_provider_added(self, provider):
-        """
-        Signal sent whenever a provider is added to the context.
-        """
+        """ Signal sent whenever a provider is added to the context. """
         logger.info(_("Provider %s added to context %s"), provider, self)
         # Invalidate the list of execution controllers as they depend
         # on the accuracy of provider_list
         self._invalidate_execution_ctrl_list()
 
-    @Signal.define
+    @morris.signal
     def on_unit_added(self, unit):
-        """
-        Signal sent whenever a unit is added to the context.
-        """
+        """ Signal sent whenever a unit is added to the context. """
         logger.debug(_("Unit %s added to context %s"), unit, self)
+        if unit.Meta.name == 'job':
+            self.on_job_added(unit)
 
-    @Signal.define
+    @morris.signal
+    def on_job_added(self, job):
+        """ Signal sent whenever a new job unit is added to the context. """
+        self._override_update(job)
+
+    @morris.signal
     def on_unit_removed(self, unit):
-        """
-        Signal sent whenever a unit is removed from the context.
-        """
+        """ Signal sent whenever a unit is removed from the context. """
         logger.debug(_("Unit %s removed from context %s"), unit, self)
 
     def compute_shared(self, cache_key, func, *args, **kwargs):
@@ -451,13 +508,12 @@ class SessionDeviceContext:
         return self._shared_cache[cache_key]
 
     def invalidate_shared(self, cache_key):
+        """ Invalidate a cached shared value. """
         if cache_key in self._shared_cache:
             del self._shared_cache[cache_key]
 
     def _compute_execution_ctrl_list(self):
-        """
-        Internal method that computes the list of execution controllers
-        """
+        """ Compute the list of execution controllers. """
         # TODO: tie this with the upcoming device patches
         import sys
         if sys.platform == 'linux':
@@ -480,14 +536,42 @@ class SessionDeviceContext:
             return []
 
     def _invalidate_execution_ctrl_list(self, *args, **kwargs):
-        """
-        Internal method that invalidates the 'execution_controller_list' cache
-        key that is used to store the list of execution controllers.
-        """
+        """ Invalidate the list of execution controllers. """
         self.invalidate_shared(self._CACHE_EXECUTION_CTRL_LIST)
 
+    def _compute_override_map(self):
+        """ Compute the map of field overrides. """
+        override_map = collections.defaultdict(list)
+        for test_plan in self._test_plan_list:
+            support = TestPlanUnitSupport(test_plan)
+            for pattern, override_list in support.override_list:
+                override_map[pattern].extend(override_list)
+        return override_map
+
+    def _invalidate_override_map(self, *args, **kwargs):
+        """ Invalidate the cached field override map. """
+        self.invalidate_shared(self._CACHE_OVERRIDE_MAP)
+
+    def _bulk_override_update(self):
+        # NOTE: there is an O(N) algorithm for that solves this but it is more
+        # complicated than I was able to write without a hard-copy reference
+        # that describes it. I will improve this method once I complete the
+        # required research.
+        for job_state in self.state.job_state_map.values():
+            job = job_state.job
+            for pattern, override_list in self.override_map.items():
+                if re.match(pattern, job.id):
+                    job_state.apply_overrides(override_list)
+
+    def _override_update(self, job):
+        job_state = self.state.job_state_map[job.id]
+        for pattern, override_list in self.override_map.items():
+            if re.match(pattern, job.id):
+                job_state.apply_overrides(override_list)
+
 
 class SessionState:
+
     """
     Class representing all state needed during a single program session.
 
@@ -567,7 +651,7 @@ class SessionState:
     :ivar dict metadata: instance of :class:`SessionMetaData`
     """
 
-    @Signal.define
+    @morris.signal
     def on_job_state_map_changed(self):
         """
         Signal fired after job_state_map is changed in any way.
@@ -582,10 +666,10 @@ class SessionState:
         and can be easily looked at by the application.
         """
 
-    @Signal.define
+    @morris.signal
     def on_job_result_changed(self, job, result):
         """
-        Signal fired after a job get changed (set)
+        Signal fired after a job get changed (set).
 
         This signal is fired each time a result is presented to the session.
 
@@ -593,7 +677,7 @@ class SessionState:
         """
         logger.info(_("Job %s result changed to %r"), job, result)
 
-    @Signal.define
+    @morris.signal
     def on_job_added(self, job):
         """
         Signal sent whenever a job is added to the session.
@@ -601,7 +685,7 @@ class SessionState:
         This signal is fired **after** :meth:`on_job_state_map_changed()`
         """
 
-    @Signal.define
+    @morris.signal
     def on_job_removed(self, job):
         """
         Signal sent whenever a job is removed from the session.
@@ -609,17 +693,13 @@ class SessionState:
         This signal is fired **after** :meth:`on_job_state_map_changed()`
         """
 
-    @Signal.define
+    @morris.signal
     def on_unit_added(self, unit):
-        """
-        Signal sent whenever a unit is added to the session.
-        """
+        """ Signal sent whenever a unit is added to the session. """
 
-    @Signal.define
+    @morris.signal
     def on_unit_removed(self, unit):
-        """
-        Signal sent whenever a unit is removed from the session.
-        """
+        """ Signal sent whenever a unit is removed from the session. """
 
     def __init__(self, unit_list):
         """
@@ -740,7 +820,7 @@ class SessionState:
 
     def update_desired_job_list(self, desired_job_list):
         """
-        Update the set of desired jobs (that ought to run)
+        Update the set of desired jobs (that ought to run).
 
         This method can be used by the UI to recompute the dependency graph.
         The argument 'desired_job_list' is a list of jobs that should run.
@@ -761,17 +841,30 @@ class SessionState:
         # to remove a problematic job and re-try. The loop provides a stop
         # condition as we will eventually run out of jobs.
         problems = []
+        # Get a copy of all the jobs as we'll be removing elements from this
+        # list to come to a stable set in the loop below.
+        job_list = self._job_list[:]
         while self._desired_job_list:
             # XXX: it might be more efficient to incorporate this 'recovery
             # mode' right into the solver, this way we'd probably save some
             # resources or runtime complexity.
             try:
                 self._run_list = DependencySolver.resolve_dependencies(
-                    self._job_list, self._desired_job_list)
+                    job_list, self._desired_job_list)
             except DependencyError as exc:
                 # When a dependency error is detected remove the affected job
                 # form _desired_job_list and try again.
-                self._desired_job_list.remove(exc.affected_job)
+                if exc.affected_job in self._desired_job_list:
+                    # The job may have been removed by now:
+                    # https://bugs.launchpad.net/plainbox/+bug/1444126
+                    self._desired_job_list.remove(exc.affected_job)
+                if exc.affected_job in job_list:
+                    # If the affected job is in the job list, remove it from
+                    # the job list we're going to consider in the next run.
+                    # This is done so that if a job depends on a broken but
+                    # existing job, it won't constantly re-add the same broken
+                    # job over and over (so that the algorithm can stop).
+                    job_list.remove(exc.affected_job)
                 # Remember each problem, this can be presented by the UI
                 problems.append(exc)
                 continue
@@ -785,6 +878,8 @@ class SessionState:
 
     def get_estimated_duration(self, manual_overhead=30.0):
         """
+        Estimate the total duration of the session.
+
         Provide the estimated duration of the jobs that have been selected
         to run in this session (maintained by calling update_desired_job_list).
 
@@ -848,7 +943,7 @@ class SessionState:
     @deprecated('0.9', 'use the add_unit() method instead')
     def add_job(self, new_job, recompute=True):
         """
-        Add a new job to the session
+        Add a new job to the session.
 
         :param new_job:
             The job being added
@@ -881,7 +976,7 @@ class SessionState:
 
     def add_unit(self, new_unit, recompute=True):
         """
-        Add a new unit to the session
+        Add a new unit to the session.
 
         :param new_unit:
             The unit being added
@@ -953,7 +1048,7 @@ class SessionState:
 
     def remove_unit(self, unit, *, recompute=True):
         """
-        Remove an existing unit from the session
+        Remove an existing unit from the session.
 
         :param unit:
             The unit to remove
@@ -1005,15 +1100,13 @@ class SessionState:
 
     @property
     def unit_list(self):
-        """
-        List of all known units
-        """
+        """ List of all known units. """
         return self._unit_list
 
     @property
     def desired_job_list(self):
         """
-        List of jobs that are on the "desired to run" list
+        List of jobs that are on the "desired to run" list.
 
         This is a list, not a set, because the dependency solver algorithm
         retains as much of the original ordering as possible. Having said that,
@@ -1024,7 +1117,7 @@ class SessionState:
     @property
     def run_list(self):
         """
-        List of jobs that were intended to run, in the proper order
+        List of jobs that were intended to run, in the proper order.
 
         The order is a result of topological sorting of the desired_job_list.
         This value is recomputed when change_desired_run_list() is called. It
@@ -1034,23 +1127,60 @@ class SessionState:
 
     @property
     def job_state_map(self):
-        """
-        Map from job id to JobState that encodes the state of each job.
-        """
+        """ Map from job id to JobState associated with each job. """
         return self._job_state_map
 
     @property
     def resource_map(self):
-        """
-        Map from resource id to a list of resource records
-        """
+        """ Map from resource id to a list of resource records. """
         return self._resource_map
 
+    def get_outcome_stats(self):
+        """
+        Process the JobState map to get stats about the job outcomes.
+
+        :returns:
+            a mapping of "outcome": "total" key/value pairs
+
+        .. note::
+            Only the outcomes seen during this session are reported, not all
+            possible values (such as crash, not implemented, ...).
+        """
+        stats = collections.defaultdict(int)
+        for job_id, job_state in self.job_state_map.items():
+            if not job_state.result.outcome:
+                continue
+            stats[job_state.result.outcome] += 1
+        return stats
+
+    def get_certification_status_map(
+            self, outcome_filter=(IJobResult.OUTCOME_FAIL,),
+            certification_status_filter=('blocker',)
+    ):
+        """
+        Get a map of jobs that have a specific certification blocker status.
+
+        Filter the Job state map to only return items with given outcomes and
+        certification statuses.
+
+        :param outcome_filter:
+            Only consider job results with those outcome values
+        :param certification_status_filter:
+            Only consider jobs with those certification status values
+        :returns:
+            a Job state map only containing job with a given outcome and
+            certification status value.
+        """
+        return {
+            job_id: job_state for job_id, job_state in
+            self.job_state_map.items()
+            if job_state.result.outcome in outcome_filter
+            and job_state.effective_certification_status
+            in certification_status_filter}
+
     @property
     def metadata(self):
-        """
-        metadata object associated with this session state.
-        """
+        """ meta-data object associated with this session state. """
         return self._metadata
 
     def _recompute_job_readiness(self):
