@@ -129,8 +129,9 @@ class SessionStorageRepository:
             pathname = os.path.join(self.location, item)
             # Make sure not to follow any symlinks here
             stat_result = os.lstat(pathname)
-            # Consider directories only
-            if stat.S_ISDIR(stat_result.st_mode):
+            # Consider non-hidden directories that end with the word .session
+            if (not item.startswith(".") and item.endswith(".session")
+                    and stat.S_ISDIR(stat_result.st_mode)):
                 logger.debug(_("Found possible session in %r"), pathname)
                 session = SessionStorage(pathname)
                 session_list.append(session)
@@ -146,17 +147,42 @@ class SessionStorageRepository:
     @classmethod
     def get_default_location(cls):
         """
-        Compute the default location of the session state repository
+        Get the default location of the session state repository
 
-        :returns: ${XDG_CACHE_HOME:-$HOME/.cache}/plainbox/sessions
+        The default location is defined by ``$PLAINBOX_SESSION_REPOSITORY``
+        which must be a writable directory (created if needed) where plainbox
+        will keep its session data. The default location, if the environment
+        variable is not provided, is
+        ``${XDG_CACHE_HOME:-$HOME/.cache}/plainbox/sessions``
         """
-        # Pick XDG_CACHE_HOME from environment
-        xdg_cache_home = os.environ.get('XDG_CACHE_HOME')
-        # If not set or empty use the default ~/.cache/
-        if not xdg_cache_home:
-            xdg_cache_home = os.path.join(os.path.expanduser('~'), '.cache')
-        # Use a directory relative to XDG_CACHE_HOME
-        return os.path.join(xdg_cache_home, 'plainbox', 'sessions')
+        repo_dir = os.environ.get('PLAINBOX_SESSION_REPOSITORY')
+        if repo_dir is not None:
+            repo_dir = os.path.abspath(repo_dir)
+        else:
+            # Pick XDG_CACHE_HOME from environment
+            xdg_cache_home = os.environ.get('XDG_CACHE_HOME')
+            # If not set or empty use the default ~/.cache/
+            if not xdg_cache_home:
+                xdg_cache_home = os.path.join(
+                    os.path.expanduser('~'), '.cache')
+            # Use a directory relative to XDG_CACHE_HOME
+            repo_dir = os.path.join(xdg_cache_home, 'plainbox', 'sessions')
+        if (repo_dir is not None and os.path.exists(repo_dir)
+                and not os.path.isdir(repo_dir)):
+            logger.warning(
+                _("Session repository %s it not a directory"), repo_dir)
+            repo_dir = None
+        if (repo_dir is not None and os.path.exists(repo_dir)
+                and not os.access(repo_dir, os.W_OK)):
+            logger.warning(
+                _("Session repository %s is read-only"), repo_dir)
+            repo_dir = None
+        if repo_dir is None:
+            repo_dir = tempfile.mkdtemp()
+            logger.warning(
+                _("Using temporary directory %s as session repository"),
+                repo_dir)
+        return repo_dir
 
 
 class LockedStorageError(IOError):
@@ -574,7 +600,11 @@ class SessionStorage:
             logger.debug(
                 # TRANSLATORS: please don't translate fsync()
                 _("Calling fsync() on descriptor %d"), next_session_fd)
-            os.fsync(next_session_fd)
+            try:
+                os.fsync(next_session_fd)
+            except OSError as exc:
+                logger.warning(_("Cannot synchronize file %r: %s"),
+                               _next_session_pathname, exc)
         finally:
             # Close the new session file
             logger.debug(_("Closing descriptor %d"), next_session_fd)
@@ -674,7 +704,11 @@ class SessionStorage:
                 logger.debug(
                     # TRANSLATORS: please don't translate fsync()
                     _("Calling fsync() on descriptor %d"), next_session_fd)
-                os.fsync(next_session_fd)
+                try:
+                    os.fsync(next_session_fd)
+                except OSError as exc:
+                    logger.warning(_("Cannot synchronize file %r: %s"),
+                                   _next_session_pathname, exc)
             finally:
                 # Close the new session file
                 logger.debug(_("Closing descriptor %d"), next_session_fd)
@@ -705,7 +739,11 @@ class SessionStorage:
 
             # TRANSLATORS: please don't translate fsync()
             logger.debug(_("Calling fsync() on descriptor %d"), location_fd)
-            os.fsync(location_fd)
+            try:
+                os.fsync(location_fd)
+            except OSError as exc:
+                logger.warning(_("Cannot synchronize directory %r: %s"),
+                               self._location, exc)
         finally:
             # Close the location directory
             logger.debug(_("Closing descriptor %d"), location_fd)
@@ -779,7 +817,11 @@ class SessionStorage:
                 logger.debug(
                     # TRANSLATORS: please don't translate fsync()
                     _("Calling fsync() on descriptor %d"), next_session_fd)
-                os.fsync(next_session_fd)
+                try:
+                    os.fsync(next_session_fd)
+                except OSError as exc:
+                    logger.warning(_("Cannot synchronize file %r: %s"),
+                                   self._SESSION_FILE_NEXT, exc)
             finally:
                 # Close the new session file
                 logger.debug(_("Closing descriptor %d"), next_session_fd)
@@ -800,7 +842,7 @@ class SessionStorage:
                 # with O_EXCL flag.
                 logger.warning(
                     _("Unable to rename/overwrite %r to %r: %r"),
-                    _next_session_pathname, _session_pathname, exc)
+                    self._SESSION_FILE_NEXT, self._SESSION_FILE, exc)
                 # TRANSLATORS: unlinking as in deleting a file
                 logger.warning(_("Unlinking %r"), self._SESSION_FILE_NEXT)
                 os.unlink(self._SESSION_FILE_NEXT, dir_fd=location_fd)
@@ -812,7 +854,11 @@ class SessionStorage:
 
             # TRANSLATORS: please don't translate fsync()
             logger.debug(_("Calling fsync() on descriptor %d"), location_fd)
-            os.fsync(location_fd)
+            try:
+                os.fsync(location_fd)
+            except OSError as exc:
+                logger.warning(_("Cannot synchronize directory %r: %s"),
+                               self._location, exc)
         finally:
             # Close the location directory
             logger.debug(_("Closing descriptor %d"), location_fd)
