@@ -1,13 +1,12 @@
 # This file is part of Checkbox.
 #
-# Copyright 2012, 2013 Canonical Ltd.
+# Copyright 2012-2015 Canonical Ltd.
 # Written by:
 #   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
 #
 # Checkbox is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3,
 # as published by the Free Software Foundation.
-
 #
 # Checkbox is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,35 +15,42 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
-
 """
 plainbox.impl.test_session
 ==========================
 
 Test definitions for plainbox.impl.session module
 """
-
+from doctest import DocTestSuite
+from doctest import REPORT_NDIFF
 from unittest import TestCase
 
 from plainbox.abc import IExecutionController
 from plainbox.abc import IJobResult
 from plainbox.impl.depmgr import DependencyDuplicateError
 from plainbox.impl.depmgr import DependencyMissingError
+from plainbox.impl.depmgr import DependencyUnknownError
 from plainbox.impl.resource import Resource
 from plainbox.impl.result import MemoryJobResult
 from plainbox.impl.secure.origin import Origin
 from plainbox.impl.secure.providers.v1 import Provider1
 from plainbox.impl.secure.qualifiers import JobIdQualifier
-from plainbox.impl.session import JobReadinessInhibitor
+from plainbox.impl.session import InhibitionCause
 from plainbox.impl.session import SessionState
 from plainbox.impl.session import UndesiredJobReadinessInhibitor
 from plainbox.impl.session.state import SessionDeviceContext
 from plainbox.impl.session.state import SessionMetaData
-from plainbox.impl.signal import SignalInterceptorMixIn
 from plainbox.impl.testing_utils import make_job
 from plainbox.impl.unit.job import JobDefinition
 from plainbox.impl.unit.unit import Unit
 from plainbox.vendor import mock
+from plainbox.vendor.morris import SignalTestCase
+
+
+def load_tests(loader, tests, ignore):
+    tests.addTests(DocTestSuite(
+        'plainbox.impl.session.state', optionflags=REPORT_NDIFF))
+    return tests
 
 
 class SessionStateSmokeTests(TestCase):
@@ -75,6 +81,14 @@ class SessionStateSmokeTests(TestCase):
 class RegressionTests(TestCase):
     # Tests for bugfixes
 
+    def test_crash_on_missing_job(self):
+        """ http://pad.lv/1334296 """
+        A = make_job("A")
+        state = SessionState([])
+        problems = state.update_desired_job_list([A])
+        self.assertEqual(problems, [DependencyUnknownError(A)])
+        self.assertEqual(state.desired_job_list, [])
+
     def test_crash_in_update_desired_job_list(self):
         # This checks if a DependencyError can cause crash
         # update_desired_job_list() with a ValueError, in certain conditions.
@@ -110,6 +124,19 @@ class RegressionTests(TestCase):
             self.assertIs(call.exception.duplicate_job, different_A)
             self.assertIs(call.exception.affected_job, different_A)
 
+    def test_dont_remove_missing_jobs(self):
+        """ http://pad.lv/1444126 """
+        A = make_job("A", depends="B")
+        B = make_job("B", depends="C")
+        state = SessionState([A, B])
+        problems = state.update_desired_job_list([A, B])
+        self.assertEqual(problems, [
+            DependencyMissingError(B, 'C', 'direct'),
+            DependencyMissingError(A, 'B', 'direct'),
+        ])
+        self.assertEqual(state.desired_job_list, [])
+        self.assertEqual(state.run_list, [])
+
 
 class SessionStateAPITests(TestCase):
 
@@ -415,13 +442,13 @@ class SessionStateReactionToJobResultTests(TestCase):
         self.assertFalse(self.job_state('X').can_start())
         self.assertFalse(self.job_state('Y').can_start())
         self.assertEqual(self.job_inhibitor('A', 0).cause,
-                         JobReadinessInhibitor.UNDESIRED)
+                         InhibitionCause.UNDESIRED)
         self.assertEqual(self.job_inhibitor('R', 0).cause,
-                         JobReadinessInhibitor.UNDESIRED)
+                         InhibitionCause.UNDESIRED)
         self.assertEqual(self.job_inhibitor('X', 0).cause,
-                         JobReadinessInhibitor.UNDESIRED)
+                         InhibitionCause.UNDESIRED)
         self.assertEqual(self.job_inhibitor('Y', 0).cause,
-                         JobReadinessInhibitor.UNDESIRED)
+                         InhibitionCause.UNDESIRED)
 
     def test_desire_job_A_updates_state_map(self):
         # This function checks what happens when the job A becomes desired via
@@ -443,7 +470,7 @@ class SessionStateReactionToJobResultTests(TestCase):
         # pinpoints the related job and related expression.
         self.assertNotEqual(self.job_state('A').readiness_inhibitor_list, [])
         self.assertEqual(self.job_inhibitor('A', 0).cause,
-                         JobReadinessInhibitor.PENDING_RESOURCE)
+                         InhibitionCause.PENDING_RESOURCE)
         self.assertEqual(self.job_inhibitor('A', 0).related_job, self.job_R)
         self.assertEqual(self.job_inhibitor('A', 0).related_expression,
                          self.job_A_expr)
@@ -474,7 +501,7 @@ class SessionStateReactionToJobResultTests(TestCase):
         # update_desired_job_list() a will still have the UNDESIRED inhibitor
         # but it will no longer have the PENDING_RESOURCE inhibitor,
         self.assertEqual(self.job_inhibitor('A', 0).cause,
-                         JobReadinessInhibitor.UNDESIRED)
+                         InhibitionCause.UNDESIRED)
         # Now if we put A on the desired list this should clear the UNDESIRED
         # inhibitor and make A runnable.
         self.session.update_desired_job_list([self.job_A])
@@ -497,7 +524,7 @@ class SessionStateReactionToJobResultTests(TestCase):
         # relationships. While a result for job A was presented, job A is still
         # inhibited by the UNDESIRED inhibitor.
         self.assertEqual(self.job_inhibitor('A', 0).cause,
-                         JobReadinessInhibitor.UNDESIRED)
+                         InhibitionCause.UNDESIRED)
 
     def test_resource_job_with_broken_output(self):
         # This function checks how SessionState parses partially broken
@@ -542,7 +569,7 @@ class SessionStateReactionToJobResultTests(TestCase):
         # pinpoints the related job and related expression.
         self.assertNotEqual(self.job_state('X').readiness_inhibitor_list, [])
         self.assertEqual(self.job_inhibitor('X', 0).cause,
-                         JobReadinessInhibitor.PENDING_DEP)
+                         InhibitionCause.PENDING_DEP)
         self.assertEqual(self.job_inhibitor('X', 0).related_job, self.job_Y)
         self.assertFalse(self.job_state('X').can_start())
 
@@ -554,7 +581,7 @@ class SessionStateReactionToJobResultTests(TestCase):
         # on Y
         self.assertNotEqual(self.job_state('X').readiness_inhibitor_list, [])
         self.assertEqual(self.job_inhibitor('X', 0).cause,
-                         JobReadinessInhibitor.PENDING_DEP)
+                         InhibitionCause.PENDING_DEP)
         self.assertEqual(self.job_inhibitor('X', 0).related_job, self.job_Y)
         self.assertFalse(self.job_state('X').can_start())
         # When a failed Y result is presented X should switch to FAILED_DEP
@@ -564,7 +591,7 @@ class SessionStateReactionToJobResultTests(TestCase):
         # PENDING_DEP it had before. Everything else should stay as-is.
         self.assertNotEqual(self.job_state('X').readiness_inhibitor_list, [])
         self.assertEqual(self.job_inhibitor('X', 0).cause,
-                         JobReadinessInhibitor.FAILED_DEP)
+                         InhibitionCause.FAILED_DEP)
         self.assertEqual(self.job_inhibitor('X', 0).related_job, self.job_Y)
         self.assertFalse(self.job_state('X').can_start())
 
@@ -589,7 +616,7 @@ class SessionStateReactionToJobResultTests(TestCase):
         # Now A is inhibited by FAILED_RESOURCE
         self.assertNotEqual(self.job_state('A').readiness_inhibitor_list, [])
         self.assertEqual(self.job_inhibitor('A', 0).cause,
-                         JobReadinessInhibitor.FAILED_RESOURCE)
+                         InhibitionCause.FAILED_RESOURCE)
         self.assertEqual(self.job_inhibitor('A', 0).related_job, self.job_R)
         self.assertEqual(self.job_inhibitor('A', 0).related_expression,
                          self.job_A_expr)
@@ -634,8 +661,49 @@ class SessionStateReactionToJobResultTests(TestCase):
         job_foo = self.session.job_state_map['foo'].job
         self.assertTrue(job_foo.id, "foo")
         self.assertTrue(job_foo.plugin, "manual")
-        # It should be linked to the job L via the via attribute
-        self.assertTrue(job_foo.via, self.job_L.checksum)
+        # It should be linked to the job L via the via_job state attribute
+        self.assertIs(
+            self.session.job_state_map[job_foo.id].via_job, self.job_L)
+
+    def test_get_outcome_stats(self):
+        result_A = MemoryJobResult({'outcome': IJobResult.OUTCOME_PASS})
+        result_L = MemoryJobResult(
+            {'outcome': IJobResult.OUTCOME_NOT_SUPPORTED})
+        result_R = MemoryJobResult({'outcome': IJobResult.OUTCOME_FAIL})
+        result_Y = MemoryJobResult({'outcome': IJobResult.OUTCOME_FAIL})
+        self.session.update_job_result(self.job_A, result_A)
+        self.session.update_job_result(self.job_L, result_L)
+        self.session.update_job_result(self.job_R, result_R)
+        self.session.update_job_result(self.job_Y, result_Y)
+        self.assertEqual(self.session.get_outcome_stats(),
+                         {IJobResult.OUTCOME_PASS: 1,
+                          IJobResult.OUTCOME_NOT_SUPPORTED: 1,
+                          IJobResult.OUTCOME_FAIL: 2})
+
+    def test_get_certification_status_map(self):
+        result_A = MemoryJobResult({'outcome': IJobResult.OUTCOME_PASS})
+        self.session.update_job_result(self.job_A, result_A)
+        self.session.job_state_map[
+            self.job_A.id].effective_certification_status = 'foo'
+        self.assertEqual(self.session.get_certification_status_map(), {})
+        self.assertEqual(self.session.get_certification_status_map(
+            outcome_filter=(IJobResult.OUTCOME_PASS,),
+            certification_status_filter=('foo',)),
+            {self.job_A.id: self.session.job_state_map[self.job_A.id]})
+        result_Y = MemoryJobResult({'outcome': IJobResult.OUTCOME_FAIL})
+        self.session.job_state_map[
+            self.job_Y.id].effective_certification_status = 'bar'
+        self.assertEqual(self.session.get_certification_status_map(), {})
+        self.assertEqual(self.session.get_certification_status_map(
+            outcome_filter=(IJobResult.OUTCOME_PASS, IJobResult.OUTCOME_FAIL),
+            certification_status_filter=('foo', 'bar')),
+            {self.job_A.id: self.session.job_state_map[self.job_A.id]})
+        self.session.update_job_result(self.job_Y, result_Y)
+        self.assertEqual(self.session.get_certification_status_map(
+            outcome_filter=(IJobResult.OUTCOME_PASS, IJobResult.OUTCOME_FAIL),
+            certification_status_filter=('foo', 'bar')),
+            {self.job_A.id: self.session.job_state_map[self.job_A.id],
+             self.job_Y.id: self.session.job_state_map[self.job_Y.id]})
 
 
 class SessionMetadataTests(TestCase):
@@ -696,14 +764,15 @@ class SessionMetadataTests(TestCase):
             metadata.app_id, 'com.canonical.certification.plainbox')
 
 
-class SessionDeviceContextTests(TestCase, SignalInterceptorMixIn):
+class SessionDeviceContextTests(SignalTestCase):
 
     def setUp(self):
         self.ctx = SessionDeviceContext()
         self.provider = mock.Mock(name='provider', spec_set=Provider1)
         self.unit = mock.Mock(name='unit', spec_set=Unit)
         self.unit.provider = self.provider
-        self.provider.get_units.return_value = ([self.unit], ())
+        self.provider.unit_list = [self.unit]
+        self.provider.problem_list = []
         self.job = mock.Mock(name='job', spec_set=JobDefinition)
         self.job.Meta.name = 'job'
 
@@ -948,10 +1017,11 @@ class SessionDeviceContextTests(TestCase, SignalInterceptorMixIn):
         Ensure that the cached list of execution controllers is invalidated
         when a new provider is added to the context
         """
-        # Let's have a fake provider ready. We need to mock the get_units()
-        # method to let us add it to the context.
+        # Let's have a fake provider ready. We need to mock unit/problem lists
+        # to let us add it to the context.
         provider2 = mock.Mock(name='provider2', spec_set=Provider1)
-        provider2.get_units.return_value = ([], [])
+        provider2.unit_list = []
+        provider2.problem_list = []
         with mock.patch.object(self.ctx, '_compute_execution_ctrl_list') as m:
             m.side_effect = lambda: mock.Mock()
             self.assertNotIn(
