@@ -39,6 +39,7 @@ from plainbox.impl.session.suspend import SessionSuspendHelper1
 from plainbox.impl.session.suspend import SessionSuspendHelper2
 from plainbox.impl.session.suspend import SessionSuspendHelper3
 from plainbox.impl.session.suspend import SessionSuspendHelper4
+from plainbox.impl.session.suspend import SessionSuspendHelper5
 from plainbox.impl.testing_utils import make_job
 from plainbox.vendor import mock
 
@@ -60,7 +61,7 @@ class BaseJobResultTestsTestsMixIn:
     """
 
     def setUp(self):
-        self.helper = SessionSuspendHelper1()
+        self.helper = self.HELPER_CLS()
         self.empty_result = self.TESTED_CLS({})
         self.typical_result = self.TESTED_CLS({
             "outcome": self.TESTED_CLS.OUTCOME_PASS,
@@ -69,39 +70,40 @@ class BaseJobResultTestsTestsMixIn:
             "return_code": 1,
             # NOTE: those are actually specific to TESTED_CLS but it is
             # a simple hack that gets the job done
-            "io_log_filename": "/nonexistent.log",
+            "io_log_filename": "/path/to/log.txt",
             "io_log": [
                 (0, 'stdout', b'first part\n'),
                 (0.1, 'stdout', b'second part\n'),
             ]
         })
+        self.session_dir = None
 
     def test_repr_xxxJobResult_outcome(self):
         """
         verify that DiskJobResult.outcome is serialized correctly
         """
-        data = self.repr_method(self.typical_result)
+        data = self.repr_method(self.typical_result, self.session_dir)
         self.assertEqual(data['outcome'], DiskJobResult.OUTCOME_PASS)
 
     def test_repr_xxxJobResult_execution_duration(self):
         """
         verify that DiskJobResult.execution_duration is serialized correctly
         """
-        data = self.repr_method(self.typical_result)
+        data = self.repr_method(self.typical_result, self.session_dir)
         self.assertAlmostEqual(data['execution_duration'], 42.5)
 
     def test_repr_xxxJobResult_comments(self):
         """
         verify that DiskJobResult.comments is serialized correctly
         """
-        data = self.repr_method(self.typical_result)
+        data = self.repr_method(self.typical_result, self.session_dir)
         self.assertEqual(data['comments'], "the screen was corrupted")
 
     def test_repr_xxxJobResult_return_code(self):
         """
         verify that DiskJobResult.return_code is serialized correctly
         """
-        data = self.repr_method(self.typical_result)
+        data = self.repr_method(self.typical_result, self.session_dir)
         self.assertEqual(data['return_code'], 1)
 
 
@@ -111,6 +113,7 @@ class SuspendMemoryJobResultTests(BaseJobResultTestsTestsMixIn, TestCase):
     """
 
     TESTED_CLS = MemoryJobResult
+    HELPER_CLS = SessionSuspendHelper1
 
     def setUp(self):
         super(SuspendMemoryJobResultTests, self).setUp()
@@ -120,7 +123,7 @@ class SuspendMemoryJobResultTests(BaseJobResultTestsTestsMixIn, TestCase):
         """
         verify that the representation of an empty MemoryJobResult is okay
         """
-        data = self.repr_method(self.empty_result)
+        data = self.repr_method(self.empty_result, self.session_dir)
         self.assertEqual(data, {
             "outcome": None,
             "execution_duration": None,
@@ -133,7 +136,8 @@ class SuspendMemoryJobResultTests(BaseJobResultTestsTestsMixIn, TestCase):
         """
         verify that MemoryJobResult.io_log is serialized correctly
         """
-        data = self.helper._repr_MemoryJobResult(self.typical_result)
+        data = self.helper._repr_MemoryJobResult(
+            self.typical_result, self.session_dir)
         self.assertEqual(data['io_log'], [
             [0, 'stdout', 'Zmlyc3QgcGFydAo='],
             [0.1, 'stdout', 'c2Vjb25kIHBhcnQK'],
@@ -146,6 +150,7 @@ class SuspendDiskJobResultTests(BaseJobResultTestsTestsMixIn, TestCase):
     """
 
     TESTED_CLS = DiskJobResult
+    HELPER_CLS = SessionSuspendHelper1
 
     def setUp(self):
         super(SuspendDiskJobResultTests, self).setUp()
@@ -155,7 +160,7 @@ class SuspendDiskJobResultTests(BaseJobResultTestsTestsMixIn, TestCase):
         """
         verify that the representation of an empty DiskJobResult is okay
         """
-        data = self.repr_method(self.empty_result)
+        data = self.repr_method(self.empty_result, self.session_dir)
         self.assertEqual(data, {
             "outcome": None,
             "execution_duration": None,
@@ -168,8 +173,30 @@ class SuspendDiskJobResultTests(BaseJobResultTestsTestsMixIn, TestCase):
         """
         verify that DiskJobResult.io_log_filename is serialized correctly
         """
-        data = self.helper._repr_DiskJobResult(self.typical_result)
-        self.assertEqual(data['io_log_filename'], "/nonexistent.log")
+        data = self.helper._repr_DiskJobResult(
+            self.typical_result, self.session_dir)
+        self.assertEqual(data['io_log_filename'], "/path/to/log.txt")
+
+
+class Suspend5DiskJobResultTests(SuspendDiskJobResultTests):
+    """
+    Tests that check how DiskJobResult is represented by SessionSuspendHelper5
+    """
+
+    TESTED_CLS = DiskJobResult
+    HELPER_CLS = SessionSuspendHelper5
+
+    def test_repr_DiskJobResult_io_log_filename__no_session_dir(self):
+        """ io_log_filename is absolute in session_dir is not used.  """
+        data = self.helper._repr_DiskJobResult(
+            self.typical_result, None)
+        self.assertEqual(data['io_log_filename'], "/path/to/log.txt")
+
+    def test_repr_DiskJobResult_io_log_filename__session_dir(self):
+        """ io_log_filename is relative if session_dir is used. """
+        data = self.helper._repr_DiskJobResult(
+            self.typical_result, "/path/to")
+        self.assertEqual(data['io_log_filename'], "log.txt")
 
 
 class SessionSuspendHelper1Tests(TestCase):
@@ -179,6 +206,7 @@ class SessionSuspendHelper1Tests(TestCase):
 
     def setUp(self):
         self.helper = SessionSuspendHelper1()
+        self.session_dir = None
 
     def test_repr_IOLogRecord(self):
         """
@@ -195,7 +223,7 @@ class SessionSuspendHelper1Tests(TestCase):
         calls _repr_MemoryJobResult
         """
         result = MemoryJobResult({})
-        self.helper._repr_JobResult(result)
+        self.helper._repr_JobResult(result, self.session_dir)
         mocked_helper._repr_MemoryJobResult.assertCalledOnceWith(result)
 
     @mock.patch('plainbox.impl.session.suspend.SessionSuspendHelper')
@@ -205,7 +233,7 @@ class SessionSuspendHelper1Tests(TestCase):
         calls _repr_DiskJobResult
         """
         result = DiskJobResult({})
-        self.helper._repr_JobResult(result)
+        self.helper._repr_JobResult(result, self.session_dir)
         mocked_helper._repr_DiskJobResult.assertCalledOnceWith(result)
 
     def test_repr_JobResult_with_junk(self):
@@ -221,7 +249,8 @@ class SessionSuspendHelper1Tests(TestCase):
         verify that representation of empty SessionMetaData is okay
         """
         # all defaults with empty values
-        data = self.helper._repr_SessionMetaData(SessionMetaData())
+        data = self.helper._repr_SessionMetaData(
+            SessionMetaData(), self.session_dir)
         self.assertEqual(data, {
             'title': None,
             'flags': [],
@@ -237,7 +266,7 @@ class SessionSuspendHelper1Tests(TestCase):
             title='USB Testing session',
             flags=['incomplete'],
             running_job_name='usb/detect'
-        ))
+        ), self.session_dir)
         self.assertEqual(data, {
             'title': 'USB Testing session',
             'flags': ['incomplete'],
@@ -248,7 +277,8 @@ class SessionSuspendHelper1Tests(TestCase):
         """
         verify that representation of empty SessionState is okay
         """
-        data = self.helper._repr_SessionState(SessionState([]))
+        data = self.helper._repr_SessionState(
+            SessionState([]), self.session_dir)
         self.assertEqual(data, {
             'jobs': {},
             'results': {},
@@ -264,28 +294,28 @@ class SessionSuspendHelper1Tests(TestCase):
         """
         verify that the json representation has the 'version' field
         """
-        data = self.helper._json_repr(SessionState([]))
+        data = self.helper._json_repr(SessionState([]), self.session_dir)
         self.assertIn("version", data)
 
     def test_json_repr_current_version(self):
         """
         verify what the version field is
         """
-        data = self.helper._json_repr(SessionState([]))
+        data = self.helper._json_repr(SessionState([]), self.session_dir)
         self.assertEqual(data['version'], 1)
 
     def test_json_repr_stores_session_state(self):
         """
         verify that the json representation has the 'session' field
         """
-        data = self.helper._json_repr(SessionState([]))
+        data = self.helper._json_repr(SessionState([]), self.session_dir)
         self.assertIn("session", data)
 
     def test_suspend(self):
         """
         verify that the suspend() method returns gzipped JSON representation
         """
-        data = self.helper.suspend(SessionState([]))
+        data = self.helper.suspend(SessionState([]), self.session_dir)
         # XXX: we cannot really test what the compressed data looks like
         # because apparently python3.2 gzip output is non-deterministic.
         # It seems to be an instance of the gzip bug that was fixed a few
@@ -321,6 +351,7 @@ class GeneratedJobSuspendTests(TestCase):
     """
 
     def setUp(self):
+        self.session_dir = None
         # Crete a "__category__" job
         self.category_job = JobDefinition({
             "plugin": "local",
@@ -364,8 +395,9 @@ class GeneratedJobSuspendTests(TestCase):
             self.category_job, self.category_result)
         # Ensure that the generator job gained the "via" attribute
         # This is how we know the code above has no typos or anything.
-        self.assertEqual(
-            self.generator_job.via, self.category_job.checksum)
+        self.assertIs(
+            self.session_state.job_state_map[self.generator_job.id].via_job,
+            self.category_job)
         # "execute" the "generator" job by showing the session the result.
         # Connect the 'on_job_added' signal to a helper function that
         # extracts the "generated" job
@@ -386,7 +418,8 @@ class GeneratedJobSuspendTests(TestCase):
         # and use the data we've defined so far to create JSON-friendly
         # description of the session state.
         self.helper = SessionSuspendHelper1()
-        self.data = self.helper._repr_SessionState(self.session_state)
+        self.data = self.helper._repr_SessionState(
+            self.session_state, self.session_dir)
 
     def test_state_tracked_for_all_jobs(self):
         """
@@ -523,12 +556,13 @@ class SessionSuspendHelper2Tests(SessionSuspendHelper1Tests):
 
     def setUp(self):
         self.helper = SessionSuspendHelper2()
+        self.session_dir = None
 
     def test_json_repr_current_version(self):
         """
         verify what the version field is
         """
-        data = self.helper._json_repr(SessionState([]))
+        data = self.helper._json_repr(SessionState([]), self.session_dir)
         self.assertEqual(data['version'], 2)
 
     def test_repr_SessionMetaData_empty_metadata(self):
@@ -536,7 +570,8 @@ class SessionSuspendHelper2Tests(SessionSuspendHelper1Tests):
         verify that representation of empty SessionMetaData is okay
         """
         # all defaults with empty values
-        data = self.helper._repr_SessionMetaData(SessionMetaData())
+        data = self.helper._repr_SessionMetaData(
+            SessionMetaData(), self.session_dir)
         self.assertEqual(data, {
             'title': None,
             'flags': [],
@@ -554,7 +589,7 @@ class SessionSuspendHelper2Tests(SessionSuspendHelper1Tests):
             flags=['incomplete'],
             running_job_name='usb/detect',
             app_blob=b'blob',
-        ))
+        ), self.session_dir)
         self.assertEqual(data, {
             'title': 'USB Testing session',
             'flags': ['incomplete'],
@@ -566,7 +601,8 @@ class SessionSuspendHelper2Tests(SessionSuspendHelper1Tests):
         """
         verify that representation of empty SessionState is okay
         """
-        data = self.helper._repr_SessionState(SessionState([]))
+        data = self.helper._repr_SessionState(
+            SessionState([]), self.session_dir)
         self.assertEqual(data, {
             'jobs': {},
             'results': {},
@@ -583,7 +619,8 @@ class SessionSuspendHelper2Tests(SessionSuspendHelper1Tests):
         """
         verify that the suspend() method returns gzipped JSON representation
         """
-        data = self.helper.suspend(SessionState([]))
+        data = self.helper.suspend(
+            SessionState([]), self.session_dir)
         # XXX: we cannot really test what the compressed data looks like
         # because apparently python3.2 gzip output is non-deterministic.
         # It seems to be an instance of the gzip bug that was fixed a few
@@ -608,12 +645,13 @@ class SessionSuspendHelper3Tests(SessionSuspendHelper2Tests):
 
     def setUp(self):
         self.helper = SessionSuspendHelper3()
+        self.session_dir = None
 
     def test_json_repr_current_version(self):
         """
         verify what the version field is
         """
-        data = self.helper._json_repr(SessionState([]))
+        data = self.helper._json_repr(SessionState([]), self.session_dir)
         self.assertEqual(data['version'], 3)
 
     def test_repr_SessionMetaData_empty_metadata(self):
@@ -621,7 +659,8 @@ class SessionSuspendHelper3Tests(SessionSuspendHelper2Tests):
         verify that representation of empty SessionMetaData is okay
         """
         # all defaults with empty values
-        data = self.helper._repr_SessionMetaData(SessionMetaData())
+        data = self.helper._repr_SessionMetaData(
+            SessionMetaData(), self.session_dir)
         self.assertEqual(data, {
             'title': None,
             'flags': [],
@@ -641,7 +680,7 @@ class SessionSuspendHelper3Tests(SessionSuspendHelper2Tests):
             running_job_name='usb/detect',
             app_blob=b'blob',
             app_id='com.canonical.certification.plainbox',
-        ))
+        ), self.session_dir)
         self.assertEqual(data, {
             'title': 'USB Testing session',
             'flags': ['incomplete'],
@@ -654,7 +693,8 @@ class SessionSuspendHelper3Tests(SessionSuspendHelper2Tests):
         """
         verify that representation of empty SessionState is okay
         """
-        data = self.helper._repr_SessionState(SessionState([]))
+        data = self.helper._repr_SessionState(
+            SessionState([]), self.session_dir)
         self.assertEqual(data, {
             'jobs': {},
             'results': {},
@@ -672,7 +712,7 @@ class SessionSuspendHelper3Tests(SessionSuspendHelper2Tests):
         """
         verify that the suspend() method returns gzipped JSON representation
         """
-        data = self.helper.suspend(SessionState([]))
+        data = self.helper.suspend(SessionState([]), self.session_dir)
         # XXX: we cannot really test what the compressed data looks like
         # because apparently python3.2 gzip output is non-deterministic.
         # It seems to be an instance of the gzip bug that was fixed a few
@@ -698,12 +738,13 @@ class SessionSuspendHelper4Tests(SessionSuspendHelper3Tests):
 
     def setUp(self):
         self.helper = SessionSuspendHelper4()
+        self.session_dir = None
 
     def test_json_repr_current_version(self):
         """
         verify what the version field is
         """
-        data = self.helper._json_repr(SessionState([]))
+        data = self.helper._json_repr(SessionState([]), self.session_dir)
         self.assertEqual(data['version'], 4)
 
     def test_repr_SessionState_typical_session(self):
@@ -733,7 +774,7 @@ class SessionSuspendHelper4Tests(SessionSuspendHelper3Tests):
         session_state = SessionState([used_job, unused_job])
         session_state.update_desired_job_list([used_job])
         session_state.update_job_result(used_job, used_result)
-        data = self.helper._repr_SessionState(session_state)
+        data = self.helper._repr_SessionState(session_state, self.session_dir)
         self.assertEqual(data, {
             'jobs': {
                 'used': ('8c393c19fdfde1b6afc5b79d0a1617ecf7531cd832a16450dc'
@@ -762,7 +803,7 @@ class SessionSuspendHelper4Tests(SessionSuspendHelper3Tests):
         """
         verify that the suspend() method returns gzipped JSON representation
         """
-        data = self.helper.suspend(SessionState([]))
+        data = self.helper.suspend(SessionState([]), self.session_dir)
         # XXX: we cannot really test what the compressed data looks like
         # because apparently python3.2 gzip output is non-deterministic.
         # It seems to be an instance of the gzip bug that was fixed a few
@@ -781,6 +822,45 @@ class SessionSuspendHelper4Tests(SessionSuspendHelper3Tests):
             b'"version":4}'))
 
 
+class SessionSuspendHelper5Tests(SessionSuspendHelper4Tests):
+    """
+    Tests for various methods of SessionSuspendHelper5
+    """
+
+    def setUp(self):
+        self.helper = SessionSuspendHelper5()
+        self.session_dir = None
+
+    def test_json_repr_current_version(self):
+        """
+        verify what the version field is
+        """
+        data = self.helper._json_repr(SessionState([]), self.session_dir)
+        self.assertEqual(data['version'], 5)
+
+    def test_suspend(self):
+        """
+        verify that the suspend() method returns gzipped JSON representation
+        """
+        data = self.helper.suspend(SessionState([]), self.session_dir)
+        # XXX: we cannot really test what the compressed data looks like
+        # because apparently python3.2 gzip output is non-deterministic.
+        # It seems to be an instance of the gzip bug that was fixed a few
+        # years ago.
+        #
+        # I've filed a bug on python3.2 in Ubuntu and Python upstream project
+        # https://bugs.launchpad.net/ubuntu/+source/python3.2/+bug/871083
+        #
+        # In the meantime we can only test that we got bytes out
+        self.assertIsInstance(data, bytes)
+        # And that we can gzip uncompress them and get what we expected
+        self.assertEqual(gzip.decompress(data), (
+            b'{"session":{"desired_job_list":[],"jobs":{},"metadata":'
+            b'{"app_blob":null,"app_id":null,"flags":[],'
+            b'"running_job_name":null,"title":null},"results":{}},'
+            b'"version":5}'))
+
+
 class RegressionTests(TestCase):
 
     def test_1388055(self):
@@ -799,10 +879,11 @@ class RegressionTests(TestCase):
         self.assertEqual(state.run_list, [job_a_dep, job_a])
         self.assertEqual(state.desired_job_list, [job_a])
         helper = SessionSuspendHelper4()
+        session_dir = None
         # Mock away the meta-data as we're not testing that
         with mock.patch.object(helper, '_repr_SessionMetaData') as m:
             m.return_value = 'mocked'
-            actual = helper._repr_SessionState(state)
+            actual = helper._repr_SessionState(state, session_dir)
         expected = {
             'jobs': {
                 job_a_dep.id: job_a_dep.checksum,
