@@ -34,22 +34,26 @@ from plainbox.abc import IProviderBackend1
 from plainbox.impl.applogic import PlainBoxConfig
 from plainbox.impl.ctrl import CheckBoxExecutionController
 from plainbox.impl.ctrl import CheckBoxSessionStateController
+from plainbox.impl.ctrl import QmlJobExecutionController
 from plainbox.impl.ctrl import RootViaPTL1ExecutionController
 from plainbox.impl.ctrl import RootViaPkexecExecutionController
 from plainbox.impl.ctrl import RootViaSudoExecutionController
 from plainbox.impl.ctrl import SymLinkNest
 from plainbox.impl.ctrl import UserJobExecutionController
 from plainbox.impl.ctrl import gen_rfc822_records_from_io_log
-from plainbox.impl.depmgr import DependencyDuplicateError
+from plainbox.impl.ctrl import get_via_cycle
 from plainbox.impl.job import JobDefinition
 from plainbox.impl.resource import Resource
 from plainbox.impl.resource import ResourceExpression
+from plainbox.impl.result import MemoryJobResult
 from plainbox.impl.secure.origin import JobOutputTextSource
 from plainbox.impl.secure.origin import Origin
 from plainbox.impl.secure.providers.v1 import Provider1
 from plainbox.impl.secure.rfc822 import RFC822Record
 from plainbox.impl.secure.rfc822 import RFC822SyntaxError
+from plainbox.impl.session import InhibitionCause
 from plainbox.impl.session import JobReadinessInhibitor
+from plainbox.impl.session import JobState
 from plainbox.impl.session import SessionState
 from plainbox.vendor import extcmd
 from plainbox.vendor import mock
@@ -113,7 +117,7 @@ class CheckBoxSessionStateControllerTests(TestCase):
         self.assertEqual(
             self.ctrl.get_inhibitor_list(session_state, j1),
             [JobReadinessInhibitor(
-                JobReadinessInhibitor.PENDING_RESOURCE,
+                InhibitionCause.PENDING_RESOURCE,
                 j2, ResourceExpression('j2.attr == "ok"'))])
 
     def test_get_inhibitor_list_FAILED_RESOURCE(self):
@@ -135,7 +139,7 @@ class CheckBoxSessionStateControllerTests(TestCase):
         self.assertEqual(
             self.ctrl.get_inhibitor_list(session_state, j1),
             [JobReadinessInhibitor(
-                JobReadinessInhibitor.FAILED_RESOURCE,
+                InhibitionCause.FAILED_RESOURCE,
                 j2, ResourceExpression('j2.attr == "ok"'))])
 
     def test_get_inhibitor_list_good_resource(self):
@@ -174,7 +178,7 @@ class CheckBoxSessionStateControllerTests(TestCase):
         self.assertEqual(
             self.ctrl.get_inhibitor_list(session_state, j1),
             [JobReadinessInhibitor(
-                JobReadinessInhibitor.PENDING_DEP, j2, None)])
+                InhibitionCause.PENDING_DEP, j2, None)])
 
     def test_get_inhibitor_list_FAILED_DEP(self):
         # verify that jobs that depend on another job that ran but
@@ -194,7 +198,7 @@ class CheckBoxSessionStateControllerTests(TestCase):
         self.assertEqual(
             self.ctrl.get_inhibitor_list(session_state, j1),
             [JobReadinessInhibitor(
-                JobReadinessInhibitor.FAILED_DEP, j2, None)])
+                InhibitionCause.FAILED_DEP, j2, None)])
 
     def test_get_inhibitor_list_good_dep(self):
         # verify that jobs that depend on another job that ran and has outcome
@@ -288,33 +292,25 @@ class CheckBoxSessionStateControllerTests(TestCase):
         verify side effects of using observe_result() that would define a new
         job
         """
-        # Create a session that knows about no jobs yet
-        # and happily adds jobs when add_job() gets called
-        session_state = mock.MagicMock(spec=SessionState)
-        session_state.add_job.side_effect = (
-            lambda new_job, recompute: new_job)
-        # Create a job of which result we'll be observing
-        job = mock.Mock(spec=JobDefinition, name='job', plugin='local')
-        # Create a result for the job we'll be observing
-        result = mock.Mock(
-            spec=IJobResult, name='result', outcome=IJobResult.OUTCOME_PASS)
-        # Mock what rfc822 parser returns
-        mock_gen.return_value = [mock.Mock(spec=RFC822Record, name='record')]
-        # Pretend that we are observing a 'result' of 'job'
-        self.ctrl.observe_result(session_state, job, result)
+        # Job A is any example job
+        job_a = JobDefinition({'id': 'a', 'plugin': 'shell', 'command': ':'})
+        # Job B is a job that prints the definition of job A
+        job_b = JobDefinition({'id': 'b', 'plugin': 'local'})
+        # Result B is a fake result of running job B
+        result_b = MemoryJobResult({'outcome': 'pass'})
+        # Session knows about just B
+        session_state = SessionState([job_b])
+        # Mock gen_rfc822_records_from_io_log to produce one mock record
+        mock_gen.return_value = [RFC822Record({})]
+        # Mock job B to create job A as a child if asked to
+        with mock.patch.object(job_b, 'create_child_job_from_record') as fn:
+            fn.side_effect = lambda record: job_a
+            # Pretend that we are observing a 'result_b' of 'job_b'
+            self.ctrl.observe_result(session_state, job_b, result_b)
         # Ensure that result got stored
-        self.assertIs(
-            session_state.job_state_map[job.id].result, result)
-        # Ensure that new job was defined
-        session_state.add_job.assert_called_once_with(
-            job.create_child_job_from_record(), recompute=False)
-        # Ensure that we didn't try to change the origin of the new job
-        self.assertFalse(
-            job.create_child_job_from_record().update_origin.called)
-        # Ensure that signals got fired
-        session_state.on_job_state_map_changed.assert_called_once_with()
-        session_state.on_job_result_changed.assert_called_once_with(
-            job, result)
+        self.assertIs(session_state.job_state_map[job_b.id].result, result_b)
+        # Ensure that job A is now via-connected to job B
+        self.assertIs(session_state.job_state_map[job_a.id].via_job, job_b)
 
     @mock.patch('plainbox.impl.ctrl.gen_rfc822_records_from_io_log')
     @mock.patch('plainbox.impl.ctrl.logger')
@@ -327,45 +323,34 @@ class CheckBoxSessionStateControllerTests(TestCase):
         We basically hope to see the old job being there intact and a warning
         to be logged.
         """
-        # Create a session that already knows about 'existing_job'
-        # and raises a DependencyDuplicateError when add_job() gets called.
-        existing_job = mock.Mock(spec=JobDefinition, name='existing_job')
-        existing_job.id = 'generated'
-        clashing_job = mock.Mock(spec=JobDefinition, name='existing_job')
-        clashing_job.id = 'generated'
-        session_state = mock.MagicMock(spec=SessionState, name='session_state')
-        session_state.add_job.side_effect = DependencyDuplicateError(
-            existing_job, clashing_job)
-        # Create a job of which result we'll be observing
-        job = mock.Mock(spec=JobDefinition, name='job', plugin='local')
-        # Have job return clashing_job when create_child_job_record() is called
-        job.create_child_job_from_record.return_value = clashing_job
-        # Create a result for the job we'll be observing
-        result = mock.Mock(
-            spec=IJobResult, name='result', outcome=IJobResult.OUTCOME_PASS)
-        # Mock what rfc822 parser returns
-        mock_gen.return_value = [mock.Mock(spec=RFC822Record, name='record')]
-        # Pretend that we are observing a 'result' of 'job'
-        self.ctrl.observe_result(session_state, job, result)
+        # Jobs A1 and A2 are simple example jobs (different, with same id)
+        job_a1 = JobDefinition(
+            {'id': 'a', 'plugin': 'shell', 'command': 'true'})
+        job_a2 = JobDefinition(
+            {'id': 'a', 'plugin': 'shell', 'command': 'false'})
+        # Job B is a job that prints the definition of job A2
+        job_b = JobDefinition({'id': 'b', 'plugin': 'local'})
+        # Result B is a fake result of running job B
+        result_b = MemoryJobResult({'outcome': 'pass'})
+        # Session knows about A1 and B
+        session_state = SessionState([job_a1, job_b])
+        # Mock gen_rfc822_records_from_io_log to produce one mock record
+        mock_gen.return_value = [RFC822Record({})]
+        # Mock job B to create job A2 as a child if asked to
+        with mock.patch.object(job_b, 'create_child_job_from_record') as fn:
+            fn.side_effect = lambda record: job_a2
+            # Pretend that we are observing a 'result_b' of 'job_b'
+            self.ctrl.observe_result(session_state, job_b, result_b)
         # Ensure that result got stored
-        self.assertIs(
-            session_state.job_state_map[job.id].result, result)
-        # Ensure that we tried to define a new job by calling add_job() with
-        # the clashing_job as argument
-        session_state.add_job.assert_called_once_with(
-            clashing_job, recompute=False)
-        # Ensure that existing job origin was *not* updated
-        self.assertFalse(existing_job.update_origin.called)
-        # Ensure that signals got fired
-        session_state.on_job_state_map_changed.assert_called_once_with()
-        session_state.on_job_result_changed.assert_called_once_with(
-            job, result)
+        self.assertIs(session_state.job_state_map[job_b.id].result, result_b)
+        # Ensure that we didn't change via_job of the job A1
+        self.assertIsNot(session_state.job_state_map[job_a1.id].via_job, job_b)
         # Ensure that a warning was logged
         mock_logger.warning.assert_called_once_with(
             ("Local job %s produced job %s that collides with"
              " an existing job %s (from %s), the new job was"
              " discarded"),
-            job.id, clashing_job.id, existing_job.id, existing_job.origin)
+            job_b.id, job_a2.id, job_a1.id, job_a1.origin)
 
     @mock.patch('plainbox.impl.ctrl.gen_rfc822_records_from_io_log')
     def test_observe_result__local_perfect_clash(self, mock_gen):
@@ -377,42 +362,30 @@ class CheckBoxSessionStateControllerTests(TestCase):
         should be updated to reflect the new association between 'existing_job'
         and 'job'
         """
-        # Create a session that already knows about 'existing_job'
-        # and returns existing_job when add_job() gets called.
-        existing_job = mock.Mock(spec=JobDefinition, name='existing_job')
-        existing_job.id = 'generated'
-        session_state = mock.MagicMock(spec=SessionState, name='session_state')
-        session_state.add_job.side_effect = (
-            lambda new_job, recompute: existing_job)
-        # Create a job of which result we'll be observing
-        job = mock.Mock(spec=JobDefinition, name='job', plugin='local')
-        # Create a result for the job we'll be observing
-        result = mock.Mock(
-            spec=IJobResult, name='result', outcome=IJobResult.OUTCOME_PASS)
-        # Mock what rfc822 parser returns
-        mock_gen.return_value = [mock.Mock(spec=RFC822Record, name='record')]
-        # Pretend that we are observing a 'result' of 'job'
-        self.ctrl.observe_result(session_state, job, result)
+        # Job A is any example job
+        job_a = JobDefinition({'id': 'a', 'plugin': 'shell', 'command': ':'})
+        # Job B is a job that prints the definition of job A
+        job_b = JobDefinition({'id': 'b', 'plugin': 'local'})
+        # Result B is a fake result of running job B
+        result_b = MemoryJobResult({'outcome': 'pass'})
+        # Session knows about A and B
+        session_state = SessionState([job_a, job_b])
+        # Mock gen_rfc822_records_from_io_log to produce one mock record
+        mock_gen.return_value = [RFC822Record({})]
+        # Mock job B to create job A as a child if asked to
+        with mock.patch.object(job_b, 'create_child_job_from_record') as fn:
+            fn.side_effect = lambda record: job_a
+            # Pretend that we are observing a 'result_b' of 'job_b'
+            self.ctrl.observe_result(session_state, job_b, result_b)
         # Ensure that result got stored
-        self.assertIs(
-            session_state.job_state_map[job.id].result, result)
-        # Ensure that we tried to define a new job using
-        # whatever create_child_job_from_record() returns.
-        session_state.add_job.assert_called_once_with(
-            job.create_child_job_from_record(), recompute=False)
-        # Ensure that the origin of the existing_job was copied
-        # from the origin of the generated job
-        existing_job.update_origin.assert_called_once_with(
-            job.create_child_job_from_record().origin)
-        # Ensure that signals got fired
-        session_state.on_job_state_map_changed.assert_called_once_with()
-        session_state.on_job_result_changed.assert_called_once_with(
-            job, result)
+        self.assertIs(session_state.job_state_map[job_b.id].result, result_b)
+        # Ensure that job A is now via-connected to job B
+        self.assertIs(session_state.job_state_map[job_a.id].via_job, job_b)
 
 
 class FunctionTests(TestCase):
     """
-    unit tests for gen_rfc822_records_from_io_log()
+    unit tests for gen_rfc822_records_from_io_log() and other functions.
     """
 
     def test_parse_typical(self):
@@ -460,6 +433,43 @@ class FunctionTests(TestCase):
             job.id, RFC822SyntaxError(
                 None, 3, "Unexpected non-empty line: 'error\\n'"))
 
+    def test_get_via_cycle__no_cycle(self):
+        job_a = mock.Mock(spec_set=JobDefinition, name='job_a')
+        job_a.id = 'a'
+        job_state_a = mock.Mock(spec_set=JobState, name='job_state_a')
+        job_state_a.job = job_a
+        job_state_a.via_job = None
+        job_state_map = {job_a.id: job_state_a}
+        self.assertEqual(get_via_cycle(job_state_map, job_a), ())
+
+    def test_get_via_cycle__trivial(self):
+        job_a = mock.Mock(spec_set=JobDefinition, name='job_a')
+        job_a.id = 'a'
+        job_state_a = mock.Mock(spec_set=JobState, name='job_state_b')
+        job_state_a.job = job_a
+        job_state_a.via_job = job_a
+        job_state_map = {job_a.id: job_state_a}
+        self.assertEqual(get_via_cycle(job_state_map, job_a), [job_a, job_a])
+
+    def test_get_via_cycle__indirect(self):
+        job_a = mock.Mock(spec_set=JobDefinition, name='job_a')
+        job_a.id = 'a'
+        job_b = mock.Mock(spec_set=JobDefinition, name='job_b')
+        job_b.id = 'b'
+        job_state_a = mock.Mock(spec_set=JobState, name='job_state_a')
+        job_state_a.job = job_a
+        job_state_a.via_job = job_b
+        job_state_b = mock.Mock(spec_set=JobState, name='job_state_b')
+        job_state_b.job = job_b
+        job_state_b.via_job = job_a
+        job_state_map = {
+            job_a.id: job_state_a,
+            job_b.id: job_state_b,
+        }
+        self.assertEqual(
+            get_via_cycle(job_state_map, job_a),
+            [job_a, job_b, job_a])
+
 
 class SymLinkNestTests(TestCase):
     """
@@ -482,13 +492,12 @@ class SymLinkNestTests(TestCase):
         verify that add_provider() adds each executable
         """
         provider = mock.Mock(name='provider', spec=Provider1)
-        provider.get_all_executables.return_value = ['exec1', 'exec2']
+        provider.executable_list = ['exec1', 'exec2']
         with mock.patch.object(self.nest, 'add_executable'):
             self.nest.add_provider(provider)
             self.nest.add_executable.assert_has_calls([
                 (('exec1',), {}),
                 (('exec2',), {})])
-        provider.get_all_executables.assert_called_once()
 
     @mock.patch('os.symlink')
     def test_add_executable(self, mock_symlink):
@@ -520,19 +529,11 @@ class CheckBoxExecutionControllerTestsMixIn:
             spec=JobDefinition,
             provider=mock.Mock(
                 name='provider',
-                # XXX: create a new type that has both IProvider1 and
-                # IProviderBackend1 API This type will define the API of the
-                # mocked provider object. We need to access IProvider1.name
-                # (for RootViaPTL1ExecutionController) but we also need to
-                # access many of IProviderBackend1 APIs for all the other code
-                # here. Instead of fusing the interfaces we can just pass a
-                # "concrete provider" "interface" just for this set of tests.
-                spec=type(
-                    'IProvider1+IProviderBackend1',
-                    (IProviderBackend1, IProvider1), {}),
+                spec=IProvider1,
                 extra_PYTHONPATH=None,
                 CHECKBOX_SHARE='CHECKBOX_SHARE',
                 data_dir='data_dir', units_dir='units_dir'))
+        self.job_state = mock.Mock(name='job_state', spec=JobState)
         # Mock the default flags (empty set)
         self.job.get_flag_set.return_value = frozenset()
         # Create mocked config.
@@ -570,7 +571,8 @@ class CheckBoxExecutionControllerTestsMixIn:
                 mock.patch.object(self.ctrl, 'configured_filesystem'), \
                 mock.patch.object(self.ctrl, 'temporary_cwd'):
             retval = self.ctrl.execute_job(
-                self.job, self.config, self.SESSION_DIR, self.extcmd_popen)
+                self.job, self.job_state, self.config, self.SESSION_DIR,
+                self.extcmd_popen)
             # Ensure that call was invoked with command end environment (passed
             # as keyword argument). Extract the return value of
             # configured_filesystem() as nest_dir so that we can pass it to
@@ -580,9 +582,11 @@ class CheckBoxExecutionControllerTestsMixIn:
             cwd_dir = self.ctrl.temporary_cwd().__enter__()
             self.extcmd_popen.call.assert_called_with(
                 self.ctrl.get_execution_command(
-                    self.job, self.config, self.SESSION_DIR, nest_dir),
+                    self.job, self.job_state, self.config, self.SESSION_DIR,
+                    nest_dir),
                 env=self.ctrl.get_execution_environment(
-                    self.job, self.config, self.SESSION_DIR, nest_dir),
+                    self.job, self.job_state, self.config, self.SESSION_DIR,
+                    nest_dir),
                 cwd=cwd_dir)
         # Ensure that execute_job() returns the return value of call()
         self.assertEqual(retval, self.extcmd_popen.call())
@@ -611,6 +615,26 @@ class CheckBoxExecutionControllerTestsMixIn:
             self.ctrl.get_CHECKBOX_DATA(self.SESSION_DIR),
             "session-dir/CHECKBOX_DATA")
 
+    @mock.patch('json.dumps')
+    @mock.patch('json.loads')
+    @mock.patch('os.fdopen')
+    def test_noreturn_flag_hangs(self, mock_os_fdopen, mock_json_loads,
+                                 mock_json_dumps):
+        """
+        verify that jobs having 'noreturn' flag call _halt after executing
+        command
+        """
+        self.job.get_flag_set.return_value = {'noreturn'}
+        with mock.patch.object(self.ctrl, 'get_execution_command'), \
+                mock.patch.object(self.ctrl, 'get_execution_environment'), \
+                mock.patch.object(self.ctrl, 'configured_filesystem'), \
+                mock.patch.object(self.ctrl, 'temporary_cwd'), \
+                mock.patch.object(self.ctrl, '_halt'):
+            self.ctrl.execute_job(
+                self.job, self.job_state, self.config, self.SESSION_DIR,
+                self.extcmd_popen)
+            self.ctrl._halt.assert_called_once_with()
+
 
 class UserJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
                                       TestCase):
@@ -626,7 +650,8 @@ class UserJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
         """
         self.assertEqual(
             self.ctrl.get_execution_command(
-                self.job, self.config, self.SESSION_DIR, self.NEST_DIR),
+                self.job, self.job_state, self.config, self.SESSION_DIR,
+                self.NEST_DIR),
             [self.job.shell, '-c', self.job.command])
 
     def test_get_checkbox_score_for_jobs_without_user(self):
@@ -664,7 +689,8 @@ class UserJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
     def test_get_execution_environment_resets_LANG(self):
         # Call the tested method
         env = self.ctrl.get_execution_environment(
-            self.job, self.config, self.SESSION_DIR, self.NEST_DIR)
+            self.job, self.job_state, self.config, self.SESSION_DIR,
+            self.NEST_DIR)
         # Ensure that LANG is reset to C.UTF-8
         self.assertEqual(env['LANG'], 'C.UTF-8')
 
@@ -673,7 +699,8 @@ class UserJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
         self.job.get_flag_set.return_value = {'preserve-locale'}
         # Call the tested method
         env = self.ctrl.get_execution_environment(
-            self.job, self.config, self.SESSION_DIR, self.NEST_DIR)
+            self.job, self.job_state, self.config, self.SESSION_DIR,
+            self.NEST_DIR)
         # Ensure that LANG is what we mocked it to be
         self.assertEqual(env['LANG'], 'fake_LANG')
 
@@ -681,7 +708,8 @@ class UserJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
     def test_get_execution_environment_keeps_PYTHONPATH(self):
         # Call the tested method
         env = self.ctrl.get_execution_environment(
-            self.job, self.config, self.SESSION_DIR, self.NEST_DIR)
+            self.job, self.job_state, self.config, self.SESSION_DIR,
+            self.NEST_DIR)
         # Ensure that extra_PYTHONPATH is preprended to PYTHONPATH
         self.assertEqual(env['PYTHONPATH'], 'PYTHONPATH')
 
@@ -691,7 +719,8 @@ class UserJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
         self.job.provider.extra_PYTHONPATH = 'extra_PYTHONPATH'
         # Call the tested method
         env = self.ctrl.get_execution_environment(
-            self.job, self.config, self.SESSION_DIR, self.NEST_DIR)
+            self.job, self.job_state, self.config, self.SESSION_DIR,
+            self.NEST_DIR)
         # Ensure that extra_PYTHONPATH is preprended to PYTHONPATH
         self.assertTrue(env['PYTHONPATH'].startswith(
             self.job.provider.extra_PYTHONPATH))
@@ -702,7 +731,8 @@ class UserJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
         self.job.provider.extra_PYTHONPATH = 'extra_PYTHONPATH'
         # Call the tested method
         env = self.ctrl.get_execution_environment(
-            self.job, self.config, self.SESSION_DIR, self.NEST_DIR)
+            self.job, self.job_state, self.config, self.SESSION_DIR,
+            self.NEST_DIR)
         # Ensure that extra_PYTHONPATH is preprended to PYTHONPATH
         self.assertTrue(env['PYTHONPATH'].startswith(
             self.job.provider.extra_PYTHONPATH))
@@ -712,7 +742,8 @@ class UserJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
     def test_get_execution_environment_sets_CHECKBOX_SHARE(self):
         # Call the tested method
         env = self.ctrl.get_execution_environment(
-            self.job, self.config, self.SESSION_DIR, self.NEST_DIR)
+            self.job, self.job_state, self.config, self.SESSION_DIR,
+            self.NEST_DIR)
         # Ensure that CHECKBOX_SHARE is set to what the job provider wants
         self.assertEqual(
             env['CHECKBOX_SHARE'], self.job.provider.CHECKBOX_SHARE)
@@ -721,7 +752,8 @@ class UserJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
     def test_get_execution_environment_sets_CHECKBOX_DATA(self):
         # Call the tested method
         env = self.ctrl.get_execution_environment(
-            self.job, self.config, self.SESSION_DIR, self.NEST_DIR)
+            self.job, self.job_state, self.config, self.SESSION_DIR,
+            self.NEST_DIR)
         # Ensure that CHECKBOX_DATA is set to what the controller wants
         self.assertEqual(
             env['CHECKBOX_DATA'],
@@ -732,7 +764,8 @@ class UserJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
         self.config.environment['key'] = 'value'
         # Call the tested method
         env = self.ctrl.get_execution_environment(
-            self.job, self.config, self.SESSION_DIR, self.NEST_DIR)
+            self.job, self.job_state, self.config, self.SESSION_DIR,
+            self.NEST_DIR)
         # Ensure that key=value was passed to the environment
         self.assertEqual(env['key'], 'value')
 
@@ -741,7 +774,8 @@ class UserJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
         self.config.environment['key'] = 'value'
         # Call the tested method
         env = self.ctrl.get_execution_environment(
-            self.job, self.config, self.SESSION_DIR, self.NEST_DIR)
+            self.job, self.job_state, self.config, self.SESSION_DIR,
+            self.NEST_DIR)
         # Ensure that 'old-value' takes priority over 'value'
         self.assertEqual(env['key'], 'old-value')
 
@@ -757,7 +791,8 @@ class RootViaPTL1ExecutionControllerTests(
     def test_get_execution_environment_is_None(self):
         # Call the tested method
         env = self.ctrl.get_execution_environment(
-            self.job, self.config, self.SESSION_DIR, self.NEST_DIR)
+            self.job, self.job_state, self.config, self.SESSION_DIR,
+            self.NEST_DIR)
         # Ensure that the environment is None
         self.assertEqual(env, None)
 
@@ -767,7 +802,7 @@ class RootViaPTL1ExecutionControllerTests(
         verify that we run plainbox-trusted-launcher-1 as the desired user
         """
         self.job.get_environ_settings.return_value = []
-        self.job.origin.source.job = mock.Mock(
+        self.job_state.via_job = mock.Mock(
             name='generator_job',
             spec=JobDefinition,
             provider=mock.Mock(
@@ -778,12 +813,12 @@ class RootViaPTL1ExecutionControllerTests(
                 units_dir="units_dir-generator",
                 CHECKBOX_SHARE='CHECKBOX_SHARE-generator'))
         # Mock the default flags (empty set)
-        self.job.origin.source.job.get_flag_set.return_value = frozenset()
+        self.job_state.via_job.get_flag_set.return_value = frozenset()
         PATH = os.pathsep.join([self.NEST_DIR, 'vanilla-path'])
         expected = [
             'pkexec', '--user', self.job.user,
             'plainbox-trusted-launcher-1',
-            '--generator', self.job.via,
+            '--generator', self.job_state.via_job.checksum,
             '-G', 'CHECKBOX_DATA=session-dir/CHECKBOX_DATA',
             '-G', 'CHECKBOX_SHARE=CHECKBOX_SHARE-generator',
             '-G', 'LANG=C.UTF-8',
@@ -801,7 +836,8 @@ class RootViaPTL1ExecutionControllerTests(
             '-T', 'PLAINBOX_SESSION_SHARE=session-dir/CHECKBOX_DATA',
         ]
         actual = self.ctrl.get_execution_command(
-            self.job, self.config, self.SESSION_DIR, self.NEST_DIR)
+            self.job, self.job_state, self.config, self.SESSION_DIR,
+            self.NEST_DIR)
         self.assertEqual(actual, expected)
 
     @mock.patch.dict('os.environ', clear=True, PATH='vanilla-path')
@@ -810,7 +846,7 @@ class RootViaPTL1ExecutionControllerTests(
         verify that we run plainbox-trusted-launcher-1 as the desired user
         """
         self.job.get_environ_settings.return_value = []
-        self.job.via = None
+        self.job_state.via_job = None
         PATH = os.pathsep.join([self.NEST_DIR, 'vanilla-path'])
         expected = [
             'pkexec', '--user', self.job.user,
@@ -825,7 +861,8 @@ class RootViaPTL1ExecutionControllerTests(
             '-T', 'PLAINBOX_SESSION_SHARE=session-dir/CHECKBOX_DATA',
         ]
         actual = self.ctrl.get_execution_command(
-            self.job, self.config, self.SESSION_DIR, self.NEST_DIR)
+            self.job, self.job_state, self.config, self.SESSION_DIR,
+            self.NEST_DIR)
         self.assertEqual(actual, expected)
 
     def test_get_checkbox_score_for_other_providers(self):
@@ -932,7 +969,8 @@ class RootViaPkexecExecutionControllerTests(
     def test_get_execution_environment_is_None(self):
         # Call the tested method
         env = self.ctrl.get_execution_environment(
-            self.job, self.config, self.SESSION_DIR, self.NEST_DIR)
+            self.job, self.job_state, self.config, self.SESSION_DIR,
+            self.NEST_DIR)
         # Ensure that the environment is None
         self.assertEqual(env, None)
 
@@ -944,7 +982,8 @@ class RootViaPkexecExecutionControllerTests(
         self.job.get_environ_settings.return_value = []
         self.assertEqual(
             self.ctrl.get_execution_command(
-                self.job, self.config, self.SESSION_DIR, self.NEST_DIR),
+                self.job, self.job_state, self.config, self.SESSION_DIR,
+                self.NEST_DIR),
             ['pkexec', '--user', self.job.user,
              'env',
              'CHECKBOX_DATA=session-dir/CHECKBOX_DATA',
@@ -986,7 +1025,8 @@ class RootViaSudoExecutionControllerTests(
         self.job.get_environ_settings.return_value = []
         self.assertEqual(
             self.ctrl.get_execution_command(
-                self.job, self.config, self.SESSION_DIR, self.NEST_DIR),
+                self.job, self.job_state, self.config, self.SESSION_DIR,
+                self.NEST_DIR),
             ['sudo', '-u', self.job.user, 'env',
              'CHECKBOX_DATA=session-dir/CHECKBOX_DATA',
              'CHECKBOX_SHARE=CHECKBOX_SHARE',
@@ -1067,3 +1107,117 @@ class RootViaSudoExecutionControllerTests(
         self.job.user = None
         # Ensure that we get a negative score for this controller
         self.assertEqual(self.ctrl.get_checkbox_score(self.job), -1)
+
+
+class QmlJobExecutionControllerTests(CheckBoxExecutionControllerTestsMixIn,
+                                     TestCase):
+    """
+    Tests for QmlJobExecutionController
+    """
+    CLS = QmlJobExecutionController
+
+    SHELL_OUT_FD = 6
+    SHELL_IN_FD = 7
+
+    def test_job_repr(self):
+        self.assertEqual(
+            self.ctrl.gen_job_repr(self.job),
+            {'id': self.job.id,
+             'summary': self.job.tr_summary(),
+             'description': self.job.tr_description()})
+
+    def test_get_execution_command(self):
+        """
+        Tests gluing of commandline arguments when running QML exec. ctrl.
+        """
+        self.assertEqual(
+            self.ctrl.get_execution_command(
+                self.job, self.job_state, self.config, self.SESSION_DIR,
+                self.NEST_DIR, self.SHELL_OUT_FD, self.SHELL_IN_FD),
+            ['qmlscene', '--job', self.job.qml_file, '--fd-out',
+             self.SHELL_OUT_FD, '--fd-in', self.SHELL_IN_FD,
+             self.ctrl.QML_SHELL_PATH])
+
+    @mock.patch('json.dumps')
+    @mock.patch('os.path.isdir')
+    @mock.patch('os.fdopen')
+    @mock.patch('os.pipe')
+    @mock.patch('os.write')
+    @mock.patch('os.close')
+    def test_execute_job(self, mock_os_close, mock_os_write, mock_os_pipe,
+                         mock_os_fdopen, mock_os_path_isdir, mock_json_dumps):
+        """
+        Test if qml exec. ctrl. correctly runs piping
+        """
+        mock_os_pipe.side_effect = [("pipe0_r", "pipe0_w"),
+                                    ("pipe1_r", "pipe1_w")]
+        with mock.patch.object(self.ctrl, 'get_execution_command'), \
+                mock.patch.object(self.ctrl, 'get_execution_environment'), \
+                mock.patch.object(self.ctrl, 'configured_filesystem'), \
+                mock.patch.object(self.ctrl, 'temporary_cwd'), \
+                mock.patch.object(self.ctrl, 'gen_job_repr', return_value={}):
+            retval = self.ctrl.execute_job(
+                self.job, self.job_state, self.config, self.SESSION_DIR,
+                self.extcmd_popen)
+            # Ensure that call was invoked with command end environment (passed
+            # as keyword argument). Extract the return value of
+            # configured_filesystem() as nest_dir so that we can pass it to
+            # other calls to get their mocked return values.
+            # Urgh! is this doable somehow without all that?
+            nest_dir = self.ctrl.configured_filesystem().__enter__()
+            cwd_dir = self.ctrl.temporary_cwd().__enter__()
+            self.extcmd_popen.call.assert_called_with(
+                self.ctrl.get_execution_command(
+                    self.job, self.config, self.SESSION_DIR, nest_dir),
+                env=self.ctrl.get_execution_environment(
+                    self.job, self.config, self.SESSION_DIR, nest_dir),
+                cwd=cwd_dir,
+                pass_fds=["pipe0_w", "pipe1_r"])
+        # Ensure that execute_job() returns the return value of call()
+        self.assertEqual(retval, self.extcmd_popen.call())
+        # Ensure that presence of CHECKBOX_DATA directory was checked for
+        mock_os_path_isdir.assert_called_with(
+            self.ctrl.get_CHECKBOX_DATA(self.SESSION_DIR))
+        self.assertEqual(mock_os_pipe.call_count, 2)
+        self.assertEqual(mock_os_fdopen.call_count, 2)
+        self.assertEqual(mock_os_close.call_count, 6)
+        # Ensure that testing_shell_data is properly created
+        mock_json_dumps.assert_called_once_with({
+            "job_repr": {},
+            "session_dir": self.ctrl.get_CHECKBOX_DATA(self.SESSION_DIR)
+        })
+        mock_os_fdopen().write.assert_called_with(mock_json_dumps())
+
+    @mock.patch('os.path.isdir')
+    @mock.patch('os.fdopen')
+    @mock.patch('os.pipe')
+    @mock.patch('os.write')
+    @mock.patch('os.close')
+    def test_pipes_closed_when_cmd_raises(
+            self, mock_os_close, mock_os_write, mock_os_pipe, mock_os_fdopen,
+            mock_os_path_isdir):
+        """
+        Test if all pipes used by execute_job() are properly closed if
+        exception is raised during execution of command
+        """
+        mock_os_pipe.side_effect = [("pipe0_r", "pipe0_w"),
+                                    ("pipe1_r", "pipe1_w")]
+        with mock.patch.object(self.ctrl, 'get_execution_command'), \
+                mock.patch.object(self.ctrl, 'get_execution_environment'), \
+                mock.patch.object(self.ctrl, 'configured_filesystem'), \
+                mock.patch.object(self.ctrl, 'temporary_cwd'), \
+                mock.patch.object(self.ctrl, 'gen_job_repr', return_value={}), \
+                mock.patch.object(self.extcmd_popen, 'call',
+                                  side_effect=Exception('Boom')):
+            with self.assertRaises(Exception):
+                self.ctrl.execute_job(
+                    self.job, self.job_state, self.config, self.SESSION_DIR,
+                    self.extcmd_popen)
+        os.close.assert_any_call('pipe0_r')
+        os.close.assert_any_call('pipe1_r')
+        os.close.assert_any_call('pipe0_w')
+        os.close.assert_any_call('pipe1_w')
+
+    def test_get_checkbox_score_for_qml_job(self):
+        self.job.plugin = 'qml'
+        self.assertEqual(self.ctrl.get_checkbox_score(self.job), 4)
