@@ -24,6 +24,7 @@
 
 import logging
 import re
+import os
 
 from plainbox.abc import IJobDefinition
 from plainbox.i18n import gettext as _
@@ -48,6 +49,10 @@ from plainbox.impl.unit.validators import UntranslatableFieldValidator
 from plainbox.impl.unit.validators import UselessFieldValidator
 from plainbox.impl.validation import Problem
 from plainbox.impl.validation import Severity
+from plainbox.impl.xparsers import Error
+from plainbox.impl.xparsers import Text
+from plainbox.impl.xparsers import Visitor
+from plainbox.impl.xparsers import WordList
 
 __all__ = ['JobDefinition', 'propertywithsymbols']
 
@@ -103,6 +108,44 @@ class _PluginValues(SymbolDef):
     user_interact = "user-interact"
     user_interact_verify = "user-interact-verify"
     shell = 'shell'
+    qml = 'qml'
+
+
+class _CertificationStatusValues(SymbolDef):
+    """
+    Symbols for each value of the JobDefinition.certification_status field
+
+    Particular values have the following meanings.
+
+    unspecified:
+        One of the new possible certification status values. This value means
+        that a job was not analyzed in the context of certification status
+        classification and it has no classification at this time. This is also
+        the implicit certification status for all jobs.
+    not-part-of-certification:
+        One of the new possible certification status values. This value means
+        that a given job may fail and this will not affect the certification
+        process in any way. Typically jobs with this certification status are
+        not executed during the certification process. In the past this was
+        informally referred to as a *blacklist item*.
+    non-blocker:
+        One of the new possible certification status values. This value means
+        that a given job may fail and while that should be regarded as a
+        possible future problem it will not block the certification process. In
+        the past this was informally referred to as a *graylist item*.
+        Canonical reserves the right to promote jobs from the *non-blocker* to
+        *blocker*.
+    blocker:
+        One of the new possible certification status values. This value means
+        that a given job must pass for the certification process to succeed. In
+        the past this was informally referred to as a *whitelist item*. The
+        term *blocker* was chosen to disambiguate the meaning of the two
+        concepts.
+    """
+    unspecified = 'unspecified'
+    not_part_of_certification = 'not-part-of-certification'
+    non_blocker = 'non-blocker'
+    blocker = 'blocker'
 
 
 class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
@@ -303,6 +346,37 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
             self.get_record_value(
                 'category_id', '2013.com.canonical.plainbox::uncategorised'))
 
+    @property
+    def qml_file(self):
+        """
+        path to a QML file that implements tests UI for this job
+
+        This property exposes a path to QML file that follows the Plainbox QML
+        Test Specification. The file will be loaded either in the native test
+        shell of the application using plainbox or with a helper, generic
+        loader for all command-line applications.
+
+        To use this property, the plugin type should be set to 'qml'.
+        """
+        qml_file = self.get_record_value('qml_file')
+        if qml_file is not None and self.provider is not None:
+            return os.path.join(self.provider.data_dir, qml_file)
+
+    @propertywithsymbols(symbols=_CertificationStatusValues)
+    def certification_status(self):
+        """
+        Get the natural certification status of this job.
+
+        The default certification status of all jobs is
+        ``CertificationStatus.unspecified``
+
+        .. note::
+            Remember that the certification status can be overridden by a test
+            plan.  You should, instead, consider the effective certification
+            status that can be obtained from :class:`JobState`.
+        """
+        return self.get_record_value('certification-status', 'unspecified')
+
     @property
     def estimated_duration(self):
         """
@@ -431,27 +505,6 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
         return self.plugin in ['manual', 'user-interact',
                                'user-interact-verify']
 
-    @property
-    def via(self):
-        """
-        The checksum of the "parent" job when the current JobDefinition comes
-        from a job output using the local plugin
-        """
-        if hasattr(self.origin.source, 'job'):
-            return self.origin.source.job.checksum
-
-    def update_origin(self, origin):
-        """
-        Change the Origin object associated with this JobDefinition
-
-        .. note::
-
-            This method is a unfortunate side effect of how via and local jobs
-            that cat existing jobs are implemented. Ideally jobs would be
-            trully immutable. Do not use this method lightly.
-        """
-        self._origin = origin
-
     def get_resource_program(self):
         """
         Return a ResourceProgram based on the 'requires' expression.
@@ -470,7 +523,7 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
             else:
                 implicit_namespace = None
             if self.imports is not None:
-                imports = self.get_imported_jobs()
+                imports = list(self.get_imported_jobs())
             else:
                 imports = None
             self._resource_program = ResourceProgram(
@@ -484,13 +537,20 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
         To combat a simple mistake where the jobs are space-delimited any
         mixture of white-space (including newlines) and commas are allowed.
         """
-        if self.depends:
-            return {
-                self.qualify_id(maybe_partial_id)
-                for maybe_partial_id in re.split('[\s,]+', self.depends)
-            }
-        else:
-            return set()
+        deps = set()
+        if self.depends is None:
+            return deps
+
+        class V(Visitor):
+
+            def visit_Text_node(visitor, node: Text):
+                deps.add(self.qualify_id(node.text))
+
+            def visit_Error_node(visitor, node: Error):
+                logger.warning(_("unable to parse depends: %s"), node.msg)
+
+        V().visit(WordList.parse(self.depends))
+        return deps
 
     def get_resource_dependencies(self):
         """
@@ -568,6 +628,8 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
             purpose = 'purpose'
             steps = 'steps'
             verification = 'verification'
+            qml_file = 'qml_file'
+            certification_status = 'certification_status'
 
         field_validators = {
             fields.name: [
@@ -616,15 +678,14 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
             ],
             fields.command: [
                 UntranslatableFieldValidator,
-                TemplateVariantFieldValidator,
                 # All jobs except for manual must have a command
                 PresentFieldValidator(
                     message=_("command is mandatory for non-manual jobs"),
-                    onlyif=lambda unit: unit.plugin != 'manual'),
+                    onlyif=lambda unit: unit.plugin not in ('manual', 'qml')),
                 # Manual jobs cannot have a command
                 UselessFieldValidator(
-                    message=_("command on a manual job makes no sense"),
-                    onlyif=lambda unit: unit.plugin == 'manual'),
+                    message=_("command on a manual or qml job makes no sense"),
+                    onlyif=lambda unit: unit.plugin in ('manual', 'qml')),
                 # We don't want to refer to CHECKBOX_SHARE anymore
                 CorrectFieldValueValidator(
                     lambda command: "CHECKBOX_SHARE" not in command,
@@ -813,5 +874,33 @@ class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
                         ' non-C locale then set the preserve-locale flag'
                     ),
                     onlyif=lambda unit: unit.command),
-            ]
+            ],
+            fields.qml_file: [
+                UntranslatableFieldValidator,
+                TemplateInvariantFieldValidator,
+                PresentFieldValidator(
+                    onlyif=lambda unit: unit.plugin == 'qml'),
+                CorrectFieldValueValidator(
+                    lambda value: value.endswith('.qml'),
+                    Problem.wrong, Severity.advice,
+                    message=_('use the .qml extension for all QML files'),
+                    onlyif=lambda unit: (unit.plugin == 'qml'
+                                         and unit.qml_file)),
+                CorrectFieldValueValidator(
+                    lambda value, unit: os.path.isfile(unit.qml_file),
+                    message=_('please point to an existing QML file'),
+                    onlyif=lambda unit: (unit.plugin == 'qml'
+                                         and unit.qml_file)),
+            ],
+            fields.certification_status: [
+                UntranslatableFieldValidator,
+                TemplateInvariantFieldValidator,
+                CorrectFieldValueValidator(
+                    lambda certification_status: (
+                        certification_status in
+                        _CertificationStatusValues.get_all_symbols()),
+                    message=_('valid values are: {}').format(
+                        ', '.join(str(sym) for sym in sorted(
+                            _CertificationStatusValues.get_all_symbols())))),
+            ],
         }
