@@ -27,11 +27,12 @@ Test definitions for plainbox.impl.unit.job module
 from unittest import TestCase
 import warnings
 
+from plainbox.impl.providers.v1 import Provider1
 from plainbox.impl.secure.origin import FileTextSource
-from plainbox.impl.secure.origin import JobOutputTextSource
 from plainbox.impl.secure.origin import Origin
 from plainbox.impl.secure.rfc822 import RFC822Record
 from plainbox.impl.unit.job import JobDefinition
+from plainbox.impl.unit.job import propertywithsymbols
 from plainbox.impl.unit.test_unit_with_id import UnitWithIdFieldValidationTests
 from plainbox.impl.unit.unit_with_id import UnitWithId
 from plainbox.impl.unit.validators import UnitValidationContext
@@ -40,7 +41,6 @@ from plainbox.impl.validation import Severity
 from plainbox.impl.validation import ValidationError
 from plainbox.testing_utils.testcases import TestCaseWithParameters
 from plainbox.vendor import mock
-from plainbox.impl.unit.job import propertywithsymbols
 
 
 class DecoratorTests(TestCase):
@@ -126,6 +126,32 @@ class TestJobDefinitionDefinition(TestCase):
         self.assertEqual(job.flags, "flags-value")
         self.assertEqual(job.category_id, "category_id-value")
 
+    def test_qml_file_property_none_when_missing_provider(self):
+        """
+        Ensure that qml_file property is set to None when provider is not set.
+        """
+        job = JobDefinition({
+            'qml_file': 'qml_file-value'
+        }, raw_data={
+            'qml_file': 'qml_file-raw'
+        })
+        self.assertEqual(job.qml_file, None)
+
+    def test_qml_file_property(self):
+        """
+        Ensure that qml_file property is properly constructed
+        """
+        mock_provider = mock.Mock()
+        type(mock_provider).data_dir = mock.PropertyMock(return_value='data')
+        job = JobDefinition({
+            'qml_file': 'qml_file-value'
+        }, raw_data={
+            'qml_file': 'qml_file-raw'
+        }, provider=mock_provider)
+        with mock.patch('os.path.join', return_value='path') as mock_join:
+            self.assertEqual(job.qml_file, 'path')
+            mock_join.assert_called_with('data', 'qml_file-value')
+
     def test_properties_default_values(self):
         """
         Ensure that all properties default to None
@@ -137,7 +163,9 @@ class TestJobDefinitionDefinition(TestCase):
         self.assertEqual(job.user, None)
         self.assertEqual(job.shell, 'bash')
         self.assertEqual(job.flags, None)
-        self.assertEqual(job.category_id, '2013.com.canonical.plainbox::uncategorised')
+        self.assertEqual(job.category_id,
+                         '2013.com.canonical.plainbox::uncategorised')
+        self.assertEqual(job.qml_file, None)
 
     def test_checksum_smoke(self):
         job1 = JobDefinition({'plugin': 'plugin', 'user': 'root'})
@@ -291,7 +319,7 @@ class JobDefinitionFieldValidationTests(UnitWithIdFieldValidationTests):
             'plugin': 'foo'
         }, provider=self.provider).check()
         message = ("field 'plugin', valid values are: attachment, local,"
-                   " manual, resource, shell, user-interact,"
+                   " manual, qml, resource, shell, user-interact,"
                    " user-interact-verify, user-verify")
         self.assertIssueFound(issue_list, self.unit_cls.Meta.fields.plugin,
                               Problem.wrong, Severity.error, message)
@@ -320,16 +348,9 @@ class JobDefinitionFieldValidationTests(UnitWithIdFieldValidationTests):
         self.assertIssueFound(issue_list, self.unit_cls.Meta.fields.command,
                               Problem.unexpected_i18n, Severity.warning)
 
-    def test_command__template_variant(self):
-        issue_list = self.unit_cls({
-            'command': 'command'
-        }, parameters={}, provider=self.provider).check()
-        self.assertIssueFound(issue_list, self.unit_cls.Meta.fields.command,
-                              Problem.constant, Severity.error)
-
     def test_command__present__on_non_manual(self):
         for plugin in self.unit_cls.plugin.symbols.get_all_symbols():
-            if plugin == 'manual':
+            if plugin in ('manual', 'qml'):
                 continue
             # TODO: switch to subTest() once we depend on python3.4
             issue_list = self.unit_cls({
@@ -348,6 +369,15 @@ class JobDefinitionFieldValidationTests(UnitWithIdFieldValidationTests):
             issue_list, self.unit_cls.Meta.fields.command,
             Problem.useless, Severity.warning)
 
+    def test_command__useless__on_qml(self):
+        issue_list = self.unit_cls({
+            'plugin': 'qml',
+            'command': 'command'
+        }, provider=self.provider).check()
+        self.assertIssueFound(
+            issue_list, self.unit_cls.Meta.fields.command,
+            Problem.useless, Severity.warning)
+
     def test_command__not_using_CHECKBOX_SHARE(self):
         issue_list = self.unit_cls({
             'command': '$CHECKBOX_SHARE'
@@ -410,8 +440,8 @@ class JobDefinitionFieldValidationTests(UnitWithIdFieldValidationTests):
             if plugin == 'manual':
                 continue
             message = ("field 'description', all jobs should have a"
-            " description field, or a set of purpose, steps and verification"
-            " fields")
+                       " description field, or a set of purpose, steps and"
+                       " verification fields")
             # TODO: switch to subTest() once we depend on python3.4
             issue_list = self.unit_cls({
                 'plugin': plugin
@@ -526,7 +556,7 @@ class JobDefinitionFieldValidationTests(UnitWithIdFieldValidationTests):
             'depends': 'some-unit'
         }, provider=self.provider)
         message = "field 'depends', unit 'ns::some-unit' is not available"
-        self.provider.get_units.return_value = ([unit], ())
+        self.provider.unit_list = [unit]
         context = UnitValidationContext([self.provider])
         issue_list = unit.check(context=context)
         self.assertIssueFound(
@@ -541,7 +571,7 @@ class JobDefinitionFieldValidationTests(UnitWithIdFieldValidationTests):
             'depends': 'some-unit'
         }, provider=self.provider)
         message = "field 'depends', the referenced unit is not a job"
-        self.provider.get_units.return_value = ([unit, other_unit], ())
+        self.provider.unit_list = [unit, other_unit]
         context = UnitValidationContext([self.provider])
         issue_list = unit.check(context=context)
         self.assertIssueFound(
@@ -561,7 +591,7 @@ class JobDefinitionFieldValidationTests(UnitWithIdFieldValidationTests):
             'requires': 'some_unit.attr == "value"'
         }, provider=self.provider)
         message = "field 'requires', unit 'ns::some_unit' is not available"
-        self.provider.get_units.return_value = ([unit], ())
+        self.provider.unit_list = [unit]
         context = UnitValidationContext([self.provider])
         issue_list = unit.check(context=context)
         self.assertIssueFound(
@@ -576,7 +606,7 @@ class JobDefinitionFieldValidationTests(UnitWithIdFieldValidationTests):
             'requires': 'some_unit.attr == "value"'
         }, provider=self.provider)
         message = "field 'requires', the referenced unit is not a job"
-        self.provider.get_units.return_value = ([unit, other_unit], ())
+        self.provider.unit_list = [unit, other_unit]
         context = UnitValidationContext([self.provider])
         issue_list = unit.check(context=context)
         self.assertIssueFound(
@@ -591,7 +621,7 @@ class JobDefinitionFieldValidationTests(UnitWithIdFieldValidationTests):
             'requires': 'some_unit.attr == "value"'
         }, provider=self.provider)
         message = "field 'requires', the referenced job is not a resource job"
-        self.provider.get_units.return_value = ([unit, other_unit], ())
+        self.provider.unit_list = [unit, other_unit]
         context = UnitValidationContext([self.provider])
         issue_list = unit.check(context=context)
         self.assertIssueFound(
@@ -639,7 +669,7 @@ class JobDefinitionFieldValidationTests(UnitWithIdFieldValidationTests):
             'category_id': 'some-unit'
         }, provider=self.provider)
         message = "field 'category_id', unit 'ns::some-unit' is not available"
-        self.provider.get_units.return_value = ([unit], ())
+        self.provider.unit_list = [unit]
         context = UnitValidationContext([self.provider])
         issue_list = unit.check(context=context)
         self.assertIssueFound(
@@ -654,7 +684,7 @@ class JobDefinitionFieldValidationTests(UnitWithIdFieldValidationTests):
             'category_id': 'some-unit'
         }, provider=self.provider)
         message = "field 'category_id', the referenced unit is not a category"
-        self.provider.get_units.return_value = ([unit, other_unit], ())
+        self.provider.unit_list = [unit, other_unit]
         context = UnitValidationContext([self.provider])
         issue_list = unit.check(context=context)
         self.assertIssueFound(
@@ -999,6 +1029,15 @@ class TestJobDefinition(TestCase):
         observed = job.get_environ_settings()
         self.assertEqual(expected, observed)
 
+    def test_dependency_parsing_quoted_word(self):
+        job = JobDefinition({
+            'id': 'id',
+            'plugin': 'plugin',
+            'depends': '"quoted word"'})
+        expected = set(['quoted word'])
+        observed = job.get_direct_dependencies()
+        self.assertEqual(expected, observed)
+
     def test_environ_parsing_single_word(self):
         job = JobDefinition({
             'id': 'id',
@@ -1065,33 +1104,6 @@ class TestJobDefinition(TestCase):
             job1.checksum,
             "cd21b33e6a2f4d1291977b60d922bbd276775adce73fca8c69b4821c96d7314a")
 
-    def test_via_does_not_change_checksum(self):
-        """
-        verify that the 'via' attribute in no way influences job checksum
-        """
-        # Create a 'parent' job
-        parent = JobDefinition({'id': 'parent', 'plugin': 'local'})
-        # Create a 'child' job, using create_child_job_from_record() should
-        # time the two so that child.via should be parent.checksum.
-        #
-        # The elaborate record that gets passed has all the meta-data that
-        # traces back to the 'parent' job (as well as some imaginary line_start
-        # and line_end values for the purpose of the test).
-        child = parent.create_child_job_from_record(
-            RFC822Record(
-                data={'id': 'test', 'plugin': 'shell'},
-                origin=Origin(
-                    source=JobOutputTextSource(parent),
-                    line_start=1,
-                    line_end=1)))
-        # Now 'child.via' should be the same as 'parent.checksum'
-        self.assertEqual(child.via, parent.checksum)
-        # Create an unrelated job 'helper' with the definition identical as
-        # 'child' but without any ties to the 'parent' job
-        helper = JobDefinition({'id': 'test', 'plugin': 'shell'})
-        # And again, child.checksum should be the same as helper.checksum
-        self.assertEqual(child.checksum, helper.checksum)
-
     def test_estimated_duration(self):
         job1 = JobDefinition({})
         self.assertEqual(job1.estimated_duration, None)
@@ -1327,3 +1339,23 @@ class JobParsingTests(TestCaseWithParameters):
         expected = set({'foo', 'bar', 'froz'})
         observed = job.get_direct_dependencies()
         self.assertEqual(expected, observed)
+
+
+class RegressionTests(TestCase):
+
+    """ Regression tests. """
+
+    def test_1444242(self):
+        """ Regression test for http://pad.lv/1444242/. """
+        provider = mock.Mock(spec_set=Provider1, name='provider')
+        provider.namespace = '2013.com.canonical.certification'
+        job = JobDefinition({
+            'id': 'audio/playback_thunderbolt',
+            'imports': 'from 2013.com.canonical.plainbox import manifest',
+            'requires': (
+                "device.category == 'AUDIO'\n"
+                "manifest.has_thunderbolt == 'True'\n"),
+        }, provider=provider)
+        prog = job.get_resource_program()
+        self.assertEqual(prog.expression_list[-1].resource_id,
+                         '2013.com.canonical.plainbox::manifest')
