@@ -46,6 +46,7 @@ from plainbox.impl.commands import ToolBase, CommandBase
 from plainbox.impl.job import JobDefinition
 from plainbox.impl.logging import setup_logging
 from plainbox.impl.providers.special import get_categories
+from plainbox.impl.providers.special import get_manifest
 from plainbox.impl.providers.v1 import InsecureProvider1PlugInCollection
 from plainbox.impl.providers.v1 import get_user_PROVIDERPATH_entry
 from plainbox.impl.secure.config import Unset
@@ -56,6 +57,8 @@ from plainbox.impl.secure.origin import UnknownTextSource
 from plainbox.impl.secure.providers.v1 import Provider1
 from plainbox.impl.secure.providers.v1 import Provider1Definition
 from plainbox.impl.secure.rfc822 import RFC822SyntaxError
+from plainbox.impl.unit.packaging import PackagingDriverError
+from plainbox.impl.unit.packaging import get_packaging_driver
 from plainbox.impl.unit.unit_with_id import UnitWithId
 from plainbox.impl.unit.validators import UnitValidationContext
 from plainbox.impl.validation import Issue
@@ -284,7 +287,7 @@ class InstallCommand(ManageCommand):
                 layout, prefix, provider).write(stream))
 
     def _copy_all_executables(self, root, prefix, layout, provider):
-        executable_list = provider.get_all_executables()
+        executable_list = provider.executable_list
         if not executable_list:
             return
         dest_map = self._get_dest_map(layout, prefix)
@@ -352,7 +355,7 @@ class InstallCommand(ManageCommand):
             # For bin_dir do the exception that any executables (also
             # counting those from build/bin) should trigger bin_dir to be
             # listed since we will create/copy files there anyway.
-            if provider.get_all_executables() != []:
+            if provider.executable_list != []:
                 config_obj.set(section, 'bin_dir', dest_map['bin'])
         return config_obj
 
@@ -402,7 +405,7 @@ class SourceDistributionCommand(ManageCommand):
     name = "sdist"
 
     _INCLUDED_ITEMS = ['manage.py', 'README.md', 'units', 'jobs', 'whitelists',
-                       'bin', 'src', 'data', 'po']
+                       'COPYING', 'bin', 'src', 'data', 'po']
 
     def register_parser(self, subparsers):
         """
@@ -831,13 +834,19 @@ class InfoCommand(ManageCommand):
         print("\t" + _("version: {}").format(provider.version))
         # TRANSLATORS: {} is the gettext translation domain of the provider
         print("\t" + _("gettext domain: {}").format(provider.gettext_domain))
-        unit_list, problem_list = provider.get_units()
+        unit_list, problem_list = provider.unit_list, provider.problem_list
         print(_("[Job Definitions]"))
         self._display_units((
             unit for unit in unit_list if unit.Meta.name == 'job'))
         print(_("[Test Plans]"))
         self._display_units((
             unit for unit in unit_list if unit.Meta.name == 'test plan'))
+        print(_("[Test Plans] (legacy)"))
+        # TRANSLATORS: {} is the name of the test provider
+        for whitelist in provider.whitelist_list:
+            print("\t" + _("{0} from {1}").format(
+                whitelist.name,
+                whitelist.origin.relative_to(self.definition.location)))
         print(_("[Other Units]"))
         self._display_units((
             unit for unit in unit_list
@@ -847,8 +856,7 @@ class InfoCommand(ManageCommand):
             # TRANSLATORS: please don't translate `manage.py validate`
             print("\t" + _("Please run `manage.py validate` for details"))
         print(_("[Executables]"))
-        executable_list = provider.get_all_executables()
-        for executable in executable_list:
+        for executable in provider.executable_list:
             print("\t{0!a}".format(os.path.basename(executable)))
 
     def _display_units(self, unit_list):
@@ -925,7 +933,10 @@ class ValidateCommand(ManageCommand):
         the command class with the ``command`` default.
         """
         parser = self.add_subcommand(subparsers)
-        parser.set_defaults(deprecated=self.deprecated, strict=self.strict)
+        parser.set_defaults(
+            new_validation_core=True,
+            deprecated=self.deprecated,
+            strict=self.strict)
         group = parser.add_argument_group(title=_("validation options"))
         group.add_argument(
             '-s', '--strict', action='store_true',
@@ -941,7 +952,7 @@ class ValidateCommand(ManageCommand):
             help=_("Support deprecated syntax and features"))
         group.add_argument(
             '-N', '--new-validation-core', action='store_true',
-            help=_("Use the new validation subsystem"))
+            help=argparse.SUPPRESS)
 
     def invoked(self, ns):
         if ns.new_validation_core:
@@ -998,7 +1009,7 @@ class ValidateCommand(ManageCommand):
             print(_("The provider seems to be valid"))
 
     def get_unit_list(self, provider):
-        unit_list, problem_list = provider.get_units()
+        unit_list, problem_list = provider.unit_list, provider.problem_list
         if problem_list:
             for exc in problem_list:
                 if isinstance(exc, RFC822SyntaxError):
@@ -1030,10 +1041,16 @@ class ValidateCommand(ManageCommand):
         provider_list = all_providers.get_all_plugin_objects()
         if all(p.name != provider.name for p in provider_list):
             provider_list.append(provider)
-        # Add the built-in 'categories' provider, unless that's the one we're testing
+        # Add the built-in 'categories' provider,
+        # unless that's the one we're testing
         categories_provider = get_categories()
         if provider.base_dir != categories_provider.base_dir:
             provider_list.append(categories_provider)
+        # Add the built-in 'manifest' provider,
+        # unless that's the one we're testing
+        manifest_provider = get_manifest()
+        if provider.base_dir != manifest_provider.base_dir:
+            provider_list.append(manifest_provider)
         _logger.info(_("Validating everything..."))
         unit_list, exc_list = self.collect_all_units(provider)
         early_issue_gen = self.get_early_issues(exc_list)
@@ -1065,7 +1082,7 @@ class ValidateCommand(ManageCommand):
                 hidden
             ).format(hidden))
             print(_(
-                "Run 'manage.py validate -N --strict --deprecated' for details"
+                "Run 'manage.py validate --strict --deprecated' for details"
             ))
         if failed:
             print(_(
@@ -1076,8 +1093,7 @@ class ValidateCommand(ManageCommand):
             print(_("The provider seems to be valid"))
 
     def collect_all_units(self, provider):
-        unit_list, exc_list = provider.get_units()
-        return unit_list, exc_list
+        return provider.unit_list, provider.problem_list
 
     def get_early_issues(self, exc_list):
         # NOTE: exc_list is a list of arbitrary exceptions
@@ -1166,7 +1182,7 @@ class I18NCommand(ManageCommand):
             help=_("don't actually do anything"))
         group = parser.add_argument_group(title=_("actions to perform"))
         group.add_argument(
-            "--dont-update-pot", default=False, action="store_false",
+            "--dont-update-pot", default=False, action="store_true",
             help=_("do not update the translation template"))
         group.add_argument(
             "--dont-merge-po", default=False, action="store_true",
@@ -1246,6 +1262,40 @@ class I18NCommand(ManageCommand):
             return 0
 
 
+@docstring(
+    # TRANSLATORS: please leave various options (both long and short forms),
+    # environment variables and paths in their original form. Also keep the
+    # special @EPILOG@ string. The first line of the translation is special and
+    # is used as the help message. Please keep the pseudo-statement form and
+    # don't finish the sentence with a dot. Pay extra attention to whitespace.
+    # It must be correctly preserved or the result won't work. In particular
+    # the leading whitespace *must* be preserved and *must* have the same
+    # length on each line.
+    N_("""
+    generate packaging meta-data
+
+    This command should not be invoked manually. It is applicable for package
+    maintainers to allow them to extract packaging meta-data applicable for
+    the current operating system from within the provider itself.
+    """))
+class PackagingCommand(ManageCommand):
+
+    def register_parser(self, subparsers):
+        self.add_subcommand(subparsers)
+
+    def invoked(self, ns):
+        driver = get_packaging_driver()
+        try:
+            driver.inspect_packaging()
+        except PackagingDriverError as exc:
+            print(str(exc))
+            return 1
+        else:
+            provider = self.get_provider()
+            driver.inspect_provider(provider)
+            driver.modify_packaging_tree()
+
+
 class ProviderManagerTool(ToolBase):
     """
     Command line tool that is covertly used by each provider's manage.py script
@@ -1296,6 +1346,7 @@ class ProviderManagerTool(ToolBase):
         I18NCommand,
         BuildCommand,
         CleanCommand,
+        PackagingCommand,
     ]
 
     # XXX: keywords=None is for anyone who has copied the example go provider
